@@ -60,11 +60,20 @@ frame. The second is the offset in the trajectory archive where it begins.
 		
 		NSWarnLog(@"Only %d bytes in archive",
 			  [trajectoryData length]);
-		NSWarnLog(@"This indicates an error occured during the simulation which generated this data");
-		NSWarnLog(@"Check the simulation logs for more information on the cause of the error");
+		NSWarnLog(@"Either an error occured during the simulation which generated this data or no coordinate data was collected");
+		//If there was some corrupted data in the file delete it
+		//trajectoryHandle is always created before this method is called.
+		if([trajectoryData length] != 0)
+			[trajectoryHandle truncateFileAtOffset: 0];
+			
 		numberTopologyCheckpoints = 0;
 		[pool release];
-		return nil;
+		
+		//Return the empty informationMatrix.
+		[NSKeyedArchiver archiveRootObject: informationMatrix 
+			toFile: trajectoryInfoPath];	
+					
+		return [informationMatrix autorelease];
 	}	
 	
 	for(i = 0; i< (int)[trajectoryData length] - 1; i++)
@@ -103,8 +112,6 @@ frame. The second is the offset in the trajectory archive where it begins.
 	else
 		NSWarnLog(@"The last configuration frame is not complete.");
 
-	[informationMatrix autorelease];
-	informationMatrix = [informationMatrix copy];
 	[NSKeyedArchiver archiveRootObject: informationMatrix toFile: trajectoryInfoPath];
 	[row release];
 	[pool release];
@@ -114,6 +121,61 @@ frame. The second is the offset in the trajectory archive where it begins.
 
 @end
 
+/**
+Checks the storage at path.
+Returns True if it exists and is a valid storage.
+Otherwise returns flase an \e error contains an error describing the problem
+*/
+BOOL AdCheckStorage(NSString* storagePath, NSError** error);
+
+BOOL AdCheckStorage(NSString* storagePath, NSError** error)
+{
+	BOOL isDirectory;
+	NSError* accessError = nil;
+	NSString* systemPath;
+	NSFileManager* fileManager;
+	
+	fileManager = [NSFileManager defaultManager];
+	systemPath = [storagePath stringByAppendingPathComponent: @"system.ad"];
+	
+	if(![fileManager fileExistsAtPath: storagePath isDirectory: &isDirectory])
+		accessError = AdCreateError(AdunCoreErrorDomain,
+					    AdCoreSimulationDataStorageError,
+					    @"Unable to access data store",
+					    [NSString stringWithFormat: 
+					     @"Data storage directory %@ does not exist",
+					     storagePath],
+					    @"Check the provided path is correct");
+	
+	//check its a results dir
+	if(!isDirectory)
+		accessError = AdCreateError(AdunCoreErrorDomain,
+					    AdCoreSimulationDataStorageError,
+					    @"Unable to access data store",
+					    [NSString stringWithFormat: 
+					     @"Specified storage %@ is not a directory",
+					     storagePath],
+					    @"Check the provided path is correct");
+	
+	//check it contains the required files
+	if(![fileManager isReadableFileAtPath: systemPath])
+		accessError = AdCreateError(AdunCoreErrorDomain,
+					    AdCoreSimulationDataStorageError,
+					    @"Unable to access data store",
+					    [NSString stringWithFormat: 
+					     @"Simulation data at %@ is not readable or not present",
+					     storagePath],
+					    @"Check the provided path is correct");
+	
+	if(accessError != nil)				
+	{
+		*error = accessError;				
+		return NO;
+	}
+	else
+		 return YES;	
+}
+
 @implementation AdFileSystemSimulationStorage
 
 + (void) initialize
@@ -174,6 +236,12 @@ frame. The second is the offset in the trajectory archive where it begins.
 	else
 		trajectoryInfo = [NSKeyedUnarchiver unarchiveObjectWithFile: trajectoryInfoPath];
 	
+	//In pre 0.13.1 versions of the framework trajectoryInfo was sometimes archived as an
+	//immutable matrix. However in order to implement AdSimulationStorageAppendMode
+	//and AdSimulationStorageUpdateMode it must be always be mutable.
+	if(![trajectoryInfo isKindOfClass: [AdMutableDataMatrix class]])
+		trajectoryInfo = [[trajectoryInfo mutableCopy] autorelease];
+	
 	[trajectoryInfo retain];
 	numberTrajectoryCheckpoints = [trajectoryInfo numberOfRows];
 }
@@ -222,43 +290,30 @@ frame. The second is the offset in the trajectory archive where it begins.
 		accessError = nil;
 	}	
 	
-	if(![fileManager fileExistsAtPath: storagePath isDirectory: &isDirectory])
-		accessError = AdCreateError(AdunCoreErrorDomain,
-				AdCoreSimulationDataStorageError,
-				@"Unable to access data store",
-				[NSString stringWithFormat: 
-					@"Data storage directory %@ does not exist",
-					storagePath],
-				@"Check the provided path is correct");
+	isAccessible = AdCheckStorage(storagePath, &accessError);
 	
-	//check its a results dir
+	if(accessError != nil)
+		[accessError retain];
+}
+
++ (BOOL) storageExistsAtLocation: (NSString*) location
+{
+	return AdCheckStorage(location, NULL);
+}
+
++ (id) storageForSimulation: (AdSimulationData*) simulation
+		inDirectory: (NSString*) directory
+		       mode: (AdSimulationStorageMode) mode
+		      error: (NSError**) error
+{		      
+	AdSimulationData* data;
 	
-	if(!isDirectory)
-		accessError = AdCreateError(AdunCoreErrorDomain,
-				AdCoreSimulationDataStorageError,
-				@"Unable to access data store",
-				[NSString stringWithFormat: 
-					@"Specified storage %@ is not a directory",
-					storagePath],
-				@"Check the provided path is correct");
-
-	//check it contains the required files
-	if(![fileManager isReadableFileAtPath: systemPath])
-		accessError = AdCreateError(AdunCoreErrorDomain,
-				AdCoreSimulationDataStorageError,
-				@"Unable to access data store",
-				[NSString stringWithFormat: 
-					@"Simulation data at %@ is not readable or not present",
-					storagePath],
-				@"Check the provided path is correct");
-
-	if(accessError != nil)				
-	{
-		[accessError retain];					
-		isAccessible = NO;
-	}
-	else
-		isAccessible = YES;
+	data = [[AdFileSystemSimulationStorage alloc]
+		initStorageForSimulation: simulation
+		inDirectory: directory
+		mode: mode
+		error: error];
+	return [data autorelease];	
 }
 
 - (id) init
@@ -281,6 +336,21 @@ frame. The second is the offset in the trajectory archive where it begins.
 			error: NULL];
 }
 
+- (id) initStorageForSimulation: (AdSimulationData*) simulation
+		    inDirectory: (NSString*) directory
+			   mode: (AdSimulationStorageMode) mode
+			  error: (NSError**) error
+{			  
+	NSString* path;
+	
+	path = [[simulation identification] stringByAppendingString: @"_Data"];
+	if(directory != nil)
+		path = [directory stringByAppendingPathComponent: path];
+		
+	return [self initSimulationStorageAtPath: path
+		mode: mode
+		error: error];
+}
 - (id) initSimulationStorageAtPath: (NSString*) path 
 	mode: (AdSimulationStorageMode) mode 
 	error: (NSError**) anError
@@ -291,9 +361,6 @@ frame. The second is the offset in the trajectory archive where it begins.
 	{
 		fileManager = [NSFileManager defaultManager];
 
-		if(anError != NULL)
-			*anError = accessError;
-
 		isAccessible = YES;
 		isTemporary = NO;
 
@@ -375,7 +442,17 @@ frame. The second is the offset in the trajectory archive where it begins.
 							columnDataTypes: [NSArray arrayWithObjects: @"int", @"int", nil]];
 				[NSKeyedArchiver archiveRootObject: trajectoryInfo 
 					toFile: trajectoryInfoPath];			
-			}	
+			}
+			else if(storageMode == AdSimulationStorageAppendMode)
+			{	
+				trajectoryHandle = [NSFileHandle fileHandleForUpdatingAtPath: trajectoryPath];
+				[trajectoryHandle retain];
+				[trajectoryHandle seekToEndOfFile];
+				energyHandle = [NSFileHandle fileHandleForUpdatingAtPath: energyPath ];
+				[energyHandle retain];
+				[self _loadTrajectoryInfo];
+				[self _countTopologyCheckpoints];
+			}
 			else
 			{
 				trajectoryHandle = [NSFileHandle fileHandleForReadingAtPath: trajectoryPath];
@@ -385,7 +462,10 @@ frame. The second is the offset in the trajectory archive where it begins.
 			}	
 		}	
 	}
-
+		
+	if(anError != NULL)
+		*anError = accessError;
+	
 	return self;
 }
 
@@ -463,7 +543,7 @@ frame. The second is the offset in the trajectory archive where it begins.
 {
 	NSString* path;
 
-	if(storageMode == AdSimulationStorageWriteMode)
+	if(storageMode & AdSimulationStorageWriteMode)
 		[NSException raise: NSInternalInconsistencyException
 			format: @"Data store in write only mode"];
 
@@ -476,7 +556,7 @@ frame. The second is the offset in the trajectory archive where it begins.
 
 - (NSData*) trajectoryCheckpoint: (int) number
 {
-	if(storageMode == AdSimulationStorageWriteMode)
+	if(storageMode & AdSimulationStorageWriteMode)
 		[NSException raise: NSInternalInconsistencyException
 			format: @"Data store in write only mode"];
 
@@ -487,6 +567,10 @@ frame. The second is the offset in the trajectory archive where it begins.
 {
 	NSString* path;
 
+	if(storageMode & AdSimulationStorageWriteMode)
+		[NSException raise: NSInternalInconsistencyException
+			    format: @"Data store in write only mode"];
+
 	if(number >= numberTopologyCheckpoints)
 		[NSException raise: NSRangeException
 			format: @"Topology - Index %d is out of range %d", 
@@ -499,7 +583,7 @@ frame. The second is the offset in the trajectory archive where it begins.
 
 - (NSData*) energyData
 {
-	if(storageMode == AdSimulationStorageWriteMode)
+	if(storageMode & AdSimulationStorageWriteMode)
 		[NSException raise: NSInternalInconsistencyException
 			format: @"Data store in write only mode"];
 	
@@ -514,7 +598,7 @@ frame. The second is the offset in the trajectory archive where it begins.
 	NSString* path;
 
 	NSDebugLLog(@"AdFileSystemSimulationStorage", @"Loading frame data");
-	if(storageMode == AdSimulationStorageWriteMode)
+	if((storageMode == AdSimulationStorageWriteMode) || (storageMode == AdSimulationStorageAppendMode))
 		[NSException raise: NSInternalInconsistencyException
 			format: @"Data store in write only mode"];
 
@@ -528,7 +612,7 @@ frame. The second is the offset in the trajectory archive where it begins.
 
 - (NSData*) systemData
 {
-	if(storageMode == AdSimulationStorageWriteMode)
+	if(storageMode & AdSimulationStorageWriteMode)
 		[NSException raise: NSInternalInconsistencyException
 			format: @"Data store in write only mode"];
 
@@ -656,11 +740,11 @@ Raises an exception if the mode is AdSimulationStorageReadMode
 */
 - (void) setEnergyData: (NSData*) data
 {
-	[energyHandle truncateFileAtOffset: 0];
 	if(storageMode == AdSimulationStorageReadMode)
 		[NSException raise: NSInternalInconsistencyException
 			format: @"Cannot write to a data store in AdSimulationStorageReadMode"];
 
+	[energyHandle truncateFileAtOffset: 0];
 	[energyHandle writeData: data];
 	[energyHandle synchronizeFile];
 }
@@ -752,13 +836,15 @@ Raises an exception if the mode is AdSimulationStorageReadMode
 {
 	[NSKeyedArchiver archiveRootObject: trajectoryInfo 
 		toFile: trajectoryInfoPath];
+	NSDebugLLog(@"AdFileSystemSimulationStorage", @"Synching trajectory");	
 	[trajectoryHandle synchronizeFile];
+	NSDebugLLog(@"AdFileSystemSimulationStorage", @"Synching energy");
 	[energyHandle synchronizeFile];
 }
 
 - (void) update
 {
-	if(storageMode == AdSimulationStorageWriteMode)
+	if((storageMode == AdSimulationStorageWriteMode) || (storageMode == AdSimulationStorageAppendMode))
 		return;
 	
 	[trajectoryInfo release];
