@@ -24,8 +24,76 @@
 
 static id ioManager;
 
+/**
+Contains methods for handling loading simulation data
+*/
+@interface AdIOManager (SimulationDataLoading)
+/**
+Loads data specified on the command line
+*/
+- (BOOL) _loadCommandLineData: (NSError**) error;
+/**
+Loads data from a previous simulation for a continuation run
+*/
+- (BOOL) _loadRestartData: (NSError**) error;
+/**
+Retreives data from the local server
+*/
+- (BOOL) _loadServerData: (NSError**) error;
+@end
+
 @implementation AdIOManager
 
+/**
+Check a directory sets an error if a file is in the way of the directory.
+It returns NO if the directory does not exist
+ */
+- (BOOL) _checkDirectory: (NSString*) directoryPath error: (NSError**) error
+{
+	BOOL isDir;
+	
+	if([fileManager fileExistsAtPath: directoryPath isDirectory: &isDir])
+	{
+		if(!isDir)
+		{
+			*error = AdCreateError(AdunCoreErrorDomain,
+					       AdCoreDirectoryStructureError,
+					       [NSString stringWithFormat: 
+						       @"A non-directory file exists at %@", directoryPath],
+					       @"The required directory cannot be created as the file is in the way.",
+					       @"Move or remove the file");
+			
+			return NO;
+		}
+		
+		return YES;
+	}
+	
+	return NO;
+}
+
+- (BOOL) _createDirectory: (NSString*) directoryPath error: (NSError**) error
+
+{
+	if(![fileManager createDirectoryAtPath: directoryPath
+				    attributes: nil])
+	{
+		
+		*error = AdCreateError(AdunCoreErrorDomain,
+				       AdCoreDirectoryStructureError,
+				       [NSString stringWithFormat: 
+					       @"Unable to create missing directory %@", directoryPath],
+				       @"This is probably because the containing directory is not writable",
+				       @"Change the permissions of the containing directory to allow file creation");
+		
+		return NO;
+	}
+	
+	NSWarnLog(@"Created missing directory %@", directoryPath);
+	
+	return YES;
+}
+
 /*
  * Connecting to and disconnecting from the local
  * AdServer instance
@@ -86,6 +154,24 @@ static id ioManager;
 
 - (void) closeConnection: (NSError*) error
 {
+	//If we're not connected just return
+	if(![self isConnected])
+		return;
+
+	/*
+	 * Before closing the connection check that the simulation information has been sent.
+	 * It will not have been sent if we are exiting due to an exception
+	 * and the exception was raised before setSimulationReferences: was called.
+	 */
+	
+	if(!simulationDataSent)
+	{
+		[serverProxy simulationData: simulationData
+				 forProcess: [[NSProcessInfo processInfo] processIdentifier]];
+		simulationDataSent = YES;		 
+	}
+		
+	
 	NSDebugLLog(@"Server", @"Server debug - Closing connection to server. Statistics are %@", 
 			[[serverProxy connectionForProxy] statistics]);
 	NSDebugLLog(@"Server", @"Server debug - Connection %@", [serverProxy connectionForProxy]);
@@ -157,7 +243,9 @@ static id ioManager;
 				forKey: @"Standard"];
 		[fileStreams setObject: [NSValue valueWithPointer: stderr] 
 				forKey: @"Error"];
-		simulationData = nil;		
+		
+		simulationData = nil;	
+		simulationDataSent = NO;	
 		outputDir = controllerOutputDir = nil;
 		adunDir = controllerDir = extensionDir = pluginDir = nil;
 		logFile = errorFile = nil;
@@ -165,10 +253,11 @@ static id ioManager;
 		externalObjects = nil;
 		adunInfo = [NSProcessInfo processInfo];
 		runMode = AdCoreUnknownRunMode;
-		processedArgs = NO;
+		processedArgs = restartRequested = NO;
 		validArgs = [[NSArray alloc] initWithObjects:
 				@"-RunMode",
 				@"-Template",
+				@"-Continue",
 				@"-SimulationOutputDir",
 				@"-ControllerOutputDir",
 				@"-ExternalObjects", 
@@ -181,7 +270,26 @@ static id ioManager;
 		[defaults setObject: [NSNumber numberWithBool: YES] forKey: @"CreateLogFiles"];
 		[defaults setObject: NSHomeDirectory() forKey: @"ProgramDirectoryLocation"];
 		[[NSUserDefaults standardUserDefaults] registerDefaults:defaults];
-		[[NSUserDefaults standardUserDefaults] synchronize];
+		
+		//Set the path ivars
+#if __FREEBSD__	
+		adunDir = [[[NSUserDefaults standardUserDefaults] 
+			    stringForKey: @"ProgramDirectoryLocation"]
+			   stringByAppendingPathComponent: @".adun"];
+#else			
+		adunDir = [[[NSUserDefaults standardUserDefaults] 
+			    stringForKey: @"ProgramDirectoryLocation"]
+			   stringByAppendingPathComponent: @"adun"];	
+#endif					
+		
+		pluginDir = [adunDir stringByAppendingPathComponent: @"Plugins"];
+		controllerDir = [pluginDir stringByAppendingPathComponent: @"Controllers"];
+		extensionDir = [pluginDir stringByAppendingPathComponent: @"Extensions"];
+		
+		[adunDir retain];
+		[pluginDir retain];
+		[controllerDir retain];
+		[extensionDir retain];
 	}
 
 	return self;
@@ -281,10 +389,17 @@ static id ioManager;
 	}	
 
 	/*
-	 * If the run mode is AdCoreCommandLineRunMode 
-	 * then a template must be supplied. 
+	 * Check if continuation of a previous simulation was requested.
 	 */
-	if(runMode == AdCoreCommandLineRunMode)
+	 
+	if((value = [userDefaults stringForKey: @"Continue"]) != nil)
+		restartRequested = YES;
+
+	/*
+	 * If the run mode is AdCoreCommandLineRunMode and we are not
+	 * continuing a previous simulation then a template must be supplied. 
+	 */
+	if((runMode == AdCoreCommandLineRunMode) && !restartRequested)
 	{
 		if((value = [userDefaults stringForKey: @"Template"]) == nil)
 		{
@@ -330,23 +445,23 @@ is created using the current directory.
 	return path;
 }
 
-- (BOOL) createLogFiles: (NSError**) error
+- (BOOL) _createLogFilesForNewRun: (NSError**) error
 {
 	NSUserDefaults* userDefaults = [NSUserDefaults standardUserDefaults];
-
+	
 	//Only create the log files if requested to do so.
 	if([userDefaults boolForKey: @"CreateLogFiles"] == NO)
 	{
 		GSPrintf(stdout, @"Log file creation supressed\n");
 		return YES;
 	}	
-
+	
 	logFile = [[NSUserDefaults standardUserDefaults] stringForKey: @"LogFile"];
 	logFile = [self _fixFilePath: logFile];
 	[logFile retain];
-
+	
 	if(![[NSFileManager defaultManager] isWritableFileAtPath:
-		 [logFile stringByDeletingLastPathComponent]])
+		[logFile stringByDeletingLastPathComponent]])
 	{
 		[logFile release];
 		logFile = [[userDefaults 
@@ -354,29 +469,29 @@ is created using the current directory.
 				valueForKey:@"LogFile"];
 		[logFile retain];		
 		NSWarnLog(@"Invalid value for user default 'LogFile' (%@). The specificed directory is not writable",
-			logFile);
+			  logFile);
 		NSWarnLog(@"Switching to registered default %@", logFile);
 		if(![[NSFileManager defaultManager] 
 			isWritableFileAtPath:
-			 [logFile stringByDeletingLastPathComponent]])
+			[logFile stringByDeletingLastPathComponent]])
 		{
 			*error = AdCreateError(AdunCoreErrorDomain,
-					AdCoreLogFileError,
-				 	[NSString stringWithFormat: 
-						@"Default log file (%@) not writable.", logFile],
-					@"This error may also indicate that a user supplied value for the LogFile default is invalid",
-					@"Check the write permissions on the adun/ directory.");
+					       AdCoreLogFileError,
+					       [NSString stringWithFormat: 
+						       @"Default log file (%@) not writable.", logFile],
+					       @"This error may also indicate that a user supplied value for the LogFile default is invalid",
+					       @"Check the write permissions on the adun/ directory.");
 			return NO;		
 		}
 	} 
 	GSPrintf(stdout, @"Log file is %@\n", logFile);
 	fflush(stdout);
-
+	
 	errorFile = [[NSUserDefaults standardUserDefaults] stringForKey: @"ErrorFile"];
 	errorFile = [self _fixFilePath: errorFile];
 	[errorFile retain];
 	if(![[NSFileManager defaultManager] isWritableFileAtPath:
-		 [errorFile stringByDeletingLastPathComponent]])
+		[errorFile stringByDeletingLastPathComponent]])
 	{
 		[errorFile release];
 		errorFile = [[userDefaults
@@ -384,107 +499,145 @@ is created using the current directory.
 				valueForKey:@"ErrorFile"];
 		[errorFile retain];		
 		NSWarnLog(@"Invalid value for user default 'ErrorFile' (%@). The specificed directory is not writable", 
-			errorFile);
+			  errorFile);
 		NSWarnLog(@"Switching to registered default %@", errorFile);
 		if(![[NSFileManager defaultManager] 
 			isWritableFileAtPath:
-			 [errorFile stringByDeletingLastPathComponent]])
+			[errorFile stringByDeletingLastPathComponent]])
 		{
 			*error = AdCreateError(AdunCoreErrorDomain,
-					AdCoreLogFileError,
-				 	@"Default error file (%@) not writable.",
-					@"This error may also indicate that a user supplied value for the ErrorFile default is invalid",
-					@"Check the write permissions on the adun/ directory.");
+					       AdCoreLogFileError,
+					       @"Default error file (%@) not writable.",
+					       @"This error may also indicate that a user supplied value for the ErrorFile default is invalid",
+					       @"Check the write permissions on the adun/ directory.");
 			return NO;		
 		}
 	} 
-
+	
 	GSPrintf(stdout, @"Error file is %@\n", errorFile);
 	fflush(stderr);
 	freopen([errorFile cString], "w", stderr);
 	freopen([logFile cString], "w", stdout);
-
-	return YES;
+	
+	return YES;	
 }
 
-/**
-Check directory sets an error if a file is in the way of the directory.
-It returns NO if the directory does not exist
-*/
-- (BOOL) _checkDirectory: (NSString*) directoryPath error: (NSError**) error
+//On a restart outputDir is assigned here.
+- (BOOL) _setupLogFilesForContinuationRun: (NSError**) error
 {
-	BOOL isDir;
-
-	if([fileManager fileExistsAtPath: directoryPath isDirectory: &isDir])
+	NSError* internalError = nil;
+	NSString* simulationDir;
+	
+	simulationDir = [[NSUserDefaults standardUserDefaults] stringForKey: @"Continue"];
+	simulationDir = [simulationDir stringByAppendingString: @"_Data"];
+	NSLog(@"Simulation dir is %@", simulationDir);
+	
+	//Check if the directory is present and that a file isnt in the way
+	if(![self _checkDirectory: simulationDir error: &internalError])
 	{
-		if(!isDir)
+		if(internalError == nil)
 		{
 			*error = AdCreateError(AdunCoreErrorDomain,
-				AdCoreDirectoryStructureError,
-				[NSString stringWithFormat: 
-				@"A non-directory file exists at %@", directoryPath],
-				@"The required directory cannot be created as the file is in the way.",
-				@"Move or remove the file");
-
-			return NO;
+					       AdCoreSimulationDataStorageError,
+					       @"Error accessing simulation data",
+					       [NSString stringWithFormat: 
+						       @"Specified data directory (%@) not present.", simulationDir],
+					       @"Check supplied simulation name is valid.");
 		}
-
-		return YES;
-	}
-
-	return NO;
-}
-
-- (BOOL) _createDirectory: (NSString*) directoryPath error: (NSError**) error
-
-{
-	if(![fileManager createDirectoryAtPath: directoryPath
-		 attributes: nil])
-	{
-
-		*error = AdCreateError(AdunCoreErrorDomain,
-				AdCoreDirectoryStructureError,
-				[NSString stringWithFormat: 
-					@"Unable to create missing directory %@", directoryPath],
-				@"This is probably because the containing directory is not writable",
-				@"Change the permissions of the containing directory to allow file creation");
-
+		else
+			*error = internalError;
+		
 		return NO;
 	}
 	
-	NSWarnLog(@"Created missing directory %@", directoryPath);
+	//Check the directory is writable
+	if(![fileManager isWritableFileAtPath: simulationDir])
+	{
+		*error = AdCreateError(AdunCoreErrorDomain,
+				       AdCoreSimulationDataStorageError,
+				       @"Error accessing simulation data",
+				       [NSString stringWithFormat: 
+					       @"Data directory (%@) not writable.", simulationDir],
+				       @"Check the write permissions of the data directory.");
+		return NO;		
+	}
 	
+	logFile = [simulationDir stringByAppendingPathComponent: @"AdunCore.log"];
+	errorFile = [simulationDir stringByAppendingPathComponent: @"AdunCore.errors"];
+	[logFile retain];
+	[errorFile retain];
+	
+	//Check the log files are writable (if they exist)
+	if([fileManager fileExistsAtPath: logFile])
+	{
+		if(![fileManager isWritableFileAtPath: logFile])
+		{
+			*error = AdCreateError(AdunCoreErrorDomain,
+			       AdCoreLogFileError,
+			       @"Error accessing log file",
+			       [NSString stringWithFormat: 
+				       @"Log file (%@) not writable.", logFile],
+			       @"Check the write permissions in the simulation data directory.");
+			return NO;		
+		}
+	} 
+
+	if([fileManager fileExistsAtPath: errorFile])
+	{
+		if(![fileManager isWritableFileAtPath: errorFile])
+		{
+			*error = AdCreateError(AdunCoreErrorDomain,
+				       AdCoreLogFileError,
+				       @"Error accessing error file",
+				       [NSString stringWithFormat: 
+					       @"Error file (%@) not writable.", errorFile],
+				       @"Check the write permissions in the simulation data directory.");
+			return NO;		
+		}
+	} 
+
+	fflush(stderr);
+	freopen([errorFile cString], "a", stderr);
+	freopen([logFile cString], "a", stdout);
+
+	GSPrintf(stdout, @"%@%@", divider, divider);
+	GSPrintf(stdout, @"%@ - Continuing Simulation\n", AdTimeStamp());
+	NSLog(@"%@", divider);
+	NSLog(@"%@ - Continuing Simulation\n", AdTimeStamp());
+	NSLog(@"%@", divider);
+
+	//Assign output dir ivar now
+	//If simulationDir has no containing path then use the
+	//current dir.
+	outputDir = [simulationDir stringByDeletingLastPathComponent];
+	if(outputDir == nil)
+		outputDir = [fileManager currentDirectoryPath];
+		
+	[outputDir retain];	
+
 	return YES;
 }
 
+- (BOOL) createLogFiles: (NSError**) error
+{
+
+	//If this is a new run create log files
+	//as specified by the command line arguments.
+	//If we are continuing a previous simulation 
+	//redirect the output to the old log files instead.
+	if(!restartRequested)
+		return [self _createLogFilesForNewRun: error];
+	else
+		return [self _setupLogFilesForContinuationRun: error];
+}
+
 - (BOOL) checkProgramDirectories: (NSError**) error
 {
 	NSArray* directoryArray;
 	NSEnumerator* directoryEnum;
 	NSString* currentDirectory;
 	id directory;
-
-//This should perhaps be checking if for FREEBSD
-//Im not sure what the cause of the case insenstive filenames is.
-#if __FREEBSD__	
-	adunDir = [[[NSUserDefaults standardUserDefaults] 
-			stringForKey: @"ProgramDirectoryLocation"]
-			stringByAppendingPathComponent: @".adun"];
-#else			
-	adunDir = [[[NSUserDefaults standardUserDefaults] 
-			stringForKey: @"ProgramDirectoryLocation"]
-			stringByAppendingPathComponent: @"adun"];	
-#endif					
-	
-	pluginDir = [adunDir stringByAppendingPathComponent: @"Plugins"];
-	controllerDir = [pluginDir stringByAppendingPathComponent: @"Controllers"];
-	extensionDir = [pluginDir stringByAppendingPathComponent: @"Extensions"];
-	
-	[adunDir retain];
-	[pluginDir retain];
-	[controllerDir retain];
-	[extensionDir retain];
-	
+		
 	directoryArray = [NSArray arrayWithObjects: 
 				adunDir,
 				pluginDir,
@@ -539,152 +692,39 @@ It returns NO if the directory does not exist
  * Loading of simulation data
  */
 
-- (BOOL) _loadServerData: (NSError**) error
-{
-	if(![self isConnected])
-	{
-		/*
-		 * An external object should detect if we are in 
-		 * AdCoreServerRunMode and the connection state before this 
-		 * calling this method. Hence we could be here due to a 
-		 * programmatic error.
-		 *
-		 * However it is also possible the server crashed after the connection
-		 * was made. Therefore we set an error instead of raising
-		 * an exception.
-		 */
-		*error = AdCreateError(AdunCoreErrorDomain,
-				AdCoreInvalidTemplateError,
-				@"Attempt to retrieve server data failed",
-				@"Program not connected to server and in AdCoreServerRunMode.",
-				@"This is possibly due to a server crash. However it could also indicate a bug in the program.");	
-		return NO;
-	}
-
-	GSPrintf(stdout, @"Retrieving data from the server.\n");
-	
-	simulatorTemplate = [serverProxy templateForProcess: 
-				[[NSProcessInfo processInfo] processIdentifier]];			
-	externalObjects = [serverProxy externalObjectsForProcess: 
-				[[NSProcessInfo processInfo] processIdentifier]];
-	
-	if(simulatorTemplate == nil)	
-	{
-		*error = AdCreateError(AdunCoreErrorDomain,
-				AdCoreInvalidTemplateError,
-				@"Error loading template",
-				@"Server returned nil for template.", 
-				@"Notify the developers of the error sending the template used to create this simulation.");	
-		return NO;
-	}
-	
-	if(externalObjects == nil || [externalObjects count] == 0)	
-	{
-		*error = AdCreateError(AdunCoreErrorDomain,
-				AdCoreInvalidTemplateError,
-				@"Error retrieving simulation object",
-				@"The server did not supply any data for the simulation",
-				@"Notify the developers of the error sending the template used to create this simulation.");	
-		return NO;
-	}
-
-	[simulatorTemplate retain];
-	[externalObjects retain];
-
-	return YES;
-}
-
-- (BOOL) _loadCommandLineData: (NSError**) error
-{
-	NSString* templateFile;
-	NSDictionary* dict;
-	NSMutableDictionary* temp;
-	NSEnumerator* keyEnum;
-	id key, object;
-	NSError* anError;
-
-	GSPrintf(stdout, @"Retrieving data from the command line.\n");
-
-	//Unarchive data in template file
-	templateFile = [[NSUserDefaults standardUserDefaults]
-			stringForKey: @"Template"];
-	simulatorTemplate = [NSMutableDictionary dictionaryWithContentsOfFile: templateFile];
-	//Check that it was unarchived correctly.
-	//Further template checks will be performed by an AdTemplateProcessor object.
-	if(simulatorTemplate == nil)
-	{
-		*error = AdCreateError(AdunCoreErrorDomain,
-				AdCoreInvalidTemplateError,
-				@"Error loading template",
-				[NSString stringWithFormat:
-					@"Unable to retrieve template from specified file %@", 
-					templateFile],
-				@"Check the specified file exists and contains a valid template object");	
-		
-		return NO;
-	}
-	else
-		[simulatorTemplate retain];
-
-	//Read in command line external objects declarations
-
-	anError = nil;
-	temp = nil;
-	if((dict = [[NSUserDefaults standardUserDefaults] dictionaryForKey: @"ExternalObjects"]) != nil)
-	{
-		keyEnum = [dict keyEnumerator];
-		temp = [NSMutableDictionary dictionary];
-		while((key = [keyEnum nextObject]))
-		{
-			object = [NSKeyedUnarchiver unarchiveObjectWithFile: 
-					[dict objectForKey: key]];
-			if(object == nil)
-			{
-				anError = AdCreateError(AdunCoreErrorDomain,
-						AdCoreInvalidTemplateError,
-						@"Error processing command line objects",
-						[NSString stringWithFormat:
-							@"Unable to retrieve object from file %@", 
-							[dict objectForKey: key]],
-						@"Check the specified file exists and contains a valid object");	
-				break;
-			}
-			[temp setObject: object
-				forKey: key];
-		}
-		
-		[externalObjects release];
-		externalObjects = [temp copy];
-	}
-
-	if(anError != nil)
-	{
-		*error = anError;
-		return NO;
-	}	
-	else
-		return YES;	
-}
-
 - (BOOL) loadData: (NSError**) error
 {
+	BOOL successFlag = NO;
+	NSString* templateFile, *dataDirectory;
+	AdFileSystemSimulationStorage* readModeStorage;
+
 	if(![self processCommandLine: error])
 		return NO;
 
-	if(runMode == AdCoreCommandLineRunMode)
-		return [self _loadCommandLineData: error];
-	else if(runMode == AdCoreServerRunMode)
-		return [self _loadServerData: error];
+	if(!restartRequested)
+	{
+		if(runMode == AdCoreCommandLineRunMode)
+			successFlag = [self _loadCommandLineData: error];
+		else if(runMode == AdCoreServerRunMode)
+			successFlag = [self _loadServerData: error];
+		else
+		{
+			/*
+			 * The run mode is still AdCoreUnknownRunMode
+			 * This should not happen since processCommandLine:
+			 * should set the run mode to one of these two values.
+			 * If we are here its due to a bug so raise an exception.
+			 */
+			
+			[NSException raise: NSInternalInconsistencyException
+				    format: @"Bug - Program in AdCoreUnknownRunMode when it should not be."];
+		}
+		
+	}
+	else
+		successFlag = [self _loadRestartData: error];
 	
-	/*
-	 * The run mode is still AdCoreUnknownRunMode
-	 * This should not happen since processCommandLine:
-	 * should set the run mode to one of these two values.
-	 * If we are here its due to a bug so raise an exception.
-	 */
-
-	 [NSException raise: NSInternalInconsistencyException
-		format: @"Bug - Program in AdCoreUnknownRunMode when it should not be."];
+	return successFlag;
 }
 
 - (void) setSimulationReferences: (NSDictionary*) inputObjects
@@ -726,8 +766,11 @@ It returns NO if the directory does not exist
 
 	//Send the simulation data object to the interface
 	if([self isConnected])
+	{
 		[serverProxy simulationData: simulationData
-			forProcess: [[NSProcessInfo processInfo] processIdentifier]];
+				 forProcess: [[NSProcessInfo processInfo] processIdentifier]];
+		simulationDataSent = YES;		 
+	}
 }
 
 /*
@@ -765,7 +808,7 @@ It returns NO if the directory does not exist
 				mode: AdSimulationStorageWriteMode
 				error: error];
 	readModeStorage = [[AdFileSystemSimulationStorage alloc]
-			initForReadingSimulationDataAtPath: dataDirectory];
+				initForReadingSimulationDataAtPath: dataDirectory];
 	[simulationData setDataStorage: readModeStorage];
 	
 	//To aid debugging etc. create a link to the dataDirectory,
@@ -902,6 +945,10 @@ It returns NO if the directory does not exist
 - (BOOL) createSimulationOutputDirectory: (NSError**) error
 {
 	NSString* optionsFile;
+	
+	//Do nothing if this is a restart
+	if(restartRequested)
+		return YES;
 
 	if(![self _createSimulationOutputDirectory: error])
 		return NO;
@@ -952,61 +999,9 @@ It returns NO if the directory does not exist
 	return YES;
 }
 
-/*
- * Input/Output related methods
- */
-
-- (FILE*) openFile: (NSString*) file  usingName: (NSString*) name flag: (NSString*) fileFlag
+- (BOOL) restartRequested
 {
-	const char* filename;
-	const char* flag;
-	FILE* file_p;
-
-	if(file == nil)
-	{
-		NSWarnLog(@"There is no file called %@\n", file);
-		return NULL;
-	}
-	
-	if(![fileManager fileExistsAtPath: file])
-		NSWarnLog(@"File %@ does not exist. Will create it if flag indicates\n", file);
-
-	filename = [file cString];
-	flag = [fileFlag cString];
-
-	//open the file
-
-	file_p = fopen(filename, flag);
-	if(file_p == NULL)
-	{
-		NSWarnLog(@"File %@ does not exist and flag is %@\n", file, fileFlag);
-		return NULL;
-	}
-	else
-		[fileStreams setObject: [NSValue valueWithPointer: file_p] forKey: name];
-
-	return file_p;
-}
-
-- (FILE*) getStreamForName: (NSString*) name
-{
-	return (FILE*)[[fileStreams objectForKey: name] pointerValue];
-}
-
-- (void) closeStreamWithName: (NSString*) name
-{
-	fclose([self getStreamForName: name]);
-	[fileStreams removeObjectForKey: name];
-}
-
-- (void) closeAllStreams
-{
-	NSEnumerator *enumerator = [fileStreams keyEnumerator];
-	id key;
-
-	while((key = [enumerator nextObject]))
-		if(![key isEqual:@"Error"] && ![key isEqual: @"Standard"])
-			[self closeStreamWithName: key];
+	return restartRequested;
 }
 
 - (void) saveResults: (NSArray*) anArray
@@ -1165,5 +1160,303 @@ It returns NO if the directory does not exist
 	return [core validCommands];
 }
 
+/*
+ * Input/Output related methods
+ */
+
+- (FILE*) openFile: (NSString*) file  usingName: (NSString*) name flag: (NSString*) fileFlag
+{
+	const char* filename;
+	const char* flag;
+	FILE* file_p;
+
+	if(file == nil)
+	{
+		NSWarnLog(@"There is no file called %@\n", file);
+		return NULL;
+	}
+	
+	if(![fileManager fileExistsAtPath: file])
+		NSWarnLog(@"File %@ does not exist. Will create it if flag indicates\n", file);
+
+	filename = [file cString];
+	flag = [fileFlag cString];
+
+	//open the file
+
+	file_p = fopen(filename, flag);
+	if(file_p == NULL)
+	{
+		NSWarnLog(@"File %@ does not exist and flag is %@\n", file, fileFlag);
+		return NULL;
+	}
+	else
+		[fileStreams setObject: [NSValue valueWithPointer: file_p] forKey: name];
+
+	return file_p;
+}
+
+- (FILE*) getStreamForName: (NSString*) name
+{
+	return (FILE*)[[fileStreams objectForKey: name] pointerValue];
+}
+
+- (void) closeStreamWithName: (NSString*) name
+{
+	fclose([self getStreamForName: name]);
+	[fileStreams removeObjectForKey: name];
+}
+
+- (void) closeAllStreams
+{
+	NSEnumerator *enumerator = [fileStreams keyEnumerator];
+	id name;
+	
+	while(name = [enumerator nextObject])
+		[self closeStreamWithName: name];
+
+}
+
+@end
+
+/*
+Contains methods for handling the loading of simulation data
+in the different run modes - server, command line and restart
+*/
+@implementation AdIOManager (SimulationDataLoading)
+
+- (BOOL) _loadServerData: (NSError**) error
+{
+	if(![self isConnected])
+	{
+		/*
+		 * An external object should detect if we are in 
+		 * AdCoreServerRunMode and the connection state before this 
+		 * calling this method. Hence we could be here due to a 
+		 * programmatic error.
+		 *
+		 * However it is also possible the server crashed after the connection
+		 * was made. Therefore we set an error instead of raising
+		 * an exception.
+		 */
+		*error = AdCreateError(AdunCoreErrorDomain,
+				       AdCoreInvalidTemplateError,
+				       @"Attempt to retrieve server data failed",
+				       @"Program not connected to server and in AdCoreServerRunMode.",
+				       @"This is possibly due to a server crash. However it could also indicate a bug in the program.");	
+		return NO;
+	}
+	
+	GSPrintf(stdout, @"Retrieving data from the server.\n");
+	
+	simulatorTemplate = [serverProxy templateForProcess: 
+			     [[NSProcessInfo processInfo] processIdentifier]];			
+	externalObjects = [serverProxy externalObjectsForProcess: 
+			   [[NSProcessInfo processInfo] processIdentifier]];
+	
+	if(simulatorTemplate == nil)	
+	{
+		*error = AdCreateError(AdunCoreErrorDomain,
+				       AdCoreInvalidTemplateError,
+				       @"Error loading template",
+				       @"Server returned nil for template.", 
+				       @"Notify the developers of the error sending the template used to create this simulation.");	
+		return NO;
+	}
+	
+	if(externalObjects == nil || [externalObjects count] == 0)	
+	{
+		*error = AdCreateError(AdunCoreErrorDomain,
+				       AdCoreInvalidTemplateError,
+				       @"Error retrieving simulation object",
+				       @"The server did not supply any data for the simulation",
+				       @"Notify the developers of the error sending the template used to create this simulation.");	
+		return NO;
+	}
+	
+	[simulatorTemplate retain];
+	[externalObjects retain];
+	
+	return YES;
+}
+
+- (BOOL) _loadCommandLineData: (NSError**) error
+{
+	NSString* templateFile;
+	NSDictionary* dict;
+	NSMutableDictionary* temp;
+	NSEnumerator* keyEnum;
+	id key, object;
+	NSError* anError;
+	
+	GSPrintf(stdout, @"Retrieving data from the command line.\n");
+	
+	//Unarchive data in template file
+	templateFile = [[NSUserDefaults standardUserDefaults]
+			stringForKey: @"Template"];
+	simulatorTemplate = [NSMutableDictionary dictionaryWithContentsOfFile: templateFile];
+	//Check that it was unarchived correctly.
+	//Further template checks will be performed by an AdTemplateProcessor object.
+	if(simulatorTemplate == nil)
+	{
+		*error = AdCreateError(AdunCoreErrorDomain,
+				       AdCoreInvalidTemplateError,
+				       @"Error loading template",
+				       [NSString stringWithFormat:
+					@"Unable to retrieve template from specified file %@", 
+					templateFile],
+				       @"Check the specified file exists and contains a valid template object");	
+		
+		return NO;
+	}
+	else
+		[simulatorTemplate retain];
+	
+	//Read in command line external objects declarations
+	
+	anError = nil;
+	temp = nil;
+	if((dict = [[NSUserDefaults standardUserDefaults] dictionaryForKey: @"ExternalObjects"]) != nil)
+	{
+		keyEnum = [dict keyEnumerator];
+		temp = [NSMutableDictionary dictionary];
+		while((key = [keyEnum nextObject]))
+		{
+			object = [NSKeyedUnarchiver unarchiveObjectWithFile: 
+				  [dict objectForKey: key]];
+			if(object == nil)
+			{
+				anError = AdCreateError(AdunCoreErrorDomain,
+							AdCoreInvalidTemplateError,
+							@"Error processing command line objects",
+							[NSString stringWithFormat:
+							 @"Unable to retrieve object from file %@", 
+							 [dict objectForKey: key]],
+							@"Check the specified file exists and contains a valid object");	
+				break;
+			}
+			[temp setObject: object
+				 forKey: key];
+		}
+		
+		[externalObjects release];
+		externalObjects = [temp copy];
+	}
+	
+	if(anError != nil)
+	{
+		*error = anError;
+		return NO;
+	}	
+	else
+		return YES;	
+}
+
+- (BOOL) _loadRestartData: (NSError**) error
+{
+	BOOL successFlag;
+	int numberOfFrames, numberOfSystems, numberOfExternalObjects;
+	NSString* templateFile, *dataDirectory, *name;
+	NSMutableDictionary* temp;
+	NSEnumerator* systemEnum;
+	AdSystemCollection* systemCollection;
+	AdFileSystemSimulationStorage* readModeStorage;
+	id system, dataSource;
+	
+	simulationData = [NSKeyedUnarchiver unarchiveObjectWithFile:
+			  [[NSUserDefaults standardUserDefaults] 
+			   stringForKey: @"Continue"]];
+	[simulationData retain];
+	
+	dataDirectory = [[simulationData identification]
+			 stringByAppendingString: @"_Data"];
+	dataDirectory = [outputDir stringByAppendingPathComponent: dataDirectory];	
+	readModeStorage = [AdFileSystemSimulationStorage 
+			   storageForSimulation: simulationData
+			   inDirectory: outputDir
+			   mode: AdSimulationStorageReadMode 
+			   error: error];
+	
+	//Load the template
+	templateFile = [dataDirectory stringByAppendingPathComponent: @"Template"];
+	simulatorTemplate = [NSDictionary dictionaryWithContentsOfFile: templateFile];
+	
+	if(*error != nil)
+	{	
+		successFlag = NO;	   
+	}
+	else if(simulatorTemplate == nil)
+	{
+		//Create a new error with the data storage error underlying it.
+		*error = AdErrorWithUnderlyingError(AdunCoreErrorDomain,
+				       AdCoreRestartError, 
+				       @"Unable to restart simulation", 
+				       @"Could not access simulation storage", 
+				       @"See underlying error for more information",
+				       *error);
+		successFlag = NO;
+	}
+	else
+	{
+		[simulationData setDataStorage: readModeStorage];
+		[simulationData loadData];
+		[simulatorTemplate retain];
+		writeModeStorage = [[AdFileSystemSimulationStorage alloc]
+				    initSimulationStorageAtPath: dataDirectory
+				    mode: AdSimulationStorageAppendMode
+				    error: error];	
+		
+		/*
+		 * FIXME: There is a problem when it comes to the external objects.
+		 * In the template the externalObjects section contains the names of files containing objects.
+		 * However this information is not stored at the end of the simulation.
+		 * Therefore on restart we cannot know which data source in the simulation data
+		 * corresponds to which external object name in the template.
+		 * The problem is compounded if extra external object were specified on the command line
+		 * since there is no recorded information on these names at all.
+		 *
+		 * There are three solutions
+		 * 1 - The entire object graph has to be encoded.
+		 * 2 - The correspondance between the template names given to each external object  
+		 * and the external objects themselves has to be recorded.
+		 * 3 - The above correpsondance has to be specified by the user on restart.
+		 *
+		 * For now only handle case where there was one external object and one data source.
+		 */
+		
+		temp = [[simulatorTemplate objectForKey: @"externalObjects"] mutableCopy];
+		systemCollection = [simulationData systemCollection];
+		numberOfFrames = [simulationData numberOfFrames];
+		numberOfSystems = [[systemCollection fullSystems] count];
+		numberOfExternalObjects = [temp count]; 
+		if(numberOfSystems == numberOfExternalObjects)
+		{
+			//Get the only data source
+			system = [[systemCollection fullSystems] objectAtIndex: 0];
+			dataSource = [system dataSource];
+			name = [[temp allKeys] objectAtIndex: 0];
+			[temp setObject: dataSource forKey: name];
+			externalObjects = [temp copy];
+			
+			NSLog(@"Loaded Restart Data -  %@", simulationData); 
+			NSLog(@"External objects - %@", externalObjects);
+			successFlag = YES;
+		}
+		else
+		{
+			 *error = AdCreateError(AdunCoreErrorDomain,
+						AdCoreRestartError, 
+						@"Error restarting simulation", 
+						@"Unable to determine external object mapping", 
+						@"As of version 0.81 only single system simulations can be restarted");
+			 successFlag = NO;
+		}
+		
+		[temp release];
+	}	
+	
+	return successFlag;	
+}
+
 @end
 
