@@ -21,6 +21,8 @@
 */
 
 #include "AdunKernel/AdunSimulationData.h"
+#include "AdunKernel/AdunFileSystemSimulationStorage.h"
+#include "AdunKernel/AdunMolecularMechanicsForceField.h"
 
 @implementation AdSimulationData
 
@@ -80,6 +82,37 @@ Creation
 
 ***************/
 
++ (id) simulationFromArchive: (NSString*) filename loadData: (BOOL) flag
+{
+	return [self simulationFromArchive: filename loadData: flag error: NULL];
+}
+
++ (id) simulationFromArchive: (NSString*) filename loadData: (BOOL) flag error: (NSError**) error
+{
+	id object;
+	AdFileSystemSimulationStorage *storage;
+	
+	object = [NSKeyedUnarchiver unarchiveObjectWithFile: filename];
+	if(![object isKindOfClass: [self class]])
+		[NSException raise: NSInvalidArgumentException
+			    format: @"File %@ does not contain a valid AdSimulationData archive", filename];
+	
+	//If flag is YES, attempt to load the simulation data.
+	//Assume its in the same directory and its name is filename_Data.		
+	
+	if(flag)
+	{
+		storage = [AdFileSystemSimulationStorage storageForSimulation: object 
+								  inDirectory: [filename stringByDeletingLastPathComponent] 
+									 mode: AdSimulationStorageReadMode 
+									error: error];
+		[object setDataStorage: storage];
+		[object loadData];
+	}
+	
+	return [object retain];
+}
+
 - (id) init
 {
 	return [self initWithName: nil];	
@@ -111,6 +144,7 @@ Creation
 
 - (void) loadData
 {
+	NSException* exception;
 	NSData *energyData;
 	NSError* error;
 	
@@ -124,9 +158,10 @@ Creation
 	{
 		NSWarnLog(@"Simulation data is not accesible raising exception");
 		error = [dataStorage accessError];
-		[NSException raise: NSInternalInconsistencyException
-			format: [[error userInfo] 
-				objectForKey: NSLocalizedDescriptionKey]];
+		exception = [NSException exceptionWithName: NSInternalInconsistencyException
+				reason: @"Unable to access data storage"
+				userInfo:[error userInfo]];
+		[exception raise];			
 	}
 	
 	//Get rid of anything we loaded previously
@@ -158,7 +193,6 @@ Creation
 	[self _loadFrames];
 	NSDebugLLog(@"AdSimulationData", @"Complete");
 		
-	
 	NSDebugLLog(@"AdSimulationData", @"Initialised AdSimulationData");
 }
 
@@ -179,6 +213,7 @@ Creation
 
 - (void) setDataStorage: (id) object
 {
+	double size;
 	NSString* dataId;
 
 	if(dataStorage != object)
@@ -197,6 +232,11 @@ Creation
 				NSWarnLog(@"Supplied data storage is not related to this object");
 				return;
 			}
+	
+	size = ((double)[dataStorage sizeOfStore])/(1024.0*1024.0);	
+	[self setValue: [NSString stringWithFormat: @"%-5.2lf MB", size]
+		forMetadataKey: @"Size"
+		inDomain: AdSystemMetadataDomain];		
 }
 
 - (AdDataSet*) energies
@@ -209,6 +249,11 @@ Creation
 	return [[systemCollection retain] autorelease];
 }
 
+- (AdDataMatrix*) frames	
+{
+	return [[frames retain] autorelease];
+}
+
 - (unsigned int) numberTrajectoryCheckpoints
 {
 	return [dataStorage numberTrajectoryCheckpoints];
@@ -457,7 +502,7 @@ Deprecated
 	NSArray* headers, *forceFieldArray;
 	NSMutableArray* stateHeaders;
 	NSEnumerator* systemEnum;
-	AdForceField* systemForceField;
+	AdMolecularMechanicsForceField* systemForceField;
 	id system, stateMatrix;
 
 	if(iterationHeader == nil)
@@ -491,8 +536,8 @@ Deprecated
 			[stateHeaders removeObject: @"KineticEnergy"];
 			[stateHeaders removeObject: @"Temperature"];
 		}	
-		//FIXME: Does not handle custom terms.
-		[stateHeaders addObjectsFromArray: [systemForceField coreTerms]];		
+		
+		[stateHeaders addObjectsFromArray: [systemForceField allTerms]];		
 		stateMatrix = [[AdMutableDataMatrix alloc]
 					initWithNumberOfColumns: [stateHeaders count]
 					columnHeaders: stateHeaders
@@ -522,44 +567,113 @@ Deprecated
 	iterationHeader: (NSString*) aString
 {
 	NSArray* headers;
+	AdDataMatrix* matrix;
+	AdFileSystemSimulationStorage* tempStorage;	//!< Used for AdSimulationStorageAppendMode
+	AdSimulationData* dataReader;
+	AdSystemCollection* storedSystemCollection;	//!< Used fo AdSimulationStorageAppendMode
 
 	if((self = [super init]))
 	{
 		if(aDataStore == nil)
 			[NSException raise: NSInvalidArgumentException
 				format: @"aDataStore cannot be nil"];
+				
+		frameOpen = NO;		
 	
-		if([aDataStore storageMode] != AdSimulationStorageWriteMode)
+		//Depending on whether the store is in write or append
+		//mode we have to initialise it in a different way
+		if([aDataStore storageMode] == AdSimulationStorageWriteMode)
+		{
+			lastFrame = -1;
+			systemCollection = nil;
+			forceFieldCollection = nil;
+			iterationHeader = [aString retain];
+			iterationValue = nil;
+			stateData = [[AdDataSet alloc] 
+					initWithName: @"StateData"
+					inputReferences: nil
+					dataGenerator: [NSBundle mainBundle]];
+			topologyData = [NSMutableDictionary new];
+			energyData = [NSMutableDictionary new];
+			//This is allocated on each trajectory checkpoint request
+			trajectoryData = nil;
+			dataStorage = [aDataStore retain];	
+			headers = [NSArray arrayWithObjects: 
+					@"Energy",
+					@"Trajectory",
+					@"Topology",
+					nil];
+			frames = [[AdMutableDataMatrix alloc]
+					initWithNumberOfColumns: 3
+					columnHeaders: headers
+					columnDataTypes: nil];	
+					
+			[self setSystems: aSystemCollection];
+			[self setForceFields: aForceFieldCollection];
+		}
+		else if([aDataStore storageMode] == AdSimulationStorageAppendMode)
+		{
+			//Since we are appending to an existing store we
+			//have to make sure the state of this object
+			//is as it would be just after the last frame was written.
+		
+			dataStorage = [aDataStore retain];
+			topologyData = [NSMutableDictionary new];
+			energyData = [NSMutableDictionary new];
+			forceFieldCollection = nil;
+			trajectoryData = nil;
+			
+			//Create a read-mode storage and an AdSimulationData instance
+			//to read the stored data we require.
+			tempStorage = [[AdFileSystemSimulationStorage alloc]
+					initForReadingSimulationDataAtPath: [dataStorage storagePath]];
+			dataReader = [AdSimulationData new];
+			[dataReader setDataStorage: tempStorage];
+			[dataReader loadData];
+			
+			storedSystemCollection = [dataReader systemCollection];
+			stateData = [dataReader energies];
+			frames = [[dataReader frames] mutableCopy];
+			lastFrame = (int)[frames numberOfRows] - 1;
+		
+			[stateData retain];
+			[dataReader release];
+			[tempStorage release];
+			
+			//Read iteration header and value from state data
+			//Assumes all systems checkpointed at same interval.
+			matrix = [[stateData dataMatrices] objectAtIndex: 0];		
+			iterationHeader = [[matrix columnHeaders] objectAtIndex: 0];
+			//Iteration value is the value for the iteration column in the last opened frame.
+			//This is usually the step the configuration generator was last at.
+			iterationValue = [matrix elementAtRow: [matrix numberOfRows] - 1
+						ofColumnWithHeader: iterationHeader];
+			[iterationHeader retain];
+			[iterationValue retain];
+			
+			//We can't use setSystemCollection: since this will trigger things we dont want.
+			//We also can't use storedSystemCollection because the objects in it
+			//may not be the same (that is they should have the same data but may not the same objects)
+			//as those in aSystemCollection.
+			//This will not overwrite the stored collection as we don't use setSystemCollection,
+			//though since they contain the same data it wouldnt really matter
+			//FIXME: This will need to be changed because there is no way to know if
+			//aSystemCollectin and storedSystemCollection contain the same data.
+			systemCollection = [aSystemCollection retain];
+			
+			//Similarly we can't use setForceFieldCollection: since this would trigger
+			//creating the stateData ivar which we already have.
+			//Therefore we set the explicitly here.
+			forceFieldCollection = [aForceFieldCollection retain];
+			
+			NSDebugLLog(@"AdSimulationDataWriter", 
+				@"Writing to append mode storage - Last frame %d, iteration header %@, last iteration value %@", 	
+				lastFrame, iterationHeader, iterationValue);					
+		}
+		else
 			[NSException raise: NSInternalInconsistencyException
-				format: @"You can only write data to a store in AdSimulationStorageWriteMode"];
-	
-		lastFrame = -1;
-		systemCollection = nil;
-		forceFieldCollection = nil;
-		iterationHeader = [aString retain];
-		iterationValue = nil;
-		stateData = [[AdDataSet alloc] 
-				initWithName: @"StateData"
-				inputReferences: nil
-				dataGenerator: [NSBundle mainBundle]];
-		topologyData = [NSMutableDictionary new];
-		energyData = [NSMutableDictionary new];
-		//This is allocated on each trajectory checkpoint request
-		trajectoryData = nil;
-		dataStorage = [aDataStore retain];	
-		headers = [NSArray arrayWithObjects: 
-				@"Energy",
-				@"Trajectory",
-				@"Topology",
-				nil];
-		frames = [[AdMutableDataMatrix alloc]
-				initWithNumberOfColumns: 3
-				columnHeaders: headers
-				columnDataTypes: nil];		
+				format: @"You can only write data to a store in AdSimulationStorageWriteMode or AdSimulationStorageAppendMode"];
 		
-		[self setSystems: aSystemCollection];
-		[self setForceFields: aForceFieldCollection];
-
 	}
 
 	return self;
@@ -604,6 +718,11 @@ Deprecated
 		[self _createStateMatrices];
 }
 
+- (AdSystemCollection*) systems
+{
+	return [[systemCollection retain] autorelease];
+}
+
 - (void) updateSystems
 {
 	NSWarnLog(@"Method %@ not implemented", NSStringFromSelector(_cmd));
@@ -663,14 +782,66 @@ Deprecated
 	NSEnumerator* dataEnum;
 	NSString* systemName;
 	id state, stateMatrix;
+	//For column addition
+	int i, noRows;
+	NSEnumerator* termEnum;
+	NSArray* headers;
+	NSMutableArray* newTerms, *column;
+	NSString* term;
 
 	dataEnum = [energyData keyEnumerator];
 	while((systemName = [dataEnum nextObject]))
 	{
 		stateMatrix = [stateData dataMatrixWithName: 
 				systemName];
-		state = [energyData objectForKey: systemName];		
-		[stateMatrix extendMatrixWithRow: state];
+		state = [energyData objectForKey: systemName];	
+		
+		/*
+		 * We need to check that the number of energy terms returned
+		 * for the system is the same as the number of terms in the matrix.
+		 * They could differ if extra terms were added or removed to the force-field
+		 * operating on the system since the last checkpoint.
+		 *
+		 * Two cases:
+		 * 1. In the case of an extra term we must add an extra column to the matrix and
+		 * fill the previous steps where the term didn't exist with 0's.
+		 * 2. In the case where a term is removed we simply put a zero for its value
+		 * until it appears again.
+		 */
+		
+		if([[state allKeys] count] > [stateMatrix numberOfColumns])
+		{
+			//Find the new keys
+			termEnum = [state keyEnumerator];
+			headers = [stateMatrix columnHeaders];
+			newTerms = [NSMutableArray array];
+			while(term = [termEnum nextObject])
+			{
+				if(![headers containsObject: term])
+					[newTerms addObject: term];
+			}
+			
+			column = [NSMutableArray new];
+			termEnum = [newTerms objectEnumerator];
+			while(term = [termEnum nextObject])
+			{
+				//Create a column for this term
+			
+				noRows = [stateMatrix numberOfRows];
+				for(i=0; i<noRows; i++)
+					[column addObject: [NSNumber numberWithDouble: 0.0]];
+					
+				[stateMatrix extendMatrixWithColumn: column];
+				[stateMatrix setHeaderOfColumn: [stateMatrix numberOfColumns] - 1 
+					to: term];
+				
+				[column removeAllObjects];
+			}
+			
+			[column release];
+		}
+		
+		[stateMatrix extendMatrixWithColumnValues: state];
 	 }		
 }
 
@@ -698,19 +869,26 @@ Deprecated
 			@"Checkpointed %d bytes of trajectory data",
 			[trajectoryData length]);
 		[checkpointed addObject: [NSNumber numberWithInt: 1]];
+		NSDebugLLog(@"AdSimulationDataWriter",
+			    @"Done ");
 	}
 	else
 		[checkpointed addObject: [NSNumber numberWithInt: 0]];
 
 	if(energyCheckpoint)
 	{
+		
 		if(forceFieldCollection == nil)
 			[checkpointed addObject: [NSNumber numberWithBool: NO]];
 		else
 		{
+			NSDebugLLog(@"AdSimulationDataWriter",
+				    @"Checkpointing energies");
 			[self _addEnergyCheckpoint];
 			[checkpointed addObject: [NSNumber numberWithInt: 1]];
 			[energyData removeAllObjects];
+			NSDebugLLog(@"AdSimulationDataWriter",
+				    @"Done ");
 		}
 	}
 	else
@@ -747,11 +925,17 @@ Deprecated
 
 	[frames extendMatrixWithRow: checkpointed];
 
+	NSDebugLLog(@"AdSimulationDataWriter",
+		    @"Cleaning up");
+
 	//Set for next frame
 	[checkpointed release];
 	energyCheckpoint = NO;
 	trajectoryCheckpoint = NO;
 	topologyCheckpoint = NO;
+	
+	NSDebugLLog(@"AdSimulationDataWriter",
+		    @"Frame closed");
 }
 
 - (int)  lastFrame
@@ -792,6 +976,11 @@ The first frame should indicate a checkpoint for everything
 	return [self _lastCheckpointInColumn: @"Trajectory"];
 }
 
+- (NSNumber*) lastIterationNumber
+{
+	return [[iterationValue retain] autorelease];
+}
+
 - (void) addTopologyCheckpoint
 {
 	NSEnumerator* systemEnum;
@@ -849,9 +1038,10 @@ The first frame should indicate a checkpoint for everything
 {
 	double potentialEnergy, totalEnergy, kineticEnergy;
 	NSEnumerator* systemEnum;
-	AdForceField* systemForceField;
+	NSMutableArray* stateTerms = [NSMutableArray new];
+	AdMolecularMechanicsForceField* systemForceField;
 	id system, state;
-
+	
 	energyCheckpoint = YES;
 	
 	//Check that a forceFieldCollection is available before continuing
@@ -863,23 +1053,25 @@ The first frame should indicate a checkpoint for everything
 
 	//Remove last energy checkpoint
 	[energyData removeAllObjects];
-	
+		
 	//Checkpoint new energies
-	//Go through each system and add the relevant data
-	//to an array. Each array is added to energyData dictionary.
-	//When the frame is close the arrays are added to the
-	//state matrix for each system (in closeFrame:)
+	//Go through each system and add the relevant data to a dictionary of (TermName:TermValue pairs). 
+	//Each dictionary is then added to energyData dictionary.
+	//When the frame is close the arrays are added to the state matrix for each system (in closeFrame:)
+	state = [NSMutableArray new];
 	systemEnum = [[systemCollection allSystems] objectEnumerator];
 	while((system = [systemEnum nextObject]))
 	{
-		state = [NSMutableArray array];
 		//FIXME: Only one force field per system supported
 		systemForceField = [[forceFieldCollection forceFieldsForSystem: system]
 					objectAtIndex: 0];
 		[state addObject: iterationValue];
+		[stateTerms addObject: iterationHeader];
+		
 		potentialEnergy = [systemForceField totalEnergy];
 		[state addObject:
 			[NSNumber numberWithDouble: potentialEnergy]];
+		[stateTerms addObject: @"PotentialEnergy"];	
 
 		totalEnergy = potentialEnergy;
 		if([system respondsToSelector: @selector(kineticEnergy)])
@@ -888,19 +1080,37 @@ The first frame should indicate a checkpoint for everything
 			totalEnergy += kineticEnergy;
 			[state addObject:
 				[NSNumber numberWithDouble: kineticEnergy]];
+			[stateTerms addObject: @"KineticEnergy"];
+				
 			[state addObject:
 				[NSNumber numberWithDouble: [system temperature]]];
+			[stateTerms addObject: @"Temperature"];		
 		}
-			
+				
 		[state addObject:
 			[NSNumber numberWithDouble: totalEnergy]];
+		[stateTerms addObject: @"TotalEnergy"];	
+			
 		[state addObjectsFromArray: 
-			[systemForceField arrayOfCoreTermEnergies]];
+			[systemForceField allEnergies]];
+		[stateTerms addObjectsFromArray: [systemForceField allTerms]];	
+			
+		NSDebugLLog(@"AdSimulationDataWriter", 
+			@"State %@. State Terms %@", state, stateTerms);	
+			
 		//Add the array containing the checkpointed energies
 		//to the energyData dictionary
-		[energyData setObject: state
+		[energyData setObject: 
+				[NSDictionary dictionaryWithObjects: state 
+					forKeys: stateTerms]
 			forKey: [system systemName]];
-	 }		
+			
+		[state removeAllObjects];
+		[stateTerms removeAllObjects];	
+	 }	
+	 
+	[stateTerms release];
+	[state release];		
 }
 
 - (void) rollBackToFrame: (unsigned int) value
