@@ -65,12 +65,22 @@ unsigned integer.
 
 	NSDebugLLog(@"AdTemplateProcessor", @"Validating controller %@", controllerName);
 	controllerDir = [[AdIOManager appIOManager] controllerDirectory];
+	
+#ifdef GNUSTEP	
 	NSDebugLLog(@"AdTemplateProcessor",
-		@"Dynamicaly loading controller from directory: %@.",
+		@"Dynamicaly loading controller from directory: %@",
 		 [controllerDir stringByAppendingPathComponent: controllerName]);
-
 	controllerBundle = [NSBundle bundleWithPath: 
 				[controllerDir stringByAppendingPathComponent: controllerName]];
+#else				
+	controllerName = [controllerName stringByAppendingPathExtension: @"bundle"];
+	NSDebugLLog(@"AdTemplateProcessor",
+		    @"Dynamicaly loading controller from directory: %@",
+		    [controllerDir stringByAppendingPathComponent: controllerName]);
+	controllerBundle = [NSBundle bundleWithPath: 
+			    [controllerDir stringByAppendingPathComponent: controllerName]];
+	controllerName = [controllerName stringByDeletingPathExtension];
+#endif			    
 	
 	if(controllerBundle == nil)
 	{
@@ -143,6 +153,7 @@ then it is not loaded.
 	NSDictionary* externalObjectFiles;
 	NSEnumerator* objectNameEnum;
 	NSString* objectName, *fileName;
+	NSData* data;
 	id object;
 
 	NSDebugLLog(@"AdTemplateProcessor", 
@@ -165,7 +176,30 @@ then it is not loaded.
 		fileName = [externalObjectFiles objectForKey: objectName];
 		NSDebugLLog(@"AdTemplateProcessor", 
 			@"Processing object %@ from file %@", objectName, fileName);
-		object = [NSKeyedUnarchiver unarchiveObjectWithFile: fileName];
+			
+		//The exception raised by NSKeyedUnarchiver if it can't understand
+		//data archive format is caught since the data may represent a plist
+		//which must be unarchived with NSPropertyListSerialization (at least on cocoa) 
+		data = [NSData dataWithContentsOfFile: fileName];
+			
+		NS_DURING
+		{
+			object = [NSKeyedUnarchiver unarchiveObjectWithData:data];
+			if(object == nil)
+				[NSException raise: NSInvalidArgumentException
+					    format: @"Data not a valid archive"];
+		}
+		NS_HANDLER
+		{		
+			NSWarnLog(@"Detected possible plist external object - attempting to decode");
+			object = [NSPropertyListSerialization propertyListFromData: data 
+								  mutabilityOption: NSPropertyListMutableContainers 
+									    format: NULL 
+								  errorDescription: NULL];		
+		}
+		NS_ENDHANDLER	
+		
+		//If object is nil give up.
 		if(object == nil)
 		{
 			*error = AdCreateError(AdunCoreErrorDomain,
@@ -272,17 +306,20 @@ then it is not loaded.
 
 	/*
 	 * Checks -
-	 * 1) If running from command line External objects sections is present
+	 * 1) If running from command line and no controller specified that an 
+		external objects sections is present
 	 * 2) templateObjects section is present
 	 * 3) All keys are NSStrings.
-	 * 4) configurationGenerator key is present.
+	 * 4) configurationGenerator key is present if no controller specified
 	 * 5) controller key is present
 	 * 6) controller key refers to a real loadable controller
 	 * 7) All Class keys refer to loaded classes
 	 */
 	
 	if([[AdIOManager appIOManager] runMode] == AdCoreCommandLineRunMode)
-		if([testTemplate valueForKey: @"externalObjects"] == nil)
+	{
+		if([testTemplate valueForKey: @"externalObjects"] == nil && 
+			![testTemplate valueForKeyPath: @"objectTemplates.controller"])
 		{
 			*error = AdCreateError(AdunCoreErrorDomain,
 					AdCoreInvalidTemplateError,
@@ -290,7 +327,8 @@ then it is not loaded.
 					@"The template must contain an externalObject section .",
 					@"Check template documentation for how to create a correct template.");
 			return NO;
-		}	
+		}
+	}	
 
 	if([testTemplate valueForKey: @"objectTemplates"] == nil)
 	{
@@ -302,22 +340,26 @@ then it is not loaded.
 		return NO;
 	}
 	
-	allKeys = [[[testTemplate objectForKey: @"externalObjects"] allKeys] mutableCopy];
-	[allKeys autorelease];
+	allKeys = [NSMutableArray array];
+	if(![testTemplate objectForKey: @"externalObjects"] == nil)
+		[allKeys addObjectsFromArray: [[testTemplate objectForKey: @"externalObjects"] allKeys]];
+
 	[allKeys addObjectsFromArray: [[testTemplate objectForKey: @"objectTemplates"] allKeys]];
 	keyEnum = [allKeys objectEnumerator];
 	while((key = [keyEnum nextObject]))
+	{
 		if(![key isKindOfClass: [NSString class]])
 		{
 			*error = AdCreateError(AdunCoreErrorDomain,
-				AdCoreInvalidTemplateError,
-				@"Invalid template.",
-				@"Non-string object used for template key.",
-				@"Check the template keys and specifications for property list objects.");
+					       AdCoreInvalidTemplateError,
+					       @"Invalid template.",
+					       @"Non-string object used for template key.",
+					       @"Check the template keys and specifications for property list objects.");
 			return NO;
-		}	
+		}
+	}
 	
-	if(![allKeys containsObject: @"configurationGenerator"])
+	if(![allKeys containsObject: @"configurationGenerator"] && ![allKeys containsObject: @"controller"])
 	{
 		*error = AdCreateError(AdunCoreErrorDomain,
 				AdCoreInvalidTemplateError,
@@ -325,7 +367,11 @@ then it is not loaded.
 				@"Each template must contain a configuration generator.",
 				@"Add a configuration generator entry to the template.");
 		return NO;
-	}	
+	}
+	else if(![allKeys containsObject: @"configurationGenerator"])
+	{	
+		NSWarnLog(@"No configuration generator specified - assuming controller will handle this");
+	}
 
 	//Checking controller key is present and the controller exists.
 	if(![allKeys containsObject: @"controller"])
@@ -423,6 +469,7 @@ then it is not loaded.
 	if(![self _instantiateExternalObjects: error])
 		return NO;
 	
+	//FIXME - May be none.
 	//Check there are at least some external objects
 	if([externalObjects count] == 0)
 	{
@@ -441,7 +488,7 @@ then it is not loaded.
 	 * When a processing error is detected an error object is created
 	 * and assigned to the processingError ivar.
 	 * An exception is then raised which we catch here.
-	 * An exception is used since the following loop can * result in
+	 * An exception is used since the following loop can result in
 	 * deeply recursive calls to instantiateObjectWithTemplate:associateName:
 	 */
 	[processingError release];
@@ -491,6 +538,11 @@ then it is not loaded.
 		return YES;
 }
 
+- (NSDictionary*) createdObjects
+{
+	return [[buildDict copy] autorelease];
+}
+
 - (AdConfigurationGenerator*) configurationGenerator
 {
 	return [buildDict objectForKey: @"configurationGenerator"];
@@ -662,6 +714,12 @@ contents are resolved using _resolveContainerEntries.
 	 * Its autoreleased so we don't have to worry about it if
 	 * something happens between now and when its added to
 	 * the build dict.
+	 * However this is dangerous as during the objects init method
+	 * autorelease could be called on this object - for example
+	 * if it was replacing itself with another object - leading
+	 * to a crash when the autorelase pool is emptied.
+	 * Hence this object is only used to access the validation methods
+	 * afterwards its replaced.
 	 */	
 	object = [[class alloc] autorelease];
 	keys = [[[objectTemplate allKeys] mutableCopy] autorelease];
@@ -741,13 +799,13 @@ contents are resolved using _resolveContainerEntries.
 	{
 		NSDebugLLog(@"AdTemplateProcessor",
 			@"Initialising %@ using initWithDictionary", object);
-		[object initWithDictionary: properties];
+		object = [[class alloc] initWithDictionary: properties];
 	}
 	else
 	{
 		NSDebugLLog(@"AdTemplateProcessor",
 			@"Initialising %@ using init", object);
-		[object init];
+		object = [[class new] autorelease];
 		NSDebugLLog(@"AdTemplateProcessor", 
 			@"Setting properties via key value coding");
 
