@@ -317,9 +317,11 @@ Create List Elements
 //and invalidate the current list.
 - (void) _handleSystemContentsChange: (NSNotification*) aNotification
 {
+
+	//Note: It is not safe to call description (i.e print ourselves)
+	//Until the reloading has been done - there is an update call in description.
 	NSDebugLLog(@"AdCellListHandler", 
-		@"%@. Received contents change notification from %@",
-		[self description],
+		@"Received contents change notification from %@",
 		[system systemName]);
 
 	//Deallocated all current cell and coordinate related ivars.
@@ -345,6 +347,8 @@ Create List Elements
 	[self _initialisationForCoordinates];
 
 	[delegate handlerDidHandleContentChange: self];
+	
+	NSDebugLLog(@"AdCellListHandler", @"Handled reload - %@",self);
 }
 
 /***********************************
@@ -357,9 +361,16 @@ Public Methods
 {
 	NSMutableString* description = [NSMutableString string];
 
+	/* 
+	 * Peform an update to be sure the reported number of interactions
+	 * is corrects for the set cutoff and nonbonded pairs.
+	 * This will have no effect if the list hasnt been created yet
+	*/
+	
 	description = [NSMutableString stringWithString: @"List handler type: Cell. "];
-	if(system != nil)
+	if(system != nil && listCreated)
 	{
+		[self update];
 		[description appendFormat: @"System: %@. Cutoff: %5.2lf. Number interactions: %d\n",
 			[system systemName], cutoff, [nonbondedList listCount]];
 		[description appendFormat: @"\tSpace dimensions: (%10.5lf, %10.5lf, %10.5lf)\n",	
@@ -553,6 +564,8 @@ Update and related functions
 		else
 			list_p = list_p->next;
 	}	
+	
+	NSDebugLLog(@"AdCellListHandler", @"Removal complete. Now %d pairs", [nonbondedList listCount]);
 }
 
 /*
@@ -582,8 +595,9 @@ The update algorithm
 	if(!listCreated)
 		return;
 
+	NSDebugLLog(@"AdCellListHandler", @"  ");
 	NSDebugLLog(@"AdCellListHandler", 
-		@"\nUpdating Lists for system %@ - Currently %d pairs", 
+		@"Updating Lists for system %@ - Currently %d pairs", 
 		[system systemName],
 		[nonbondedList listCount]);
 	
@@ -956,6 +970,15 @@ Accessors
 
 	cutoff = aValue;
 	[self _initialiseDependants];
+	
+	//If the list has been created and the cutoff is changed
+	//then the cell space has to be recreated.
+	if(listCreated)
+	{
+		[self clearCellMatrices];
+		[self initialiseCells];
+		[self _assignCellIndexes];
+	}
 }
 
 - (double) cutoff
