@@ -557,7 +557,8 @@ groupProperties matrices
 	[solventGrid release];
 	[dataSource release];
 	[systemName release];
-	gsl_rng_free(twister);
+	if(twister != NULL)
+		gsl_rng_free(twister);
 	
 	[super dealloc];
 }
@@ -604,7 +605,7 @@ groupProperties matrices
 	
 	total = 0;
 	pairsEnum = [nonbondedPairs objectEnumerator];
-	while(set = [pairsEnum nextObject])
+	while((set = [pairsEnum nextObject]))
 		total += [set count];
 	
 	[description appendFormat: @"\nThere are %d nonbonded pairs\n", total];
@@ -870,11 +871,13 @@ groupProperties matrices
 	
 	coordinates = [AdDataMatrix matrixFromADMatrix: [system coordinates]];
 
-	//FIXME: Taking advantage of the fact that
-	//AdMoleculeCavity does not work properly. 
+	//The cavity surface is defined by the vdw radius of each atom.
+	//However the vdw radius of the solvent atoms must also be taken
+	//into account. Hence we place atoms at least 2*vdw radii away.
+	//There are obviously more accurate ways to do this.
 	cavity = [[AdMoleculeCavity alloc]
 			initWithSystem: system 
-			factor: 1.0];
+			factor: 2.0];
 
 	[cavity autorelease];
 	numberRemovedMolecules = [self setExclusionArea: cavity];
