@@ -59,6 +59,48 @@ static NSArray* integerEncodings;
 
 @implementation NSArray (AdKernelAdditions)
 
++ (id) arrayFromCIntArray: (int*) buffer ofLength: (int) length
+{
+	int i;
+	id array, finalArray;
+	
+	array = [NSMutableArray new];
+	for(i=0; i<length; i++)
+		[array addObject: [NSNumber numberWithInt: buffer[i]]];
+	
+	//If this message was sent to NSArray return a non-mutable array	
+	if(![self isSubclassOfClass: [NSMutableArray class]])
+	{
+		finalArray = [array copy];
+		[array release];
+	}
+	else
+		finalArray = array;
+	
+	return [finalArray autorelease];	
+}
+
++ (id) arrayFromCDoubleArray: (double*) buffer ofLength: (int) length
+{
+	int i;
+	id array, finalArray;
+	
+	array = [NSMutableArray new];
+	for(i=0; i<length; i++)
+		[array addObject: [NSNumber numberWithDouble: buffer[i]]];
+		
+	//If this message was sent to NSArray return a non-mutable array	
+	if(![self isSubclassOfClass: [NSMutableArray class]])
+	{
+		finalArray = [array copy];
+		[array release];
+	}
+	else
+		finalArray = array;
+		
+	return [finalArray autorelease];	
+}
+
 - (NSArray*) subarrayFromElementSelection: (NSIndexSet*) indexSet
 {
 	int index, i;
@@ -94,6 +136,68 @@ static NSArray* integerEncodings;
 	return [arrayCopy autorelease];
 }
 
+- (int) countForObject: (id) object
+{
+	int count = 0;
+	NSEnumerator* arrayEnum;
+	id element;
+	
+	arrayEnum = [self objectEnumerator];
+	while((element = [arrayEnum nextObject]))
+		if([element isEqual: object])
+			count ++;
+	
+	return count;		
+}
+
+-(double*) cDoubleRepresentation;
+{
+	int i = 0;
+	double value;
+	double *cArray;
+	NSEnumerator* arrayEnum;
+	id object;
+	
+	cArray = malloc([self count]*sizeof(double));
+	arrayEnum = [self objectEnumerator];
+	
+	while((object = [arrayEnum nextObject]))
+	{
+		value = 0;
+		if([object respondsToSelector: @selector(doubleValue)])
+			value = [object doubleValue];
+		
+		cArray[i] = value;	
+		i++;
+	}
+	
+	return cArray;
+}
+
+-(gsl_vector*) gslVectorRepresentation;
+{
+	int i = 0;
+	double value;
+	gsl_vector* vector;
+	NSEnumerator* arrayEnum;
+	id object;
+	
+	vector = gsl_vector_alloc([self count]);
+	arrayEnum = [self objectEnumerator];
+	
+	while((object = [arrayEnum nextObject]))
+	{
+		value = 0;
+		if([object respondsToSelector: @selector(doubleValue)])
+			value = [object doubleValue];
+		
+		gsl_vector_set(vector, i, value);	
+		i++;
+	}
+	
+	return vector;
+}
+
 @end
 
 /*
@@ -199,6 +303,7 @@ to stringValue, intValue and doubleValue */
 	return object;	
 }
 
+//Note - This can only be called once
 - (void) setDataTypesToTypesInArray: (NSArray*) array
 {
 	NSMutableArray* typeArray = [NSMutableArray new];
@@ -592,7 +697,8 @@ The AdDataMatrix class implementation
 					[anArray count], numberOfColumns];
 			}
 			else
-				columnHeaders = [anArray mutableCopy];
+				[columnHeaders addObjectsFromArray: anArray];
+				
 		}
 		else
 		{
@@ -642,6 +748,28 @@ The AdDataMatrix class implementation
 	[super dealloc];
 }
 
+- (id) valueForKey: (NSString*) key
+{
+	int index;
+	id value;
+	
+	NSDebugLLog(@"AdDataMatrix", @"Request for key %@", key);
+	NSDebugLLog(@"AdDataMatrix", @"Receiver class - %@", [self class]);
+	
+	//Check if its a request for a column
+	index = [self indexOfColumnWithHeader: key];
+	if(index != NSNotFound)
+	{
+		value = [self column: index];
+	}
+	else
+	{
+		value = [super valueForKey: key];
+	}
+	
+	return [[value retain] autorelease];
+}
+
 - (NSString*) stringRepresentation
 {
 	NSMutableString* string = [NSMutableString string];
@@ -654,6 +782,7 @@ The AdDataMatrix class implementation
 	
 	[string appendString: @"\n"];
 	
+	//Simplify by using componentsJoinedByString.
 	rowEnum = [self rowEnumerator];
 	while((row = [rowEnum nextObject]))
 	{
@@ -668,7 +797,6 @@ The AdDataMatrix class implementation
 
 - (NSString*) description
 {
-	int i;
 	unsigned int width;
 	NSMutableString* string = [NSMutableString string];
 	NSEnumerator* headerEnum;
@@ -681,24 +809,32 @@ The AdDataMatrix class implementation
 			width = [header length];
 
 	[string appendFormat: @"%@ - ", name];
-	[string appendFormat: @"Dimensions - %d x %d\n\n", 
+	[string appendFormat: @"Dimensions - %d x %d\n", 
 		numberOfRows, numberOfColumns];
-	[string appendString: @"Column Headers:\n\t"];
-	headerEnum = [columnHeaders objectEnumerator];
-	i = 0;
-	while((header = [headerEnum nextObject]))
-	{
-		i++;
-		[string appendFormat: @"%-*@", width + 4, header];
-		if(i%5 == 0)
-			[string appendString: @"\n\t"];
-	}	
-	
-	[string appendString: @"\n"];
 
 	return string;
 }
 
+- (BOOL) compareCRepresentations: (AdDataMatrix*) aMatrix
+{
+	BOOL retval = NO;
+	AdMatrix* matrixOne, *matrixTwo;
+	
+	matrixOne = [self cRepresentation];
+	matrixTwo = [aMatrix cRepresentation];
+	
+	//The function returns C99 type bool
+	if(AdCompareDoubleMatrices(matrixOne, matrixTwo, 1E-12) == true)
+	{
+		retval = YES;
+	}
+	
+	[[AdMemoryManager appMemoryManager] freeMatrix: matrixOne];
+	[[AdMemoryManager appMemoryManager] freeMatrix: matrixTwo];
+	
+	return retval;
+}
+
 /*
  * Accessing Elements
 */
@@ -862,6 +998,11 @@ The AdDataMatrix class implementation
 	return [[columnHeaders copy] autorelease];
 }
 
+- (id) headerForColumn: (unsigned int) index
+{
+	return [columnHeaders objectAtIndex: index];
+}
+
 - (unsigned int) indexOfColumnWithHeader: (NSString*) header
 {
 	return [columnHeaders indexOfObject: header];
@@ -1913,48 +2054,61 @@ Mutable subclass of AdDataMatrix
 
 	//if there are any previous columns check that this one is
 	//the right length
-
 	if(numberOfColumns != 0 && numberOfRows != 0)
 		if([anArray count] != numberOfRows)
 			[NSException raise: NSInvalidArgumentException
 				format: @"Incorrect number of elements in array (%d, %d)",
 				[anArray count], numberOfRows];	
-	
-	initialObjectClass = NSStringFromClass([[anArray objectAtIndex: 0] class]);
-
-	if((class = [self allowedSuperClassForObject: [anArray objectAtIndex: 0]]) != nil)
+				
+	//In the case where there are no rows but the column data types have been set
+	//this method can't be used to add an extra column.
+	//This is because we can't determine the data type for this column.
+	if((numberOfRows == 0) && ([columnDataTypes count] != 0))
 	{
-		dataType = [classToDataTypeMap objectForKey:
-				NSStringFromClass(class)];
+		[NSException raise: NSInvalidArgumentException
+			format: @"You cannot add an extra column where there are no rows and the column data types are set"];
 	}
-	else
-	{
-		dataType = [classToDataTypeMap objectForKey: initialObjectClass];
-		if(dataType == nil)
-			[NSException raise: NSInvalidArgumentException
-				format: @"AdDataMatrix - Cannot insert objects of class %@",
-				initialObjectClass];
-	}			
-
-	[columnDataTypes addObject: dataType];
-
-	//Convert the rest of the array to the correct type.
-	//We could add each element using extendMatrixWithRow:
-	//however we want to check if all the objects in anArray are
-	//convertible first.
 
 	convertedArray = [NSMutableArray array];
-	[convertedArray addObject: [anArray objectAtIndex: 0]];
-	for(i=1; i<(int)[anArray count]; i++)
+	if([anArray count] != 0)
 	{
-		element = [self representationOfObject: [anArray objectAtIndex: i]
-				forColumn: numberOfColumns];
-		[convertedArray addObject: element];		
-	}	
-
+		//If there are elements in the array the first one sets the data type
+		initialObjectClass = NSStringFromClass([[anArray objectAtIndex: 0] class]);
+		
+		if((class = [self allowedSuperClassForObject: [anArray objectAtIndex: 0]]) != nil)
+		{
+			dataType = [classToDataTypeMap objectForKey:
+				    NSStringFromClass(class)];
+		}
+		else
+		{
+			dataType = [classToDataTypeMap objectForKey: initialObjectClass];
+			if(dataType == nil)
+				[NSException raise: NSInvalidArgumentException
+					    format: @"AdDataMatrix - Cannot insert objects of class %@",
+				 initialObjectClass];
+		}	
+	
+		[columnDataTypes addObject: dataType];
+	
+		//Convert the rest of the array to the correct type.
+		//We could add each element using extendMatrixWithRow:
+		//however we want to check if all the objects in anArray are
+		//convertible first.
+	
+		[convertedArray addObject: [anArray objectAtIndex: 0]];
+		for(i=1; i<(int)[anArray count]; i++)
+		{
+			//Use numberOfColumns since we added an object to columnDataTypes
+			//but haven't incremented numberOfColumns yet (done at end)
+			element = [self representationOfObject: [anArray objectAtIndex: i]
+						     forColumn: numberOfColumns];
+			[convertedArray addObject: element];		
+		}
+	}
+		
 	//If there are no rows we have to create
 	//one for each element we are going to add.
-
 	if(numberOfRows == 0)
 		for(i=0; i<(int)[convertedArray count]; i++)
 		{		
@@ -1973,6 +2127,38 @@ Mutable subclass of AdDataMatrix
 	numberOfColumns++; 
 }
 
+- (void) extendMatrixWithColumnValues: (NSDictionary*) values
+{
+	unsigned int i, index;
+	NSMutableArray *row;
+	NSEnumerator* keyEnum;
+	id key;
+
+	//Create a row of zeros
+	row = [NSMutableArray new];
+	for(i=0; i<numberOfColumns; i++)
+		[row addObject: [NSNumber numberWithDouble: 0]];
+		
+	//For each key in the dictionary find the index of the corresponding column.
+	//Then replace the object at that index in row with the value of the key
+	keyEnum = [values keyEnumerator];	
+	while(key = [keyEnum nextObject])
+	{		  	
+		index = [self indexOfColumnWithHeader: key];
+		if(index != NSNotFound)
+		{
+			[row replaceObjectAtIndex: index 
+				withObject: [values objectForKey: key]];
+		}
+		else
+			NSWarnLog(@"No column header matches key %@. Ignoring", key);
+	}
+	
+	//Add the row
+	[self extendMatrixWithRow: row];
+	[row release];
+}
+
 - (void) extendMatrixWithMatrix: (AdDataMatrix*) extendingMatrix
 {
 	NSEnumerator* rowEnum;
