@@ -441,7 +441,7 @@ raised.
 	
 	total = 0;
 	pairsEnum = [nonbondedPairs objectEnumerator];
-	while(set = [pairsEnum nextObject])
+	while((set = [pairsEnum nextObject]))
 		total += [set count];
 
 	[description appendFormat: @"\nThere are %d nonbonded pairs\n", total];
@@ -1067,18 +1067,22 @@ by one.
 This ensures that each index in the group matrix refers
 to the correct row in the properties and configuration
 matrices after an element is removed from them.
+\todo Use int matrix?
 */
 - (void) _reindexElementsFrom: (unsigned int) elementIndex
 	inGroupMatrixOfInteraction: (NSString*) interaction	
 {
-	int i,j;
+	NSAutoreleasePool* pool = [NSAutoreleasePool new];
+	int i,j, numberOfRows, numberOfColumns;
 	unsigned int value;
 	AdMutableDataMatrix *groupMatrix;
 
 	groupMatrix = [interactionGroups objectForKey: interaction];
-	for(i=0; i<(int)[groupMatrix numberOfRows]; i++)
+	numberOfRows = [groupMatrix numberOfRows];
+	numberOfColumns = [groupMatrix numberOfColumns];
+	for(i=0; i< numberOfRows; i++)
 	{
-		for(j=0; j< (int)[groupMatrix numberOfColumns]; j++)
+		for(j=0; j< numberOfColumns; j++)
 		{
 			value = [[groupMatrix elementAtRow: i
 		 			column: j] intValue];
@@ -1091,6 +1095,8 @@ matrices after an element is removed from them.
 			}
 		}
 	}
+	
+	[pool release];
 }
 
 /**
@@ -1111,6 +1117,8 @@ updated to use such index arrays more generaly.
 	
 	newArray = [NSMutableArray new];
 	indexEnum = [indexArray objectEnumerator];
+	//Switch to iterating over the indexSet as its
+	//possibly quicker than all the copying.
 	while((indexSet = [indexEnum nextObject]))
 	{
 		if([indexSet lastIndex] >= elementIndex)
