@@ -188,9 +188,9 @@ static NSArray* knownTypes;
 	return [self initWithSystem: nil];
 }
 
-- (id) initWithSystem: (id) system
+- (id) initWithSystem: (id) aSystem
 {
-	return [self initWithSystem: nil
+	return [self initWithSystem: aSystem
 		cutoff: 12.0
 		updateInterval: 20
 		epsilonOne: 1.0
@@ -797,23 +797,28 @@ Return YES if the term can calculate forces.
 	return [[pairs retain] autorelease];
 }
 
-- (double) espilonOne
+- (double) epsilonOne
 {
 	return epsilon1;
 }
 
-- (void) setEspilonOne: (double) value
+- (void) setEpsilonOne: (double) value
 {
 	epsilon1 = value;
 	[self _calculateGRFParameters];
 }
 
-- (double) espilonTwo
+- (double) epsilonTwo
 {
 	return epsilon2;
 }
 
-- (void) setEspilonTwo: (double) value
+- (double) permittivity
+{
+	return [self epsilonOne];
+}
+
+- (void) setEpsilonTwo: (double) value
 {
 	epsilon2 = value;
 	[self _calculateGRFParameters];
@@ -830,4 +835,27 @@ Return YES if the term can calculate forces.
 	[self _calculateGRFParameters];
 }
 
+/**
+ Returns a pointer to the beginning of the list of nonbonded interaction pairs the receiver uses.
+ See interface definition for more.
+ */
+- (ListElement*) interactionList
+{
+	return interactionList;
+}
+
+- (id) copyWithZone:(NSZone *)aZone
+{
+	return [[[self class] alloc]
+		initWithSystem: system
+			cutoff: cutoff
+		updateInterval: updateInterval
+		    epsilonOne: epsilon1
+		    epsilonTwo: epsilon2
+			 kappa: kappa
+		nonbondedPairs: nil
+	   externalForceMatrix: NULL
+	      listHandlerClass: listHandlerClass];
+}
+
 @end
