@@ -27,18 +27,6 @@
 
 static NSArray* forceFields;
 
-/**
-Private methods for maintaining state
-*/
-@interface AdMolecularMechanicsForceField (PrivateInternals)
-- (void) _systemCleanUp;
-- (void) _updateAccelerations;
-- (void) _initialisationForSystem;
-- (void) _createReciprocalMassArray;
-- (void) _handleSystemContentsChange: (NSNotification*) aNotification;
-- (AdMatrix*) _internalDataStorageForGroups: (AdDataMatrix*) groups parameters: (AdDataMatrix*) parameters;
-@end
-
 BOOL forceFieldDebug = NO;
 
 @implementation AdMolecularMechanicsForceField
@@ -47,6 +35,7 @@ BOOL forceFieldDebug = NO;
 {
 	static BOOL didInitialize = NO;
 	BOOL diagnoseBondedFunctions, diagnoseNonbondedFunctions, diagnoseForceFieldFunctions;
+	BOOL checkForceMagnitudes;
 	NSUserDefaults* userDefaults = [NSUserDefaults standardUserDefaults];
 
 	if(didInitialize)
@@ -100,6 +89,14 @@ BOOL forceFieldDebug = NO;
 		__HarmonicImproperTorsionEnergyDebug__ = true;
 		__HarmonicImproperTorsionForceDebug__ = true;
 		forceFieldDebug = YES;
+		AdHarmonicBondDebugInfo();
+		AdHarmonicAngleDebugInfo();
+		AdFourierTorsionDebugInfo();
+		AdHarmonicImproperTorsionDebugInfo();
+		
+		fflush(stderr);
+		
+		NSLog(@"Force debug lines contain the force magnitude as a final term\n");
 	}
 	
 	if(diagnoseNonbondedFunctions)
@@ -112,8 +109,20 @@ BOOL forceFieldDebug = NO;
 		__GRFNonbondedForceDebug__ = true;
 		__GRFNonbondedEnergyDebug__ = true;
 		forceFieldDebug = YES;
+		
+		AdNonbondedDebugInfo();
+		fflush(stderr);
 	}
 
+	checkForceMagnitudes = [userDefaults boolForKey: @"CheckForceMagnitudes"];
+	if(checkForceMagnitudes)
+	{	
+		NSLog(@"Force magnitude checking turned on.");
+		NSLog(@"All force mangitudes will be checked for inifities and Nan's");
+		__CheckForceMagnitude__ = true;
+		forceFieldDebug = YES;
+	}
+	
 	didInitialize = YES;
 }
 
@@ -135,6 +144,24 @@ BOOL forceFieldDebug = NO;
 	return nil;
 }
 
++ (id) forceFieldForSystem: (AdSystem*) system
+{
+	Class forceFieldClass;
+	NSString* forceFieldName;
+	AdDataSource* dataSource;
+
+	dataSource = [system dataSource];
+	forceFieldName = [dataSource valueForMetadataKey: @"ForceField"];
+	if(forceFieldName == nil)
+		return nil;
+	
+	forceFieldClass = [self classForForceField: forceFieldName];
+	if(forceFieldClass == nil)
+		return nil;
+		
+	return [[[forceFieldClass alloc] initWithSystem: system] autorelease];
+}
+
 - (id) init
 {
 	return [self initWithSystem: nil];
@@ -177,6 +204,7 @@ BOOL forceFieldDebug = NO;
 	[state release];
 	[availableTerms release];
 	[customTerms release];
+	[customTermNames release];
 	[nonbondedTerm release];
 	[vdwInteractionType release];
 	[memoryManager freeMatrix: bonds];
@@ -216,14 +244,48 @@ BOOL forceFieldDebug = NO;
 		fflush(stderr);
 }
 
+- (AdMatrix*) evaluateFiniteDifferenceForcesForTerm: (NSString*) term
+{
+	AdMatrix* coordinates;
+	
+	//Deactivate all energy terms
+	
+	//Activate the requested term only
+	
+	//do finite difference
+	/*coordinates = [[self system] coordinates];
+	for(i=0; i<coordinates->no_rows; i++)
+	{
+		for(j=0;j<coordinates->no_columns; j++)
+		{
+			//Perturb-atom position
+			[system object: self willBeginWritingToMatrix: 
+		}
+	}*/
+	
+}
+
 //custom term methods
 
 - (void) addCustomTerm: (id) object withName: (NSString*) name
 {
 	if([object conformsToProtocol: @protocol(AdForceFieldTerm)])
 	{
+		if([customTerms objectForKey: name] != nil)
+		{
+			[NSException raise: NSInvalidArgumentException
+				format: @"A custom term called %@ already exists." 
+				" Remove it before adding a new term with the same name"];
+		}
+		
+		//Custom terms is a dictionry (hash). 
+		//This means the keys are not ordered which would lead to a problem
+		//for the allTerms/allEnergies methods - they would not return terms in 
+		//the same order each time.
+		//In order to overcome this we have to keep an additional customTermName array.
+		
 		[customTerms setObject: object forKey: name];
-		//FIXME: Deal with the case where name already exists.
+		[customTermNames addObject: name];
 		[availableTerms addObject: name];
 		[[state valueForKey: @"CustomTerms"] 
 			setObject: [NSNumber numberWithInt: 0]
@@ -249,11 +311,16 @@ BOOL forceFieldDebug = NO;
 {
 	NSEnumerator* keyEnum;
 	NSString* termName;
+	NSMutableArray* oldNames;
 	NSMutableDictionary* oldTerms;
 
 	//Remove all previous terms	
 	oldTerms = [[customTerms mutableCopy] autorelease];
+	oldNames = [[customTermNames mutableCopy] autorelease];
+	
 	[customTerms removeAllObjects];
+	[customTermNames removeAllObjects];
+	
 	keyEnum = [aDict keyEnumerator];
 	while((termName = [keyEnum nextObject]))
 	{
@@ -265,8 +332,10 @@ BOOL forceFieldDebug = NO;
 		NS_HANDLER
 		{	
 			[customTerms removeAllObjects];
+			[customTermNames removeAllObjects];
 			//Restore the old terms
 			customTerms = [oldTerms retain];
+			customTermNames = [oldNames retain];
 			[localException raise];
 		}
 		NS_ENDHANDLER
@@ -276,6 +345,7 @@ BOOL forceFieldDebug = NO;
 - (void) removeCustomTermWithName: (NSString*) name
 {
 	[customTerms removeObjectForKey: name];
+	[customTermNames removeObject: name];
 	[[state valueForKey: @"CustomTerms"]
 		removeObjectForKey: name];
 	[availableTerms removeObject: name];	
@@ -401,10 +471,13 @@ Accessors
 
 - (NSArray*) arrayOfEnergiesForTerms: (NSArray*) terms notFoundMarker: (id) anObject
 {
+	int index, count = 0;
 	double value;
 	NSArray* potentials;
 	NSMutableArray* array = [NSMutableArray array];
-	NSEnumerator* potentialsEnum;
+	NSMutableArray* notFound = [NSMutableArray array];
+	NSEnumerator* potentialsEnum, *notFoundEnum;
+	NSString* termName;
 	id object;
 
 	potentials = [[state valueForKey: @"TermPotentials"]
@@ -418,11 +491,28 @@ Accessors
 			[array addObject: [NSNumber numberWithDouble: value]];
 		}
 		else
+		{
 			[array addObject: anObject];
+			[notFound addObject: [terms objectAtIndex: count]];
+		}
+		count++;
 	}		
 
-	//FIXME: Have to search custom terms for any value not
-	//found above
+	//Check if any of the terms not found above are custom terms.
+	notFoundEnum = [notFound objectEnumerator];
+	while(termName = [notFoundEnum nextObject])
+	{
+		if([customTermNames containsObject: termName])
+		{
+			object = [customTerms objectForKey: termName];
+			if([object canEvaluateEnergy])
+			{
+				index = [terms indexOfObject: termName];
+				[array replaceObjectAtIndex: index 
+						 withObject: [NSNumber numberWithDouble: [object energy]]];
+			}
+		}
+	}
 
 	return array;
 }
@@ -436,6 +526,50 @@ Accessors
 		forKeys: array];
 }
 
+
+//FIMXE: Their is an ambiguity in the meaning of 'term'.
+//Its refering both to components AND energies.
+//For example Nonbonded is listed as an availableTerm.
+//but coreTerms returns both vdw and electrostatic contributions - nonbonded is not present
+
+- (NSArray*) allTerms
+{
+	NSMutableArray* array = [NSMutableArray array];
+	NSEnumerator* nameEnum;
+	NSString* termName;
+	id object;
+	
+	[array addObjectsFromArray: [self coreTerms]];
+	nameEnum = [customTermNames objectEnumerator];
+	while(termName = [nameEnum nextObject])
+	{
+		object = [customTerms objectForKey: termName];
+		if([object canEvaluateEnergy])
+			[array addObject: termName];
+	}
+	
+	return [[array copy] autorelease];
+}
+
+- (NSArray*) allEnergies
+{
+	NSMutableArray* array = [NSMutableArray array];
+	NSEnumerator* nameEnum;
+	NSString* termName;
+	id object;
+	
+	[array addObjectsFromArray: [self arrayOfCoreTermEnergies]];
+	nameEnum = [customTermNames objectEnumerator];
+	while(termName = [nameEnum nextObject])
+	{
+		object = [customTerms objectForKey: termName];
+		if([object canEvaluateEnergy])
+			[array addObject: [NSNumber numberWithDouble: [object energy]]];
+	}
+	
+	return [[array copy] autorelease];
+}
+
 - (NSArray*) coreTerms
 {
 	return [[coreTerms retain] autorelease];
@@ -534,7 +668,7 @@ Accessors
 	NSDictionary* energies;
 	NSArray* array;
 	NSEnumerator* termEnum;
-	id term;
+	id term, object;
 	
 	description = [NSMutableString string];
 	[description appendFormat: @"ForceField: %@\n", [state objectForKey: @"ForceField"]];
@@ -576,6 +710,15 @@ Accessors
 		while(term = [termEnum nextObject])
 			[description appendFormat: @"\t%25@: %10.5lf Sim Units\n", 
 				term, [[energies objectForKey: term] doubleValue]];
+				
+		termEnum = [customTerms keyEnumerator];
+		while(term = [termEnum nextObject])
+		{		
+			object = [customTerms objectForKey: term];
+			if([object canEvaluateEnergy])
+				[description appendFormat: @"\t%25@: %10.5lf Sim Units\n", 
+					term, [object energy]];
+		}
 	}
 		
 	return description;
