@@ -25,21 +25,8 @@
 
 static NSArray* knownTypes;
 
-
 @implementation AdPureNonbondedTerm
 
-/*
-Temporary test method - To be moved.
-
-- (void) checkMultiProcessor
-{
-	numberProcessors = sysconf(_SC_NPROCESSORS_CONF);  
-	NSLog(@"There are %d processors", sysNumProcs);
-
-	//Should also check which are available etc.
-	
-}*/
-
 - (BOOL) _checkMatrix: (AdDataMatrix*) matrix containsParametersForType: (NSString*) type
 {
 	NSArray* headers;
@@ -187,9 +174,9 @@ Temporary test method - To be moved.
 	return [self initWithSystem: nil];
 }
 
-- (id) initWithSystem: (id) system
+- (id) initWithSystem: (id) aSystem
 {
-	return [self initWithSystem: nil
+	return [self initWithSystem: aSystem
 		cutoff: 12.0
 		updateInterval: 20
 		permittivity: 1.0
@@ -236,7 +223,7 @@ Temporary test method - To be moved.
 		memoryManager = [AdMemoryManager appMemoryManager];
 		permittivity = permittivityValue;
 		cutoff = aDouble;
-		buffer = 1.0;
+		buffer = 1.5;
 		updateInterval = anInt;
 		
 		if(aClass ==  nil)
@@ -627,6 +614,30 @@ Temporary test method - To be moved.
 			to: anInt];
 }
 
+- (void) setAutoUpdateList: (BOOL) value
+{
+	if((value == YES) && (messageId == nil))
+	{
+		NSLog(@"Turning list auto-update ON");
+		messageId = [[NSProcessInfo processInfo]
+			     globallyUniqueString];
+		[messageId retain];
+		[[AdMainLoopTimer mainLoopTimer] 
+			sendMessage: @selector(update)
+			toObject: listHandler
+			interval: updateInterval
+			name: messageId];
+	}
+	else if((value == NO) && (messageId != nil))
+	{
+		NSLog(@"Turning list auto-update OFF");
+		[[AdMainLoopTimer mainLoopTimer]
+			removeMessageWithName: messageId];
+		[messageId release];
+		messageId = nil;
+	}
+}
+
 - (void) updateList: (BOOL) reset
 {
 	[listHandler update];
@@ -788,5 +799,26 @@ Return YES if the term can calculate forces.
 	return [[pairs retain] autorelease];
 }
 
+/**
+ Returns a pointer to the beginning of the list of nonbonded interaction pairs the receiver uses.
+ See interface definition for more.
+ */
+- (ListElement*) interactionList
+{
+	return interactionList;
+}
+
+- (id) copyWithZone: (NSZone*) aZone
+{
+	return [[[self class] alloc]
+		initWithSystem: system
+			cutoff: cutoff
+		updateInterval: updateInterval
+		  permittivity: permittivity
+		nonbondedPairs: nil
+	   externalForceMatrix: NULL
+	      listHandlerClass: listHandlerClass];
+}
+
 @end
 
