@@ -264,6 +264,23 @@ Object Creation
 
 ****************/
 
++ (id) alloc
+{
+	AdSCAAS* object;
+
+	//Need to set twister = NULL since it seems
+	//that gsl_rng_free can't handle a junk pointer and will
+	//cause the program to crash
+	//This will happen if this class is allocated and released
+	//without being initialised.
+	//FIXME: Make twister a class variable to avoid this?
+	
+	object = [super alloc];
+	object->twister = NULL;
+	
+	return object;
+}
+
 - (id) init
 {
 	return [self initWithSystem: nil];
@@ -295,6 +312,7 @@ Object Creation
 		sphereRadius = inner_sphere = 0;
 		containedSystems = nil;
 		memoryManager = [AdMemoryManager appMemoryManager];
+		twister = NULL;
 
 		if(depth <= 0)
 		{
@@ -323,7 +341,9 @@ Object Creation
 	[memoryManager freeArray: solventMasses];
 	[memoryManager freeArray: solventCharges];
 	free(radial_distance);
-	gsl_rng_free(twister);
+	if(twister != NULL)
+		gsl_rng_free(twister);
+		
 	[super dealloc];
 }
 
@@ -340,7 +360,7 @@ Object Creation
 	{	
 		[description appendString: @"\tContained systems: "];
 		systemEnum = [containedSystems objectEnumerator];
-		while(sys = [systemEnum nextObject])
+		while((sys = [systemEnum nextObject]))
 			[description appendFormat: @"%@, ", [sys systemName]];
 			
 		[description appendFormat: @"\n\tSolute charge: %8.4lf\n", solute_charge];	
@@ -684,6 +704,9 @@ Public Methods
 	AdSystem* containedSystem;
 	NSNotificationCenter* notificationCenter = [NSNotificationCenter defaultCenter];
 	
+	if(containedSystems == nil)
+		return;
+	
 	containedSystemsEnum = [containedSystems objectEnumerator];
 	while((containedSystem = [containedSystemsEnum nextObject]))
 		[notificationCenter removeObserver: self
