@@ -177,9 +177,9 @@ Initialises the term to operate on \e system
 	return [self initWithSystem: nil];
 }
 
-- (id) initWithSystem: (id) system
+- (id) initWithSystem: (id) aSystem
 {
-	return [self initWithSystem: nil
+	return [self initWithSystem: aSystem
 		cutoff: 12.0
 		updateInterval: 20
 		permittivity: 1.0
@@ -782,4 +782,25 @@ Return YES if the term can calculate forces.
 	return [[pairs retain] autorelease];
 }
 
+/**
+ Returns a pointer to the beginning of the list of nonbonded interaction pairs the receiver uses.
+ See interface definition for more.
+ */
+- (ListElement*) interactionList
+{
+	return interactionList;
+}
+
+- (id) copyWithZone:(NSZone *)aZone
+{
+	return [[[self class] alloc]
+		initWithSystem: system
+			cutoff: cutoff
+		updateInterval: updateInterval
+		  permittivity: permittivity
+		nonbondedPairs: nil
+	   externalForceMatrix: NULL
+	      listHandlerClass: listHandlerClass];
+}
+
 @end
