@@ -1,5 +1,5 @@
 /*
-   Project: UL
+   Project: AdunKernel
 
    Copyright (C) 2005 Michael Johnston & Jordi Villa-Freixa
 
@@ -226,6 +226,11 @@ returned structure as is responsible for deallocating it
 */
 - (AdMatrix*) cRepresentation;
 /**
+Compares the c representation of the receiver with that of \e aMatrix
+using AdCompareDoubleMatrices() with a tolerance of 1E-12.
+*/
+- (BOOL) compareCRepresentations: (AdDataMatrix*) aMatrix;
+/**
 Places the matrix contents in the buffer \e aMatrix.
 \e aMatrix must have the correct dimensions. If not an 
 NSInvalidArgumentException is raised.
@@ -272,6 +277,12 @@ then an array containing the default titles is returned i.e.
 */
 - (NSArray*) columnHeaders;
 /**
+Returns the header for column \e index.
+Raises an NSRangeException if \e index is greater than
+or equal to the number of columns in the receiver.
+*/
+- (id) headerForColumn: (unsigned int) index;
+/**
 If a column with \e header exists it returns
 the index of the first one found. Otherwise returns
 NSNotFound
@@ -479,12 +490,46 @@ the data type of the corresponding column.
 /**
 Adds \e column to the end of the matrix. All elements in
 the column must be of the same class which determines the dataType of
-the column.
+the column. 
 \param array The array to be appended 
-\note add exceptions
+
+The number of elements in the column must be equal to the number
+of rows in the matrix - if not an NSInvalidArgumentException is raised.
+
+If there are no rows in the matrix then passing an empty array will cause an empty
+extra column to be added.
+The only exception to this is if the column data types were bset on initialisation.
+In that case the data type for the new column can't be determined and this method raises an exception.
+The new column can be added once the first row of the matrix is present however.
 */
 - (void) extendMatrixWithColumn: (NSArray*) column;
 /**
+Adds a row using a dictionary of columnHeader:value pairs.
+That is, in the new row the value for each column will be the object in \e values
+whose key corresponds to that columns column header.
+ 
+Columns not specified in the dictionary (i.e. their column header is not a key)
+are given values of zero (or the equivalent).
+Keys which don't have corresponding headers (i.e. indexOfColumnWithHeader:() passing the key returns NSNotFound)
+are ignored.
+
+For example a matrix has 3 columns with the following headers
+('Col 1', 'Col 2' , Col 3')
+
+Create a dictionary with the value you want to insert for each column
+{'Col 1':'Milk', 'Col2':100}
+
+Passing this dictionary to this method adds a new row with the following entries
+
+('Milk', 0, '100)
+
+So it is equivalent to calling extendMatrixWithRow:() with this array.
+The advantage of this method is that it allows you to add values to a matrix using
+columnHeader strings.
+*/
+
+- (void) extendMatrixWithColumnValues: (NSDictionary*) values;
+/**
 Removes row number \e rowIndex from the matrix. 
 Raises an NSRangeException if \e rowIndex is outside the range of the rows in the reciever.
 */
@@ -530,6 +575,31 @@ by \e indexSet. If an index in \e indexSet exceeds the range of elements in
 the receiver an NSRangeException is raised.
 */
 - (NSArray*) subarrayFromElementSelection: (NSIndexSet*) indexSet;
+/**
+Creates an NSArray object from a C array of doubles of length \e length
+*/
++ (id) arrayFromCDoubleArray: (double*) array ofLength: (int) length;
+/**
+Creates an NSArray object from a C array of ints of length \e length
+*/
++ (id) arrayFromCIntArray: (int*) array ofLength: (int) length;
+/**
+ Returns a c double array of the contents of the receiver.
+ If an object responds to doubleValue, it is used to retreive the 
+ value put in the array.
+ Otherwise 0 is used.
+ The caller is owns the returned array and is responsible for freeing it.
+ */
+- (double*) cDoubleRepresentation;
+/**
+Returns the number of times \e object appears in the receiver
+*/
+- (int) countForObject: (id) object;
+/*
+Re*turns a gsl_vector struct containing the contents of the receiver.
+The same considerations as outlined in cDoubleRepresentation() apply here.
+*/
+- (gsl_vector*) gslVectorRepresentation;
 @end
 
 //For decoding old ULMatrix objects
