@@ -22,7 +22,6 @@
 #ifndef _ADMULTITHREADED_TERM
 #define _ADMULTITHREADED_TERM
 #include "AdunKernel/AdunNonbondedTerm.h"
-#include "AdunKernel/AdunPureNonbondedTerm.h"
 
 /**
 \ingroup Inter
@@ -31,10 +30,13 @@ Enables multithreading of a nonbonded term object.
 */
 @interface AdMultithreadedNonbondedTerm: AdNonbondedTerm
 {
-	id mainThread;
-	id threadedObj;
+	int numberOfProcessors;
+	id mainTerm;
+	id threadedTerms;
+	id allTerms;
 	id dividedPairs;
 	AdMatrix* forces;
+	id threadManager;
 }
 /**
 Returns a new AdMultithreadedNonbondedTerm instance that
