@@ -1,6 +1,6 @@
 #include <Base/AdBaseFunctions.h>
 
-inline double AdCalculateKineticEnergy(DoubleMatrix* velocities, double* masses)
+double AdCalculateKineticEnergy(DoubleMatrix* velocities, double* masses)
 {
 	register int i, j;
 	double *vhold;
