@@ -29,7 +29,16 @@ bool __ShiftedNonbondedForceDebug__ = false;
 bool __GRFNonbondedEnergyDebug__ = false;
 bool __GRFNonbondedForceDebug__ = false;
 
-inline void AdNonbondedEnergyLog(char* cutType, char* ljType, int a1, int a2, double ljA,
+void AdNonbondedDebugInfo(void)
+{
+	fprintf(stderr, "\nNonbonded debug on\n");
+	fprintf(stderr, "Energy Info - Calc Type, Lennard Jones Type, Atom one, Atom two, Parameter A");
+	fprintf(stderr, " Parameter B, Charge Product, Separation, Cumulative EST Potential, Cumulative LJ Potential\n");
+	fprintf(stderr, "Parameter A and B will depend on LJ Type\n");
+	fprintf(stderr, "Force Info - Same as energy plus force magnitude as last value\n\n");
+}
+
+void AdNonbondedEnergyLog(char* cutType, char* ljType, int a1, int a2, double ljA,
 		double ljB, double charge, double sep, double estPot, double ljPot, bool flag)
 {
 	if(flag)
@@ -37,15 +46,52 @@ inline void AdNonbondedEnergyLog(char* cutType, char* ljType, int a1, int a2, do
 			cutType, ljType, a1, a2, ljA, ljB, charge, sep, estPot, ljPot);
 }
 
-inline void AdNonbondedForceLog(char* cutType, char* ljType, int a1, int a2, double ljA,
+void AdNonbondedForceLog(char* cutType, char* ljType, int a1, int a2, double ljA,
 		double ljB, double charge, double sep, double estPot, double ljPot, double force, bool flag)
 {
+	if(__CheckForceMagnitude__)
+	{
+		if(isnan(force) || isinf(force))
+		{	
+			fprintf(stderr, "Detected invalid force\n");
+			AdNonbondedDebugInfo();
+			fprintf(stderr, "%-8s%-3s%-6d%-6d%-12.5lf%-12.5lf%-12.5lf%-12.5lf%-12.lf%-12.5lf%-12.5lf\n",
+				cutType, ljType, a1, a2, ljA, ljB, charge, sep, estPot, ljPot, force);	
+		}
+	}
+
 	if(flag)
 		fprintf(stderr, "%-8s%-3s%-6d%-6d%-12.5lf%-12.5lf%-12.5lf%-12.5lf%-12.lf%-12.5lf%-12.5lf\n",
 			cutType, ljType, a1, a2, ljA, ljB, charge, sep, estPot, ljPot, force);	
 }
 
-inline void AdCoulombAndLennardJonesAEnergy(ListElement* interaction, 
+double AdCoulombEnergy(double separation, double chargeOne, double chargeTwo, double relativePermittivity, double constant)
+{
+	return (1/relativePermittivity)*constant*chargeOne*chargeTwo/pow(separation, 2);
+}
+
+double AdLennardJonesAEnergy(double separation, double a1, double b1, double a2, double b2)
+{
+	double holder;
+
+	holder = pow((1/separation), 6);
+	
+	return (a1*a2)*pow(holder, 2) - (b1*b2)*holder;
+}
+
+double AdLennardJonesBEnergy(double separation, double wellDepth1, double equilibriumSeparation1, 
+					double wellDepth2, double equilibriumSeparation2)
+{
+	double holder, wellDepth, equilibriumSeparation;
+
+	wellDepth = sqrt(wellDepth1*wellDepth2);
+	equilibriumSeparation = equilibriumSeparation1 + equilibriumSeparation2;
+	holder = pow((equilibriumSeparation/separation), 6);
+	
+	return 4*wellDepth*(pow(holder, 2) - holder);
+}
+
+void AdCoulombAndLennardJonesAEnergy(ListElement* interaction, 
 		double** coordinates, 
 		double EPSILON_RP, 
 		double cutoff,
@@ -70,11 +116,12 @@ inline void AdCoulombAndLennardJonesAEnergy(ListElement* interaction,
 	
 	//calculate the interatomic distance
 	Ad3DVectorLength(&seperation_s);
-	if(seperation_s.length > cutoff)
-		return;
 	
 	//add this length to the linked list element the bond belongs to
 	interaction->length = seperation_s.length;
+	
+	if(seperation_s.length > cutoff)
+		return;
 
 	//get reciprocal of seperation
 	length_rec = 1/seperation_s.length;
@@ -88,13 +135,13 @@ inline void AdCoulombAndLennardJonesAEnergy(ListElement* interaction,
 	*vdw_pot += lennardJonesA - lennardJonesB;	
 
 #ifdef BASE_NONBONDED_DEBUG
-	AdNonbondedEnergyLog("Normal", "A", atomOne, atomTwo, lennardJonesA, lennardJonesB, 
-		chargeProduct, separation_s.length, est_hold, (lennardJonesA - lennardJonesB),
+	AdNonbondedEnergyLog("Normal", "A", atom_one, atom_two, lennardJonesA, lennardJonesB, 
+		chargeProduct, seperation_s.length, est_hold, (lennardJonesA - lennardJonesB),
 		__NonbondedEnergyDebug__); 
 #endif	
 }
 
-inline void AdCoulombAndLennardJonesAForce(ListElement* interaction, 
+void AdCoulombAndLennardJonesAForce(ListElement* interaction, 
 		double** coordinates, 
 		double** forces,
 		double EPSILON_RP, 
@@ -121,12 +168,13 @@ inline void AdCoulombAndLennardJonesAForce(ListElement* interaction,
 	
 	//calculate the interatomic distance
 	Ad3DVectorLength(&seperation_s);
-	if(seperation_s.length > cutoff)
-		return;
-	
-	//add this length to the linked list wlement the bond belongs to
+
+	//add this length to the linked list element the bond belongs to
 	interaction->length = seperation_s.length;
 
+	if(seperation_s.length > cutoff)
+		return;
+				
 	//get reciprocal of seperation
 	length_rec = 1/seperation_s.length;
 	
@@ -167,14 +215,14 @@ inline void AdCoulombAndLennardJonesAForce(ListElement* interaction,
 	forces[atom_two][2] -= *(seperation_s.vector + 2);
 	
 #ifdef BASE_NONBONDED_DEBUG
-	AdNonbondedForceLog("Normal", "A", atomOne, atomTwo, lennardJonesA, lennardJonesB, 
-			     chargeProduct, separation_s.length, est_hold,
-			    (lennardJonesA - lennardJonesB), forceMag,
+	AdNonbondedForceLog("Normal", "A", atom_one, atom_two, lennardJonesA, lennardJonesB, 
+			     chargeProduct, seperation_s.length, est_hold,
+			    (lennardJonesA - lennardJonesB), force_mag,
 			    __NonbondedForceDebug__); 
 #endif
 }
 
-inline void AdCoulombAndLennardJonesAForceTest(ListElement* interaction, 
+void AdCoulombAndLennardJonesAForceTest(ListElement* interaction, 
 		Vector3D* seperation_s, 
 		double** forces,
 		double EPSILON_RP, 
@@ -238,7 +286,7 @@ inline void AdCoulombAndLennardJonesAForceTest(ListElement* interaction,
 	forces[atom_two][2] -= *(seperation_s->vector + 2);
 }
 
-inline void AdShiftedCoulombAndLennardJonesAForce(ListElement* interaction, 
+void AdShiftedCoulombAndLennardJonesAForce(ListElement* interaction, 
 		double** coordinates, 
 		double** forces,
 		double EPSILON_RP, 
@@ -265,9 +313,12 @@ inline void AdShiftedCoulombAndLennardJonesAForce(ListElement* interaction,
 	*(seperation_s.vector + 2) = coordinates[atom_one][2] - coordinates[atom_two][2];
 	
 	Ad3DVectorLength(&seperation_s);
+	
+	//add this length to the linked list element the bond belongs to
+	interaction->length = seperation_s.length;
+	
 	if(seperation_s.length > cut)
 		return;
-	interaction->length = seperation_s.length;
 
 	length_rec = 1/seperation_s.length;
 	vdw_hold = pow(length_rec, 6);
@@ -305,14 +356,14 @@ inline void AdShiftedCoulombAndLennardJonesAForce(ListElement* interaction,
 	forces[atom_two][2] -= *(seperation_s.vector + 2);
 	
 #ifdef BASE_NONBONDED_DEBUG
-	AdNonbondedForceLog("Shifted", "A", atomOne, atomTwo, lennardJonesA, lennardJonesB, 
-			    chargeProduct, separation_s.length, est_hold,
-			    (lennardJonesA - lennardJonesB), forceMag,
+	AdNonbondedForceLog("Shifted", "A", atom_one, atom_two, lennardJonesA, lennardJonesB, 
+			    chargeProduct, seperation_s.length, est_hold,
+			    (lennardJonesA - lennardJonesB), force_mag,
 			    __ShiftedNonbondedForceDebug__); 
 #endif
 }
 
-inline void AdGRFCoulombAndLennardJonesAForce(ListElement* interaction, 
+void AdGRFCoulombAndLennardJonesAForce(ListElement* interaction, 
 		double** coordinates, 
 		double** forces,
 		double EPSILON_RP, 
@@ -339,9 +390,13 @@ inline void AdGRFCoulombAndLennardJonesAForce(ListElement* interaction,
 	*(seperation_s.vector + 2) = coordinates[atom_one][2] - coordinates[atom_two][2];
 
 	Ad3DVectorLength(&seperation_s);
+	
+	//add this length to the linked list element the bond belongs to
+	interaction->length = seperation_s.length;
+	
 	if(seperation_s.length > cutoff)
 		return;
-	interaction->length = seperation_s.length;
+		
 	length_rec = 1/seperation_s.length;
 	
 	vdw_hold = pow(length_rec, 6);
@@ -373,14 +428,14 @@ inline void AdGRFCoulombAndLennardJonesAForce(ListElement* interaction,
 	forces[atom_two][2] -= *(seperation_s.vector + 2);
 	
 #ifdef BASE_NONBONDED_DEBUG
-	AdNonbondedForceLog("GRF", "A", atomOne, atomTwo, lennardJonesA, lennardJonesB, 
-			    chargeProduct, separation_s.length, est_hold,
-			    (lennardJonesA - lennardJonesB), forceMag,
+	AdNonbondedForceLog("GRF", "A", atom_one, atom_two, lennardJonesA, lennardJonesB, 
+			    chargeProduct, seperation_s.length, est_hold,
+			    (lennardJonesA - lennardJonesB), force_mag,
 			    __GRFNonbondedForceDebug__); 
 #endif
 }
 
-inline void AdShiftedCoulombAndLennardJonesAEnergy(ListElement* interaction, 
+void AdShiftedCoulombAndLennardJonesAEnergy(ListElement* interaction, 
 		double** coordinates, 
 		double EPSILON_RP, 
 		double cut,
@@ -405,9 +460,12 @@ inline void AdShiftedCoulombAndLennardJonesAEnergy(ListElement* interaction,
 	*(seperation_s.vector + 2) = coordinates[atom_one][2] - coordinates[atom_two][2];
 	
 	Ad3DVectorLength(&seperation_s);
+	
+	//add this length to the linked list element the bond belongs to
+	interaction->length = seperation_s.length;
+	
 	if(seperation_s.length > cut)
 		return;
-	interaction->length = seperation_s.length;
 			
 	length_rec = 1/seperation_s.length;
 	
@@ -425,13 +483,13 @@ inline void AdShiftedCoulombAndLennardJonesAEnergy(ListElement* interaction,
 	*vdw_pot += lennardJonesA - lennardJonesB;	
 	
 #ifdef BASE_NONBONDED_DEBUG
-	AdNonbondedEnergyLog("Shifted", "A", atomOne, atomTwo, lennardJonesA, lennardJonesB, 
-			     chargeProduct, separation_s.length, est_hold, (lennardJonesA - lennardJonesB),
+	AdNonbondedEnergyLog("Shifted", "A", atom_one, atom_two, lennardJonesA, lennardJonesB, 
+			     chargeProduct, seperation_s.length, est_hold, (lennardJonesA - lennardJonesB),
 			     __ShiftedNonbondedEnergyDebug__); 
 #endif
 }
 
-inline void AdGRFCoulombAndLennardJonesAEnergy(ListElement* interaction, 
+void AdGRFCoulombAndLennardJonesAEnergy(ListElement* interaction, 
 		double** coordinates, 
 		double EPSILON_RP, 
 		double cutoff,
@@ -456,9 +514,13 @@ inline void AdGRFCoulombAndLennardJonesAEnergy(ListElement* interaction,
 	*(seperation_s.vector + 2) = coordinates[atom_one][2] - coordinates[atom_two][2];
 	
 	Ad3DVectorLength(&seperation_s);
+
+	//add this length to the linked list element the bond belongs to
+	interaction->length = seperation_s.length;
+	
 	if(seperation_s.length > cutoff)
 		return;
-	interaction->length = seperation_s.length;
+	
 	length_rec = 1/seperation_s.length;
 	vdw_hold = pow(length_rec, 6);
 	est_hold = EPSILON_RP*chargeProduct*(length_rec + b0);
@@ -469,14 +531,14 @@ inline void AdGRFCoulombAndLennardJonesAEnergy(ListElement* interaction,
 	*vdw_pot += lennardJonesA - lennardJonesB;
 		
 #ifdef BASE_NONBONDED_DEBUG
-	AdNonbondedEnergyLog("GRF", "A", atomOne, atomTwo, lennardJonesA, lennardJonesB, 
-			     chargeProduct, separation_s.length, est_hold, (lennardJonesA - lennardJonesB),
+	AdNonbondedEnergyLog("GRF", "A", atom_one, atom_two, lennardJonesA, lennardJonesB, 
+			     chargeProduct, seperation_s.length, est_hold, (lennardJonesA - lennardJonesB),
 			     __GRFNonbondedEnergyDebug__); 
 #endif
 }
 
 /*
-inline void AdCoulombAndLennardJonesAForce(ListElement* interaction, 
+void AdCoulombAndLennardJonesAForce(ListElement* interaction, 
 		double** coordinates, 
 		double** forces,
 		double EPSILON_RP, 
