@@ -25,6 +25,18 @@
 bool __FourierTorsionEnergyDebug__ = false;
 bool __FourierTorsionForceDebug__ = false;
 
+void AdFourierTorsionDebugInfo(void)
+{
+	fprintf(stderr, "\nFourier torsion debug on\n");
+	fprintf(stderr, "Energy Info - Atom one, Atom two, Atom three, Atom four");
+	fprintf(stderr, " Torsion constant, Period, Phase, Calculated Angle, Cumulative Potential\n\n");
+}
+
+double AdCalculateFourierTorsionEnergy(double angle, double period, double phase, double torsionConstant)
+{
+	return torsionConstant*(1 + cos(period*angle - phase)); 
+}
+
 void AdFourierTorsionEnergy(double *interaction, double** coordinates, double* tor_pot)
 {
 	register int i;
@@ -71,12 +83,12 @@ void AdFourierTorsionEnergy(double *interaction, double** coordinates, double* t
 	//check if the cosine of the angle is between -1 or 1. 
 	//It could have slipped beyond these bounds becuase of the limits to precision of doubles
 
-	if(cosine_ang > 1)
+	if(cosine_ang >= 1)
 	{
 		cosine_ang = 1;
-		angle = 0;
+		angle = 1E-15;
 	}
-	else if(cosine_ang < -1)
+	else if(cosine_ang <= -1)
 	{
 		cosine_ang = -1;
 		angle = M_PI;
@@ -207,7 +219,7 @@ void AdFourierTorsionEnergy(double *interaction, double** coordinates, double* t
 #endif
 }
 
-inline void AdFourierTorsionForce(double *interaction, double** coordinates, double** forces, double *tor_pot)
+int AdFourierTorsionForce(double *interaction, double** coordinates, double** forces, double *tor_pot)
 {
 	register int i, j;
 	int counter;
@@ -217,9 +229,10 @@ inline void AdFourierTorsionForce(double *interaction, double** coordinates, dou
 	double dot[12], product[12];
 	double *forceVector;
 	Vector3D n_one, n_two, ba, bc, cd, torsionForce;
+	int error=0;
 
 	//decode interaction
-
+	
 	atom_one = (int)interaction[0];
 	atom_two = (int)interaction[1];
 	atom_three = (int)interaction[2];
@@ -256,12 +269,12 @@ inline void AdFourierTorsionForce(double *interaction, double** coordinates, dou
 	//check if the cosine of the angle is between -1 or 1. 
 	//It could have slipped beyond these bounds becuase of the limits to precision of doubles
 	
-	if(cosine_ang > 1)
+	if(cosine_ang >= 1)
 	{
 		cosine_ang = 1;
-		angle = 0;
+		angle = 1E-15;
 	}
-	else if(cosine_ang < -1)
+	else if(cosine_ang <= -1)
 	{
 		cosine_ang = -1;
 		angle = M_PI;
@@ -364,6 +377,7 @@ inline void AdFourierTorsionForce(double *interaction, double** coordinates, dou
 				coordinates[atom_four][1],	
 				coordinates[atom_four][2]);	
 			fflush(stderr);	
+			error=1;
 		}
 		
 		*tor_pot += tor_cnst*(1 + holder);
@@ -398,6 +412,7 @@ inline void AdFourierTorsionForce(double *interaction, double** coordinates, dou
 				coordinates[atom_four][1],	
 				coordinates[atom_four][2]);	
 			fflush(stderr);	
+			error=1;
 		}
 
 		*tor_pot += tor_cnst*(1 - holder);
@@ -432,6 +447,7 @@ inline void AdFourierTorsionForce(double *interaction, double** coordinates, dou
 				coordinates[atom_four][1],	
 				coordinates[atom_four][2]);	
 			fflush(stderr);	
+			error=1;
 		}
 
 		*tor_pot += tor_cnst*(1 + holder);
@@ -523,10 +539,10 @@ inline void AdFourierTorsionForce(double *interaction, double** coordinates, dou
 			*tor_pot);
 	}
 #endif
-
+	return error;
 }
 
-inline double AdCalculateTorsionAngle(int* torsion, double** coordinates)
+double AdCalculateTorsionAngle(int* torsion, double** coordinates)
 {
 	register int i;
 	int atom_one, atom_two, atom_three, atom_four;
@@ -563,12 +579,12 @@ inline double AdCalculateTorsionAngle(int* torsion, double** coordinates)
 
 	//Check if the cosine of the angle is between -1 or 1. 
 	//It could have slipped beyond these bounds becuase of the limits to precision of doubles
-	if(cosine_ang > 1)
+	if(cosine_ang >= 1)
 	{
 		cosine_ang = 1;
-		angle = 0;
+		angle = 1E-15;
 	}
-	else if(cosine_ang < -1)
+	else if(cosine_ang <= -1)
 	{
 		cosine_ang = -1;
 		angle = M_PI;
