@@ -40,8 +40,7 @@ Sort;
 \ingroup Functions
 @{
 */
-
-inline int AdIndexSorter(const void* el_one, const void* el_two);
+int AdIndexSorter(const void* el_one, const void* el_two);
 int AdAscendingIntSort(const void* numberOne, const void* numberTwo);
 
 /** \@}**/
