@@ -54,7 +54,7 @@
 
 -(id)init    // @nodoc
 {
-	[super init];
+	self = [super init];
 	residues = RETAIN([NSMutableArray arrayWithCapacity: 100]);
 	residueKeys = [NSMutableDictionary new] ;
 	solvent = RETAIN([NSMutableArray arrayWithCapacity: 10]);
@@ -341,6 +341,31 @@
 	return [solvent objectEnumerator];
 }
 
+/*
+ *   return enumerator over all residues
+ */
+-(NSArray*)residues
+{
+	return [[residues copy] autorelease];
+}
+
+
+/*
+ *   return enumerator over all hetero groups
+ */
+-(NSArray*)heterogens
+{
+	return [[heterogens copy] autorelease];
+}
+
+
+/*
+ *   return enumerator over all solvent
+ */
+-(NSArray*)solvent
+{
+	return [[solvent copy] autorelease];
+}
 
 /*
  *   count residues
