@@ -244,6 +244,16 @@ static double mkFloat (const char *buffer, int len);
 	
 	/* where we can temporarily store atoms */
 	temporaryatoms = [NSMutableDictionary new];
+	
+	
+	//Set isStrict flag based on user options
+	//At the moment this sets if four letter residue names will be parsed.
+	NSDictionary* defaults;
+	
+	defaults = [NSDictionary dictionaryWithObject: [NSNumber numberWithBool: NO] 
+			forKey: @"StrictPDBParsing"];
+	[[NSUserDefaults standardUserDefaults] registerDefaults: defaults];
+	isStrict = [[NSUserDefaults standardUserDefaults] boolForKey: @"StrictPDBParsing"];
 
 	return self;
 }
@@ -503,7 +513,6 @@ static double mkFloat (const char *buffer, int len);
 	}
 }
 
-
 -(oneway void)readAtom:(in NSString*)line
 {
 	unsigned int i;
@@ -527,13 +536,31 @@ static double mkFloat (const char *buffer, int len);
 	[line getCString: buffer maxLength: 90];
 	/* serial number */
 	serial = mkInt(buffer+6,5); /* 7 - 11 atom serial number */
+	
 	/* atom name */
 	for (i=0; i<4; i++) { aname[i]=buffer[i+12]; } /* 13 - 16 atom name */
 	aname[4]='\0';
 	for (i=3; i>0; i--) { if (aname[i]==' ') aname[i]='\0'; } /* remove trailing whitespace */
+	
 	/* residue name */
-	for (i=0; i<3; i++) { rname[i]=buffer[i+17]; } /* 18 - 20 residue name */
-	rname[3]='\0';
+	/* 18 - 20 residue name */
+	for (i=0; i<3; i++) 
+	{ 
+		rname[i]=buffer[i+17]; 
+	} 
+	
+	//If we are in strict mode only read up to 19.
+	//Also do this if 20 is blank 
+	if((buffer[20] == ' ') || isStrict)
+	{
+		rname[3] = '\0';
+	}
+	else
+	{
+		rname[3]=buffer[20];
+		rname[4]='\0';
+	}
+	
 	/* chain identifier */
 	chain = buffer[21]; /* 22 chain identifier */
 	/* residue sequence nr */
@@ -680,7 +707,7 @@ static double mkFloat (const char *buffer, int len);
 	char buffer[90];
 	unsigned int serial,resnr;
 	char aname[5]; /* atom name */
-	char rname[4]; /* residue name */
+	char rname[5]; /* residue name */
 	double x,y,z,occ,bfact;
 	char icode;
 	char chain;
@@ -701,9 +728,26 @@ static double mkFloat (const char *buffer, int len);
 	/* atom name */
 	for (i=0; i<4; i++) { aname[i]=buffer[i+12]; } /* 13 - 16 atom name */
 	aname[4]='\0';
+	
 	/* residue name */
-	for (i=0; i<3; i++) { rname[i]=buffer[i+17]; } /* 18 - 20 residue name */
-	rname[3]='\0';
+	/* 18 - 20 residue name */
+	for (i=0; i<3; i++) 
+	{ 
+		rname[i]=buffer[i+17]; 
+	}
+	
+	//If we are in strict mode only read up to 19.
+	//Also do this if 20 is blank 
+	if((buffer[20] == ' ') || isStrict)
+	{
+		rname[3] = '\0';
+	}
+	else
+	{
+		rname[3]=buffer[20];
+		rname[4]='\0';
+	}
+	
 	if ((rname[0]=='H' && rname[1]=='O' && rname[2]=='H')
 	 || (rname[0]=='S' && rname[1]=='O' && rname[2]=='L')
 	 || (rname[0]=='D' && rname[1]=='I' && rname[2]=='S'))
