@@ -506,13 +506,30 @@ static NSString *mkTextDate (NSCalendarDate *caldate);
 		[[residue name] getCString: tbuffer maxLength: 8]; /* residue name */
 		buffer[17] = tbuffer[0]; buffer[18] = tbuffer[1]; buffer[19] = tbuffer[2];
 	}
-	else	
+	else if([[residue name] length] == 1)	
 	{
 		//Single letter nucleic acid id - Using the same code as above
 		//leads to putting the NULL character in buffer[18].
 		[[residue name] getCString: tbuffer maxLength: 8]; /* residue name */
 		buffer[17] = tbuffer[0]; buffer[18] = 32; buffer[19] = 32;
 	}
+	else if([[residue name] length] == 4)
+	{
+		NSWarnLog(@"Four letter residue names are not supported by PDB standard");
+		[[residue name] getCString: tbuffer maxLength: 8];
+		buffer[17] = tbuffer[0]; 
+		buffer[18] = tbuffer[1]; 
+		buffer[19] = tbuffer[2];
+		if(isStrict)
+		{
+			NSWarnLog(@"StrictPDBWriting is YES. Writing a three letter code");
+		}
+		else
+		{
+			NSWarnLog(@"StrictPDBWriting is NO. Writing a four letter code");
+			buffer[20] = tbuffer[3];	
+		}
+	}
 	
         snprintf(tbuffer, 10, "% 4u    ", [[residue number] intValue]); /* residue number */
         buffer[22]=tbuffer[0]; buffer[23]=tbuffer[1]; buffer[24]=tbuffer[2]; buffer[25]=tbuffer[3];
