@@ -35,6 +35,8 @@
 {
 	NSDictionary* infoDict;
 	NSDictionary* conversionFactors;
+	NSMutableString* resultsString;
+	NSMutableDictionary* pluginOptions;
 }
 
 @end
