@@ -161,8 +161,33 @@ and ensure its executable. Check AdServer.log for more information."]];
 	freopen([logFile cString], "w", stderr);
 }
 
+- (void) _connectToDefaultDatabase
+{
+	NSString *location, *hostname, *clientName;
+	NSError *error = nil;
+
+	//Create an interface to the default filesystem database
+	location = [[NSUserDefaults standardUserDefaults] stringForKey: @"DefaultDatabase"];
+	hostname = [[NSHost currentHost] name];
+	clientName = [NSString stringWithFormat: @"server@%@", hostname];
+	defaultBackend = [[ULFileSystemDatabaseBackend alloc] 
+				initWithDatabaseName: location 
+				clientName: clientName 
+				error: &error];
+	if(error != nil)
+	{
+		AdLogError(error);
+	}
+	else
+	{
+		NSLog(@"Connected to default database at %@ as %@", 
+			[defaultBackend databaseName], [defaultBackend clientName]);
+	}
+}
+
 - (id) init
 {
+	int portNumber;
 	id adunFile;
 	id port;
 
@@ -172,26 +197,27 @@ and ensure its executable. Check AdServer.log for more information."]];
 		if([[NSUserDefaults standardUserDefaults] boolForKey: @"RedirectOutput"])
 			[self _redirectOutput];
 		
-		if([[NSUserDefaults standardUserDefaults] boolForKey: @"AdServerIsDistributed"])
+		if([[NSUserDefaults standardUserDefaults] boolForKey: @"IsDistributed"])
 		{
 
-			GSPrintf(stderr, @"Using ports to enable distributed computing.\n");
+			NSLog(@"Using ports to enable distributed computing.\n");
+			portNumber = [[NSUserDefaults standardUserDefaults] integerForKey: @"PortNumber"];
 #ifdef GNUSTEP
-			port = [NSSocketPort portWithNumber: 1079
+			port = [NSSocketPort portWithNumber: portNumber
 				onHost: nil
 				forceAddress: nil
 				listener: YES];
 #else
 			port = [[NSSocketPort alloc]
-				initWithTCPPort: 1079];
+				initWithTCPPort: portNumber];
 			[port autorelease];
 #endif				
 			connection = [[NSConnection alloc] 
 					initWithReceivePort: port
 					sendPort: nil];
-			GSPrintf(stderr, @"Port is %@\n", [port description]);		
+			NSLog(@"Port is %@\n", [port description]);		
 			NSDebugLLog(@"AdunServer", @"Connection is %@", connection);		
-			GSPrintf(stderr, @"Connection is %@\n", connection);		
+			NSLog(@"Connection is %@\n", connection);		
 			if([connection registerName: @"AdunServer" withNameServer:
 					 [NSSocketPortNameServer sharedInstance]] == NO)
 			{
@@ -202,8 +228,7 @@ and ensure its executable. Check AdServer.log for more information."]];
 		}
 		else
 		{
-			GSPrintf(stderr, 
-				@"Using default message ports. Distributed computing facilites disabled.\n");
+			NSLog(@"Using default message ports. Distributed computing facilites disabled.\n");
 			
 			connection = [NSConnection defaultConnection];
 			[connection setRootObject: self];
@@ -227,7 +252,10 @@ and ensure its executable. Check AdServer.log for more information."]];
 		processErrors = [NSMutableDictionary new];
 		storedMessages = [NSMutableDictionary new];
 		disconnectedProcesses = [NSMutableArray new];
-
+		
+		//Create the default database interface
+		[self _connectToDefaultDatabase];
+		
 		//see if we can locate the adun core executable
 		adunCorePath = [[[NSUserDefaults standardUserDefaults] stringForKey: @"AdunCorePath"] 
 					stringByAppendingPathComponent: @"AdunCore"];
@@ -254,7 +282,7 @@ and ensure its executable. Check AdServer.log for more information."]];
 				[self _adunCoreError];	
 		}
 		else
-			GSPrintf(stderr, @"Found Adun Core Executable at %@\n", adunCorePath);
+			NSLog(@"Found Adun Core Executable at %@\n", adunCorePath);
 	}
 	
 	fflush(stderr);
@@ -272,13 +300,21 @@ and ensure its executable. Check AdServer.log for more information."]];
 	[processErrors release];
 	[disconnectedProcesses release];
 	[storedMessages release];
+	[defaultBackend release];
 
 	[super dealloc];
 }
 
+@end
+
+/**
+Contains the methods used by other processes to launch and interact with remote simulations.
+*/
+@implementation AdServer (RemoteSimulationManagement)
+
 //Refactor these messages to use execute:error:process
 
-- (NSError*) startSimulation: (id) process arguments: (NSArray*) args
+- (NSError*) startSimulation: (id) process
 {
 	id task;
 	NSMutableDictionary* userInfo;
@@ -289,18 +325,20 @@ and ensure its executable. Check AdServer.log for more information."]];
 	NSDebugLLog(@"AdunServer", @"Recieved a start simulation message");
 	NSDebugLLog(@"AdunServer", @"Process Object is %@", [process description]);
 	NSDebugLLog(@"AdunServer", @"Found Adun Core Executable at %@", adunCorePath);
-	NSDebugLLog(@"AdunServer", @"Process arguments %@", args);
+	NSDebugLLog(@"AdunServer", @"Process arguments %@", [process arguments]);
 				
 	NS_DURING
 	{
+		[process setStarted: [NSDate date]];
 		task = [NSTask launchedTaskWithLaunchPath: adunCorePath
-			arguments: args];
+			arguments: [process arguments]];
 
 		//FIXME: Its possible that the task will send back a request
 		//before we can complete the next two steps causing it to fail.
 		//We need to use an identifier other than the pid which isnt available
 		//until the task launches e.g. unique process object string.
 		//This requires refactoring all the communication protocols however.
+		
 		[process setProcessIdentifier: [task processIdentifier]];
 		[processes setObject: process 
 			forKey: [NSNumber numberWithInt: [task processIdentifier]]];
@@ -308,6 +346,8 @@ and ensure its executable. Check AdServer.log for more information."]];
 			forKey: [NSNumber numberWithInt: [task processIdentifier]]];
 		[state setObject: [NSNumber numberWithBool: NO] 
 			forKey: [NSNumber numberWithInt: [task processIdentifier]]];
+		
+		[process setProcessStatus: @"Running"];	
 	}
 	NS_HANDLER
 	{
@@ -315,6 +355,7 @@ and ensure its executable. Check AdServer.log for more information."]];
 		[userInfo setObject: 
 			[NSString stringWithFormat: @"Unable to lauch simulation - %@", [localException reason]]
 			forKey: NSLocalizedDescriptionKey];
+		[process setProcessStatus: @"Error"];		
 
 		return [NSError errorWithDomain: @"AdServerErrorDomain"
 				code: 1
@@ -477,26 +518,21 @@ you get a seg fault
 	return [interface validCommands];
 }
 
+@end
+
+/**
+Contains the methods used by an AdunCore instance to communicate with the server 
+that launched it.
+Data sent back by the simulation is passed to the remote process that requested its
+creation using ULProcess methods.
+*/
+@implementation AdServer (AdunCoreInterface)
+
 - (void) useInterface: (id) object forProcess: (int) pid
 {
 	[interfaces setObject: object forKey: [NSNumber numberWithInt: pid]];
 }
 
-- (void) acceptingRequests: (int) pid
-{
-	[state setObject: [NSNumber numberWithBool: YES] forKey: [NSNumber numberWithInt: pid]];
-	
-	NSDebugLLog(@"AdunServer", @"Process %d is accepting requests", pid);
-}
-
-- (void) closeConnectionForProcess: (int) pid error: (NSError*) error
-{
-	[interfaces removeObjectForKey: [NSNumber numberWithInt: pid]];
-	
-	if(error != nil)
-		[processErrors setObject: error forKey: [NSNumber numberWithInt: pid]];
-}
-
 - (bycopy NSDictionary*) externalObjectsForProcess: (int) pid
 {
 	id process;
@@ -513,6 +549,13 @@ you get a seg fault
 	return [process simulationTemplate];
 }
 
+- (void) acceptingRequests: (int) pid
+{
+	[state setObject: [NSNumber numberWithBool: YES] forKey: [NSNumber numberWithInt: pid]];
+	
+	NSDebugLLog(@"AdunServer", @"Process %d is accepting requests", pid);
+}
+
 - (void) controllerData: (id) results forProcess: (int) pid
 {
 	id  process;
@@ -568,4 +611,35 @@ you get a seg fault
 		[process setSimulationData: data];
 	}
 }
+
+- (void) closeConnectionForProcess: (int) pid error: (NSError*) error
+{
+	[interfaces removeObjectForKey: [NSNumber numberWithInt: pid]];
+	
+	if(error != nil)
+		[processErrors setObject: error forKey: [NSNumber numberWithInt: pid]];
+}
+
 @end
+
+/**
+Initial testing implementation of the RemoteDataManagement category lacking advanced security.
+*/
+@implementation AdServer (RemoteDataManagement)
+
+- (ULFileSystemDatabaseBackend*) backendForDefaultFileSystemDatabase
+{
+	NSDebugLLog(@"AdunServer", @"Returning database interface %@", defaultBackend);
+	return [[defaultBackend retain] autorelease];
+}
+
+- (ULFileSystemDatabaseBackend*) backendForFileSystemDatabase: (NSString*) path
+{
+	//Incomplete implementation
+	return [self backendForDefaultFileSystemDatabase];
+}
+
+@end
+
+
+
