@@ -105,6 +105,12 @@
 	{
 		closeImage = [[NSImage alloc] initWithContentsOfFile: path];
 	}
+	
+	path = [[NSBundle mainBundle] pathForImageResource: @"export.png"];
+      	if (path != nil)
+	{
+		exportImage = [[NSImage alloc] initWithContentsOfFile: path];
+	}
 }
 
 - (void)awakeFromNib
@@ -196,151 +202,6 @@
 	threadError = YES;
 }
 
-/**
- * Managing opening, closing & selecting data sets.
- * An opened data set is one that appears in the data set list.
- */
-
-- (void) openDataSets: (NSArray*) anArray
-{
-	NSEnumerator* dataSetEnum;
-	AdDataSet* dataSet;
-	
-	dataSetEnum = [anArray objectEnumerator]; 
-	while(dataSet = [dataSetEnum nextObject])
-		[self openDataSet: dataSet];
-}
-
-- (void) openDataSet: (AdDataSet*) aDataSet
-{
-	BOOL selectNewSet = NO;
-	NSEnumerator* dataSetEnum;
-	AdDataSet* set;
-
-	if([openDataSets containsObject: aDataSet])
-		return;
-	
-	//Check if any of the opened data sets has the same name. 
-	//Only one data set with the same name can be open at the 
-	//same time.
-	if([dataSetList indexOfItemWithTitle: [aDataSet name]] != -1)
-	{
-		dataSetEnum = [openDataSets objectEnumerator];
-		while(set = [dataSetEnum nextObject])
-		{
-			if([[set name] isEqual: [aDataSet name]])
-				break;
-		}
-		//If this data set is displayed we want the new
-		//one to replace it. Check if this is the case.
-		if([dataView dataSet] == set)
-			selectNewSet = YES;
-			
-		[self closeDataSet: set];
-	}
-	
-	[openDataSets addObject: aDataSet];
-	[self setAvailableDataSets: openDataSets];
-	[self createGnuplotFilesForDataSet: aDataSet];
-	
-	//We only select the new set here if we
-	//had to close one with the same name
-	//while it was being displayed
-	if(selectNewSet)
-		[self selectDataSet: aDataSet];
-}
-
-- (void) closeSelectedDataSet: (id) sender
-{
-	[self closeDataSet: selectedDataSet];
-}
-
-- (void) closeDataSet: (AdDataSet*) aDataSet
-{
-	//The dataset may only be present in the openDataSets array.
-	//If this is so then it will be deallocated when removed and the
-	//program will crash during removeGnuplotFilesForDataSet:
-	//Hence we retain/autorelease it here to prevent this.
-	[[aDataSet retain] autorelease];
-	if(![openDataSets containsObject: aDataSet])
-		return;
-	
-	[openDataSets removeObject: aDataSet];
-	[self setAvailableDataSets: openDataSets];
-	[self removeGnuplotFilesForDataSet: aDataSet];
-}
-
-- (void) selectDataSet: (AdDataSet*) aDataSet
-{
-	if(![openDataSets containsObject: aDataSet])
-		return;
-	
-	[dataSetList selectItemAtIndex: 
-		[openDataSets indexOfObject: aDataSet]];
-	[self dataSetDidChange: self];	
-}
-
-- (void) setAvailableDataSets: (NSArray*) array
-{
-	NSEnumerator* arrayEnum;
-	AdDataSet* dataSet;
-
-	[dataSetList removeAllItems];
-	arrayEnum = [array objectEnumerator];
-	while(dataSet = [arrayEnum nextObject])
-		[dataSetList addItemWithTitle: [dataSet name]];
-	
-	if([dataSetList numberOfItems] != 0)	
-	{
-		//If the last selected data set is in the new list
-		//keep it selected. Otherwise select the first item.
-		if([openDataSets containsObject: selectedDataSet])
-		{
-			[dataSetList selectItemAtIndex: 
-				[openDataSets indexOfObject: selectedDataSet]];
-		}		
-		else	
-		{
-			[dataSetList selectItemAtIndex: 0];
-			[self dataSetDidChange: self];
-		}	
-		//Enable the close button
-		[[[toolbar items] objectAtIndex: 3] setEnabled: YES];	
-	
-	}
-	else
-	{		
-		//Nothing is open so clear the view
-		[dataView setDataSet: nil];
-		[dataView clearDataSet];
-		[selectedDataSet release];
-		selectedDataSet = nil;
-		[[[toolbar items] objectAtIndex: 3] setEnabled: NO];	
-	}
-}
-
-//Called when user selects a different data set
-//from the data set list
-- (void) dataSetDidChange: (id) sender
-{
-	NSString* name;
-	NSEnumerator* dataSetEnum;
-	AdDataSet* dataSet;
-
-	name = [dataSetList titleOfSelectedItem];
-	dataSetEnum = [openDataSets objectEnumerator];
-	while(dataSet = [dataSetEnum nextObject])
-		if([[dataSet name] isEqual: name])
-		{
-			[selectedDataSet release];
-			selectedDataSet = dataSet;
-			[selectedDataSet retain];
-			[dataView setDataSet: selectedDataSet];
-			[dataView displayData];
-			break;
-		}
-}
-
 /***************
 
 Opening and Closing the view
@@ -488,9 +349,26 @@ Opening and Closing the view
 		return [self _validateLoad: sender];
 }
 
+- (BOOL) _validateExport: (id) sender
+{
+	//FIXME: Make ULExportController first responder to export:
+	//Just put export validation here for the moment.
+	//It the command is exportAs: we have to ask the export controller 
+	//if the export is possible
+	if([NSStringFromSelector([sender action]) isEqual: @"exportAs:"])
+		return [[ULExportController sharedExportController] 
+			canExportCurrentPasteboardObjectAs: [sender tag]];
+	
+	//Everything can be exported via the export panel
+	if([[[ULPasteboard appPasteboard] availableTypes] count] == 1)
+		return YES;
+		
+	return NO;	
+}
+
 - (BOOL) validateMenuItem: (NSMenuItem*) menuItem
 {
-	id action;	
+	id action;
 	SEL selector;
 	
 	//NSPopUpButton is also a menu - however we want
@@ -498,7 +376,9 @@ Opening and Closing the view
 	if([menuItem action] == NULL)
 		return YES;
 	
-	if([NSStringFromSelector([menuItem action]) isEqual: @"deselectAllRows:"])
+	action = NSStringFromSelector([menuItem action]);
+	
+	if([action isEqual: @"deselectAllRows:"])
 	{
 		if([[loadedObjectsTable selectedRowIndexes] count] > 0)
 			return YES;	
@@ -506,9 +386,13 @@ Opening and Closing the view
 			return NO;
 	}	
 
-	if([NSStringFromSelector([menuItem action]) isEqual: @"loadExternal:"])
+	if([action isEqual: @"loadExternal:"])
 			return YES;	
-
+	
+	//Do export explicitly for now
+	if([action isEqual: @"export:"] || [action isEqual: @"exportAs:"])
+		return [self _validateExport: menuItem];
+				
 	action = [NSStringFromSelector([menuItem action]) capitalizedString];
 	action = [NSString stringWithFormat: @"_validate%@", action];
 	selector = NSSelectorFromString(action);
@@ -840,9 +724,11 @@ Menu Commands
 			[loadedObjectsTable reloadData];
 		}
 		
-		//Regardless of whether the data set was saved we
-		//have to output the gnuplot files again.
-		[self createGnuplotFilesForDataSet: selectedDataSet];
+		//Regardless of whether the data set was saved we.
+		//Use a thread since they may be very large.
+		[NSThread detachNewThreadSelector: @selector(threadedCreateGnuplotFilesForDataSet:) 
+					 toTarget: self 
+				       withObject: selectedDataSet];
 	}
 	else
 		NSRunAlertPanel(@"Error",
@@ -852,6 +738,54 @@ Menu Commands
 			nil);
 }
 
+- (void) export: (id) sender
+{
+	/*
+	 * Delegate to the shared export controller.
+	 * It will retrieve the selected object from us via
+	 * the pasteboard.
+	 */
+	[[ULExportController sharedExportController] export: self];
+}
+
+/**
+ \e sender is the menu item that was clicked
+ */
+- (void) exportAs: (id) sender
+{
+	[[ULExportController sharedExportController] 
+	 exportCurrentPasteboardObjectAs: [sender tag]];
+}
+
+//Handels return from exportDisplayedTable save panel.
+- (void) savePanelDidEnd:(NSSavePanel *)sheet returnCode:(int)returnCode  contextInfo:(void  *)contextInfo
+{
+	NSString* filename, *string;
+	AdDataMatrix* matrix;
+	
+	if(returnCode == NSOKButton)
+	{
+		filename = [sheet filename];
+		filename = [filename stringByDeletingPathExtension];
+		filename = [filename stringByAppendingPathExtension: @"csv"];
+		matrix = [dataView displayedMatrix];
+		string = [matrix stringRepresentation];
+		[string writeToFile: filename atomically: NO];
+	}
+}
+
+- (void) exportDisplayedTable: (id) sender
+{
+	NSSavePanel* savePanel = [NSSavePanel savePanel];
+
+	[savePanel beginSheetForDirectory:  nil
+		file: nil 
+		modalForWindow: window 
+		modalDelegate: self 
+		didEndSelector: @selector(savePanelDidEnd:returnCode:contextInfo:) 
+		contextInfo: NULL];
+}
+
 - (void) deselectAllRows: (id) sender
 {
 	int row;
@@ -955,55 +889,6 @@ Menu Commands
 	}
 }
 
-/**
-Pasteboard Methods
-**/
-
-- (NSArray*) availableTypes
-{
-	NSString* type;
-
-	if([selectedObjects count] > 0)
-	{
-		type = NSStringFromClass([[selectedObjects objectAtIndex: 0] class]);
-		return [NSArray arrayWithObject: type];
-	}	
-	else
-		return [NSArray array];
-		
-}
-
-- (id) objectForType: (NSString*) type;
-{
-	if([[self availableTypes] containsObject: type])
-		return [selectedObjects objectAtIndex: 0];
-	else
-		return nil;
-}
-
-- (NSArray*) objectsForType: (NSString*) type
-{
-	if([[self availableTypes] containsObject: type])
-		return [NSArray arrayWithObject: [selectedObjects objectAtIndex: 0]];
-	else
-		return nil;
-
-}
-
-- (int) countOfObjectsForType: (NSString*) type
-{
-	if([[self availableTypes] containsObject: type])
-		return 1;
-	else
-		return 0;
-
-}
-
-- (void) pasteboardChangedOwner: (id) pasteboard
-{
-	[self deselectAllRows: self];
-}
-
 /***************
 
 Loaded Objects Table Data Source Methods
@@ -1106,435 +991,212 @@ Window Delegate Methods
 
 @end
 
-/*
- * Extensions to ULAnalyser to
- * enable gnuplot integration.
- */
-
-@implementation ULAnalyser (ULAnalyserGnuplotExtensions)
-
-- (void) gnuplotDealloc
-{
-	[pipey release];
-	[outPipe release];
-	[gnuplotOutput release];
-	[gnuplotError release];
-	[gnuplotDir release];
-	[gnuplot terminate];
-	[gnuplot release];
-	[history release];
-	[[NSNotificationCenter defaultCenter]
-		removeObserver: self];
-}
-
 /**
-Wrapper around gnuplot launch with error handling.
-Returns the location at which to put the prompt in the
-interface
-*/
-- (int) _launchGnuplot
+ Pasteboard Methods
+ **/
+@implementation ULAnalyser (PasteboardDataSource)
+
+- (NSArray*) availableTypes
 {
-	NSRange endRange;
-	NSMutableString *errorString;
-
-	NS_DURING
-	{
-		[gnuplot launch];
-		gnuplotRunning = YES;
-		endRange.location = 0;
-	}
-	NS_HANDLER
-	{
-		if([[localException name] isEqual: @"NSInvalidArgumentException"])
-		{
-			errorString = [NSMutableString string];
-			[errorString appendFormat: 
-				@"Unable to launch gnuplot from %@\n", [gnuplot launchPath]];
-			NSWarnLog(@"%@", errorString);	
-			[errorString appendString:
-				@"Either gnuplot is not installed or the exectuable is in another location\n"];
-			[errorString appendString:
-				@"If the latter specify the directory containing the exectutable using"];
-			[errorString appendString:
-				@" the preferences panel (Info->Preferences)\n"];
-			[errorString appendString:
-				@"Or execute,\n\n\t'defaults write UL GnuplotPath $PATH_TO_GNUPLOT'\n\n"];
-			[errorString appendString: @"on the command line\n"];	
-
-			endRange.location = 0;
-			endRange.length = 0;
-			[gnuplotInterface replaceCharactersInRange:endRange 
-				withString:errorString];
-			endRange.location = [[gnuplotInterface textStorage] length];	
-			
-			//Indicate that gnuplot failed to launch
-			gnuplotRunning = NO;
-		}
-		else
-			[localException raise];
-	}
-	NS_ENDHANDLER
-
-	return endRange.location;
-}
-
-- (void) _setupGnuplotDirectory
-{
-	NSFileManager* fileManager;
-	NSEnumerator* gnuplotDirEnum;
-	NSString* contentObject, *path;
-	NSError* error = nil;
-
-	//Create gnuplot directory if it doesnt exist
-	fileManager = [NSFileManager defaultManager];
-	gnuplotDir = [[[ULIOManager appIOManager] applicationDir]
-			stringByAppendingPathComponent: @".gnuplot"];
-	[gnuplotDir retain];		
-	if(![fileManager directoryExistsAtPath: gnuplotDir error: &error])
-	{
-		if(error != nil)
-			ULRunErrorPanel(error);
-		else
-			[fileManager createDirectoryAtPath: gnuplotDir
-				attributes: nil
-				error: NULL];
-	}
+	NSString* type;
 	
-	//Remove any files in the gnuplot directory	
-	gnuplotDirEnum = [[fileManager directoryContentsAtPath: gnuplotDir]
-				objectEnumerator];
-	while((contentObject = [gnuplotDirEnum nextObject]))
+	if([selectedObjects count] > 0)
 	{
-		path = [gnuplotDir stringByAppendingPathComponent: contentObject];
-		[fileManager removeFileAtPath: path handler: NULL];
-	}
-	
-	//Register for ULAnalyserDataSetViewColumnOrderDidChangeNotification
-	//so we can update the relevant file.
-	[[NSNotificationCenter defaultCenter]
-		addObserver: self
-		selector: @selector(updateFileForDisplayedMatrix:)
-		name: @"ULAnalyserDataSetViewColumnOrderDidChangeNotification"
-		object: nil];
-}
-
-- (void) setupGnuplotInterface
-{
-	BOOL isDir;
-	NSRange endRange;
-	NSString* launchPath;
-	NSMutableString *errorString;
-	NSFileManager* fileManager;
-	ULIOManager* ioManager = [ULIOManager appIOManager];
-
-	//Create and/or clear the gnuplot directory
-	//if necessary.
-	[self _setupGnuplotDirectory];
-
-	history = [NSMutableArray new];
-	historyDepth = 100;
-	currentHistoryPosition = 0;
-
-	gnuplotRunning = NO;
-	[gnuplotInterface setDelegate: self];
-	pipey =  [NSPipe new]; 
-	outPipe = [NSPipe new];
-	gnuplotOutput =  [[pipey fileHandleForWriting] retain]; 
-	gnuplotError = [[outPipe fileHandleForReading] retain];
-	gnuplot = [NSTask new];
-	launchPath = [[NSUserDefaults standardUserDefaults]
-			stringForKey: @"GnuplotPath"];
-	
-	//Check the path specifies a file and not a directory
-	[[NSFileManager defaultManager]
-		fileExistsAtPath: launchPath
-		isDirectory: &isDir];
-
-	//If it is a directory print an error message and return
-	//without attempting to launch gnuplot
-	if(isDir)
-	{
-		errorString = [NSMutableString string];
-		[errorString appendFormat: @"Specified path %@ refers to a directory\n",
-			launchPath];
-		[errorString appendFormat: @"Path must be the full path to the executable\n",
-			launchPath];
-			
-		endRange.location = 0;
-		endRange.length = 0;
-		[gnuplotInterface replaceCharactersInRange:endRange 
-			withString:errorString];
-		endRange.location = [[gnuplotInterface textStorage] length];	
-	}
-	else
-	{
-		[gnuplot setLaunchPath: launchPath];
-		[gnuplot setCurrentDirectoryPath: gnuplotDir];
-		[gnuplot setStandardInput: pipey];
-		[gnuplot setStandardError: outPipe];
-
-		/*
-		 * Attempt to launch gnuplot.
-		 * If this fails output a message to the gnuplot terminal
-		 * The terminal will continue to accept input but nothing
-		 * will actuall work
-		 */
-
-		endRange.location = [self _launchGnuplot];
+		type = NSStringFromClass([[selectedObjects objectAtIndex: 0] class]);
+		return [NSArray arrayWithObject: type];
 	}	
-
-	endRange.length = 0;
-	[gnuplotInterface replaceCharactersInRange:endRange withString:@"gnuplot> "];
-
-	gnuplotPrompt.location = endRange.location;
-	gnuplotPrompt.length = [[gnuplotInterface textStorage] length] - endRange.location;
-}
-
-/******************
-
-Gnuplot History
-
-*******************/
-
-- (void) _addStringToHistory: (NSString*) string
-{
-	[history addObject: string];
-
-	if([history count] == historyDepth)
-		[history removeObjectAtIndex: 0];
-
-	currentHistoryPosition = [history count];
-}
-
-- (NSString*) _previousStringFromHistory
-{
-	id string;
-
-	NS_DURING
-	{
-		currentHistoryPosition--;
-		string = [history objectAtIndex: currentHistoryPosition];
-	}
-	NS_HANDLER
-	{
-		if([[localException name] isEqual: NSRangeException])
-		{
-			if([history count] != 0)
-			{
-				currentHistoryPosition = 0;
-				string = [history objectAtIndex: currentHistoryPosition];
-			}
-			else
-				string = @"";
-		}
-	}
-	NS_ENDHANDLER
+	else
+		return [NSArray array];
 	
-	return string;
 }
 
-- (NSString*) _nextStringFromHistory
+- (id) objectForType: (NSString*) type;
 {
-	id string;
-
-	NS_DURING
-	{
-		currentHistoryPosition++;
-		string = [history objectAtIndex: currentHistoryPosition];
-	}
-	NS_HANDLER
-	{
-		if([[localException name] isEqual: NSRangeException])
-		{
-			currentHistoryPosition = [history count];
-			string = @"";
-		}
-	}
-	NS_ENDHANDLER
-	
-	return string;
+	if([[self availableTypes] containsObject: type])
+		return [selectedObjects objectAtIndex: 0];
+	else
+		return nil;
 }
 
+- (NSArray*) objectsForType: (NSString*) type
+{
+	if([[self availableTypes] containsObject: type])
+		return [NSArray arrayWithObject: [selectedObjects objectAtIndex: 0]];
+	else
+		return nil;	
+}
+
+- (int) countOfObjectsForType: (NSString*) type
+{
+	if([[self availableTypes] containsObject: type])
+		return 1;
+	else
+		return 0;	
+}
+
+- (void) pasteboardChangedOwner: (id) pasteboard
+{
+	[self deselectAllRows: self];
+}
+@end
+
 /**
- * Methods for handling the text files containing the data displayed in the
- * data table in the interface. These files are what gnuplot uses to plot the data.
+ Contains methods for opening, selecting and closing
+ the data sets displayed in the data set list 
  */
+@implementation ULAnalyser (ULAnalyserDataSetOpening)
 
-/**
-Outputs \e aMatrix which is in \e aDataSet to a file called dataSetName.matrixName in the gnuplot
-directory. The columns in the matrix are output in the order defined by \e columnHeaders. This is an
-array of the column headers (NSStrings) of \e aMatrix in the desired order.
-Raises an NSInternalInconsistencyException of \e aMatrix is not in \e aDataSet. 
-Raises an NSInvalidArgumentException if any of the headers in \e columnHeaders is not a header of
-a column of \e aMatrix.
-*/
-- (void) _outputFileForMatrix: (AdDataMatrix*) aMatrix 
-	inDataSet: (AdDataSet*) aDataSet 
-	columnOrder: (NSArray*) columnHeaders
+- (void) openDataSets: (NSArray*) anArray
 {
-	int i, j, noColumns;
-	NSEnumerator* columnEnum;
-	NSString* header, *fileName, *string;
-	id value;
-	FILE* matrixFile;
+	NSEnumerator* dataSetEnum;
+	AdDataSet* dataSet;
 	
-	//Create the path to the matrix file
-	fileName = [NSString stringWithFormat: @"%@.%@",
-			[aDataSet name], [aMatrix name]];
-	fileName = [gnuplotDir stringByAppendingPathComponent: fileName];	
+	dataSetEnum = [anArray objectEnumerator]; 
+	while(dataSet = [dataSetEnum nextObject])
+		[self openDataSet: dataSet];
+}
+
+- (void) openDataSet: (AdDataSet*) aDataSet
+{
+	BOOL selectNewSet = NO;
+	NSEnumerator* dataSetEnum;
+	AdDataSet* set;
 	
-	//open the file
-	matrixFile = fopen([fileName cString], "w");
-	for(i=0; i < (int)[aMatrix numberOfRows]; i++)
+	if([openDataSets containsObject: aDataSet])
+		return;
+	
+	//Check if any of the opened data sets has the same name. 
+	//Only one data set with the same name can be open at the same time.
+	if([dataSetList indexOfItemWithTitle: [aDataSet name]] != -1)
 	{
-		columnEnum = [columnHeaders objectEnumerator];
-		while(header = [columnEnum nextObject])
+		dataSetEnum = [openDataSets objectEnumerator];
+		while(set = [dataSetEnum nextObject])
 		{
-			value = [aMatrix elementAtRow: i
-					ofColumnWithHeader: header];
-			GSPrintf(matrixFile, @"%-@ ", value);		
+			if([[set name] isEqual: [aDataSet name]])
+				break;
 		}
-		fprintf(matrixFile, "\n");
+		//If this data set is displayed we want the new
+		//one to replace it. Check if this is the case.
+		if([dataView dataSet] == set)
+			selectNewSet = YES;
+		
+		[self closeDataSet: set];
 	}
 	
-	fclose(matrixFile);
-}
-
-- (void) _outputFileForMatrix: (AdDataMatrix*) aMatrix inDataSet: (AdDataSet*) aDataSet
-{
-	[self _outputFileForMatrix: aMatrix
-		inDataSet: aDataSet
-		columnOrder: [aMatrix columnHeaders]];
-}
-
-- (void) updateFileForDisplayedMatrix: (NSNotification*) aNotification
-{
-	[self _outputFileForMatrix: [dataView displayedMatrix]		
-		inDataSet: [dataView dataSet]
-		columnOrder: [dataView orderedColumnHeaders]];
-}
-
-- (void) createGnuplotFilesForDataSet: (AdDataSet*) aDataSet
-{
-	NSEnumerator* matrixEnum;
-	AdDataMatrix* matrix;
+	[openDataSets addObject: aDataSet];
+	[self setAvailableDataSets: openDataSets];
 	
-	matrixEnum = [[aDataSet dataMatrices] objectEnumerator];
-	while(matrix = [matrixEnum nextObject])
-		[self _outputFileForMatrix: matrix
-			inDataSet: aDataSet];
-}
-
-//Removing files
-
-- (void) _removeFileForMatrix: (AdDataMatrix*) aMatrix inDataSet: (AdDataSet*) aDataSet
-{
-	NSFileManager* fileManager;
-	NSString* fileName;
+	//Output the files in a dedicated thread.
+	[NSThread detachNewThreadSelector: @selector(threadedCreateGnuplotFilesForDataSet:) 
+				 toTarget: self 
+			       withObject: aDataSet];
 	
-	fileManager = [NSFileManager defaultManager];
-	fileName = [NSString stringWithFormat: @"%@.%@",
-		[aDataSet name], [aMatrix name]];
-	fileName = [gnuplotDir stringByAppendingPathComponent: fileName];		
-	[fileManager removeFileAtPath: fileName 
-		handler: NULL];		
+	//We only select the new set here if we
+	//had to close one with the same name
+	//while it was being displayed
+	if(selectNewSet)
+		[self selectDataSet: aDataSet];
 }
 
-- (void) removeGnuplotFilesForDataSet: (AdDataSet*) aDataSet
+- (void) closeSelectedDataSet: (id) sender
 {
-	NSEnumerator* matrixEnum;
-	AdDataMatrix* matrix;
-	
-	matrixEnum = [[aDataSet dataMatrices] objectEnumerator];
-	while(matrix = [matrixEnum nextObject])
-		[self _removeFileForMatrix: matrix
-			inDataSet: aDataSet];	
+	[self closeDataSet: selectedDataSet];
 }
 
-- (void) removeGnuplotFilesForDataSets: (NSArray*) anArray
+- (void) closeDataSet: (AdDataSet*) aDataSet
+{
+	//The dataset may only be present in the openDataSets array.
+	//If this is so then it will be deallocated when removed and the
+	//program will crash during removeGnuplotFilesForDataSet:
+	//Hence we retain/autorelease it here to prevent this.
+	[[aDataSet retain] autorelease];
+	if(![openDataSets containsObject: aDataSet])
+		return;
+	
+	[openDataSets removeObject: aDataSet];
+	[self setAvailableDataSets: openDataSets];
+	[self removeGnuplotFilesForDataSet: aDataSet];
+}
+
+- (void) selectDataSet: (AdDataSet*) aDataSet
+{
+	if(![openDataSets containsObject: aDataSet])
+		return;
+	
+	[dataSetList selectItemAtIndex: 
+	 [openDataSets indexOfObject: aDataSet]];
+	[self dataSetDidChange: self];	
+}
+
+- (void) setAvailableDataSets: (NSArray*) array
 {
 	NSEnumerator* arrayEnum;
-	id dataSet;
+	AdDataSet* dataSet;
 	
-	arrayEnum = [anArray objectEnumerator];
+	[dataSetList removeAllItems];
+	arrayEnum = [array objectEnumerator];
 	while(dataSet = [arrayEnum nextObject])
-		[self removeGnuplotFilesForDataSet: dataSet];
-}
-
-/********************
-
-Gnuplot TextView Delegate Methods 
-
-********************/
-
-- (BOOL) textView: (NSTextView*) aTextView doCommandBySelector:(SEL)aSelector
-{
-	NSRange endRange;
-	NSString* string, *errorString;
-	NSData* data;
-
-	if([NSStringFromSelector(aSelector) isEqual: @"insertNewline:"])
-	{
-		commandRange.length = [[aTextView textStorage] length] - commandRange.location;
-		string = [[[aTextView textStorage] attributedSubstringFromRange: commandRange] string];
-		[self _addStringToHistory: string];
-		string = [NSString stringWithFormat: @"%@\n", string];
-		data  = [string dataUsingEncoding: NSASCIIStringEncoding];
-
-		//Only write the data if gnuplot is running 
-		if(gnuplotRunning)
-			[gnuplotOutput writeData: data];
-
-		endRange.location = [[aTextView textStorage] length];
-		endRange.length = 0;
-		[aTextView replaceCharactersInRange:endRange withString:
-			[NSString stringWithFormat: @"\n", string]];
-
-		endRange.location = [[aTextView textStorage] length];
-		gnuplotPrompt.location = endRange.location;
-		endRange.length = 0;
-		[aTextView replaceCharactersInRange:endRange withString: @"gnuplot> "];
-		gnuplotPrompt.length = [[aTextView textStorage] length] - gnuplotPrompt.location;
-		//make sure the cursor appears after the ">"
-		endRange.location = [[aTextView textStorage] length];
-		[aTextView setSelectedRange: endRange];
-		[aTextView scrollRangeToVisible: endRange];
-		commandRange.location = [[aTextView textStorage] length];
-		return YES;
-	}
-	else if([NSStringFromSelector(aSelector) isEqual: @"moveUp:"])
-	{
-		string = [self _previousStringFromHistory];
-		commandRange.length = [[aTextView textStorage] length] - commandRange.location;
-		[aTextView replaceCharactersInRange:commandRange withString: string];
-		return YES;
-	}	
-	else if([NSStringFromSelector(aSelector) isEqual: @"moveDown:"])
-	{
-		string = [self _nextStringFromHistory];
-		commandRange.length = [[aTextView textStorage] length] - commandRange.location;
-		[aTextView replaceCharactersInRange:commandRange withString: string];
-		return YES;
-	}
-
-	return NO;
-}
-
-- (BOOL) textView: (NSTextView*) aTextView
-	shouldChangeTextInRange:  (NSRange) range
-	replacementString: (NSString*) string
-{
-	NSRange intersectionRange;
-	intersectionRange = NSIntersectionRange(range, gnuplotPrompt);
+		[dataSetList addItemWithTitle: [dataSet name]];
 	
-	if(intersectionRange.length == 0)
-		return YES;
+	if([dataSetList numberOfItems] != 0)	
+	{
+		//If the last selected data set is in the new list
+		//keep it selected. Otherwise select the first item.
+		if([openDataSets containsObject: selectedDataSet])
+		{
+			[dataSetList selectItemAtIndex: 
+			 [openDataSets indexOfObject: selectedDataSet]];
+		}		
+		else	
+		{
+			[dataSetList selectItemAtIndex: 0];
+			[self dataSetDidChange: self];
+		}	
+		//Enable the close button
+		[[[toolbar items] objectAtIndex: 3] setEnabled: YES];	
+		//Enable the export buttonn
+		[[[toolbar items] objectAtIndex: 4] setEnabled: YES];
+		
+	}
 	else
-		return NO;
+	{		
+		//Nothing is open so clear the view
+		[dataView setDataSet: nil];
+		[dataView clearDataSet];
+		[selectedDataSet release];
+		selectedDataSet = nil;
+		[[[toolbar items] objectAtIndex: 3] setEnabled: NO];
+		[[[toolbar items] objectAtIndex: 4] setEnabled: NO];	
+	}
+}
+
+//Called when user selects a different data set
+//from the data set list
+- (void) dataSetDidChange: (id) sender
+{
+	NSString* name;
+	NSEnumerator* dataSetEnum;
+	AdDataSet* dataSet;
+	
+	name = [dataSetList titleOfSelectedItem];
+	dataSetEnum = [openDataSets objectEnumerator];
+	while(dataSet = [dataSetEnum nextObject])
+		if([[dataSet name] isEqual: name])
+		{
+			[selectedDataSet release];
+			selectedDataSet = dataSet;
+			[selectedDataSet retain];
+			[dataView setDataSet: selectedDataSet];
+			[dataView displayData];
+			break;
+		}
 }
 
 @end
 
+/**
+Toolbar delegate methods
+*/
 @implementation ULAnalyser (NSToolbarDelegate)
 
 - (NSToolbarItem*)toolbar: (NSToolbar*)toolbar
@@ -1546,7 +1208,7 @@ willBeInsertedIntoToolbar: (BOOL)flag
 
 	if([itemIdentifier isEqual: @"ApplyItem"])
 	{
-		[toolbarItem setLabel: @"Apply"];
+		[toolbarItem setLabel: @"Apply Plugin"];
 		[toolbarItem setImage: applyImage];
 		[toolbarItem setTarget: self];
 		[toolbarItem setAction: @selector(applyCurrentPlugin:)];     
@@ -1570,7 +1232,7 @@ willBeInsertedIntoToolbar: (BOOL)flag
 		[toolbarItem setAction: @selector(reloadSimulation:)];     
 		[toolbarItem setEnabled: NO];
 		[toolbarItem setTag: 2];
-		[toolbarItem setToolTip: @"Update a running simulation"];
+		[toolbarItem setToolTip: @"Reload the data for a running simulation"];
 #if NeXT_Foundation_LIBRARY == 1		
 		//Toolbar autovalidation not implemented on GNUstep (08/2007)
 		[toolbarItem setAutovalidates: NO];
@@ -1591,6 +1253,20 @@ willBeInsertedIntoToolbar: (BOOL)flag
 		[toolbarItem setAutovalidates: NO];
 #endif	
 	}
+	else if([itemIdentifier isEqual: @"ExportTable"])
+	{
+		[toolbarItem setLabel: @"Export Table"];
+		[toolbarItem setImage: exportImage];
+		[toolbarItem setTarget: self];
+		[toolbarItem setAction: @selector(exportDisplayedTable:)]; 
+		[toolbarItem setEnabled: NO];    
+		[toolbarItem setTag: 4];
+		[toolbarItem setToolTip: @"Exports the currently displayed table as a comma separated value file"];
+#if NeXT_Foundation_LIBRARY == 1		
+		//Toolbar autovalidation not implemented on GNUstep (08/2007)
+		[toolbarItem setAutovalidates: NO];
+#endif	
+	}	
 	 
 	return toolbarItem;
 }
@@ -1598,18 +1274,20 @@ willBeInsertedIntoToolbar: (BOOL)flag
 - (NSArray*) toolbarAllowedItemIdentifiers: (NSToolbar*)toolbar
 {
   return [NSArray arrayWithObjects: @"ApplyItem", 
-		  @"SaveItem",
 		  @"ReloadItem",
+		  @"SaveItem",
 		  @"CloseItem",
+		  @"ExportTable",
 		  nil];
 }
 
 - (NSArray*) toolbarDefaultItemIdentifiers: (NSToolbar*)toolbar
 { 
   return [NSArray arrayWithObjects: @"ApplyItem", 
-		  @"SaveItem", 
 		  @"ReloadItem",
+		  @"SaveItem", 
 		  @"CloseItem",
+		  @"ExportTable",
 		  nil];
 }
 
