@@ -150,6 +150,8 @@
 		cut = NO;
 		editedObject = nil;
 		progressPanel = nil;
+		currentObjects = nil;
+		oldObjects = nil;
 	}
 
 	return self;
@@ -159,7 +161,9 @@
 {
 	NSError* error;
 	NSEnumerator* errorEnum, *clientEnum, *schemaEnum;
+	NSEnumerator* columnEnum;
 	NSDictionary* userInfo;
+	NSTableColumn* column;
 	id lastColumn, schema, client, delete;
 
 	databaseInterface = [[ULDatabaseInterface databaseInterface] retain];
@@ -197,8 +201,31 @@
 
 	lastColumn = [browserView tableColumnWithIdentifier: @"Created"]; 
 	[lastColumn setMinWidth: [[lastColumn headerCell] cellSize].width + 1.5];
+	//This stops the cell being dark on Gnustep default theme
+	[(NSTextFieldCell*)[lastColumn headerCell] setDrawsBackground: NO];
 
+	//Font size is too big on Apple - make it smaller
+	//Set the font size of the browser display
+#ifndef GNUSTEP
+	float size;
+	id cell;
+	
+	columnEnum = [[browserView tableColumns] objectEnumerator];
+	while(column = [columnEnum nextObject])
+	{
+		cell = [column dataCell];
+		size = [NSFont systemFontSizeForControlSize: NSSmallControlSize];
+		[cell setFont: [NSFont fontWithName: [[cell font] fontName] size: size]];
+		[[column dataCell] setControlSize: NSSmallControlSize];	
+	}
+	
+	[browserView setRowHeight:
+		[[NSFont fontWithName: [[cell font] fontName] size: size]
+			defaultLineHeightForFont] + 1];
+#endif	
+	[self willReloadData];
 	[browserView reloadData];
+	[self didReloadData];
 
 	//Expand until the schemas
 	[browserView expandUntilLevel: 2];		
@@ -217,6 +244,8 @@
 
 - (void) dealloc
 {
+	[currentObjects release];
+	[oldObjects release];
 	[allowedActions release];
 	[path release];
 	[databaseInterface release];
@@ -252,7 +281,9 @@
 		[progressPanel release];
 		progressPanel = nil;
 	}	
+	[self willReloadData];
 	[browserView reloadData];
+	[self didReloadData];
 }
 
 - (BOOL) validateMenuItem: (NSMenuItem*) menuItem
@@ -304,6 +335,12 @@
 			return NO;
 	}		
 
+	//It the command is exportAs: we have to ask the export controller 
+	//if the export is possible
+	if([NSStringFromSelector([menuItem action]) isEqual: @"exportAs:"])
+		return [[ULExportController sharedExportController] 
+				canExportCurrentPasteboardObjectAs: [menuItem tag]];
+
 	if([allowedActions containsObject: NSStringFromSelector([menuItem action])])
 	{
 		if(itemLevel == 3)
@@ -512,7 +549,9 @@
 	[errors release];
 	[locations release];
 
+	[self willReloadData];
 	[browserView reloadData];
+	[self didReloadData];
 	//update selection
 	if(lastRow >= [browserView numberOfRows])
 		lastRow = [browserView numberOfRows] - 1;
@@ -771,6 +810,15 @@
 	[[ULExportController sharedExportController] export: self];
 }
 
+/**
+\e sender is the menu item that was clicked
+*/
+- (void) exportAs: (id) sender
+{
+	[[ULExportController sharedExportController] 
+		exportCurrentPasteboardObjectAs: [sender tag]];
+}
+
 - (BOOL) _canImportObject: (id) object 
 		toSchema: (NSString*) schemaName 
 		ofClient: (NSString*) clientName 
@@ -837,8 +885,6 @@
 
 		openPanel = [NSOpenPanel openPanel];	
 		[openPanel setTitle: @"Import Data"];
-		[openPanel setDirectory: 
-			[NSHomeDirectory() stringByAppendingPathComponent: @"adun"]];
 		result = [openPanel runModalForTypes: nil];
 		filename = [openPanel filename];
 
@@ -856,8 +902,8 @@
 				matrix = [AdMutableDataMatrix matrixFromStringRepresentation: string];
 				name =  [[filename stringByDeletingPathExtension]
 						lastPathComponent];
-				[matrix setName:  @"Table"];		
-				object = [[AdDataSet alloc] initWithName: name];
+				[matrix setName:  name];		
+				object = [[AdDataSet alloc] initWithName: @"ImportedTable"];
 				[object autorelease];
 				[object addDataMatrix: matrix];
 			}
