@@ -26,12 +26,25 @@
 
 @implementation ULAnalysisManager
 
-- (Class) _loadBundle: (NSString*) pluginName fromDir: (NSString*) dir 
+//Note requires the .bundle extension to be present on Mac.
+- (BOOL) _plugin: (NSString*) pluginName inDirectory: (NSString*) dir
+{
+	NSArray* directoryContents;
+	
+	directoryContents = [[NSFileManager defaultManager] directoryContentsAtPath: dir];
+	return [directoryContents containsObject: pluginName];
+}
+
+- (Class) loadBundle: (NSString*) pluginName fromDir: (NSString*) dir 
 {
 	NSBundle *pluginBundle;
 	Class pluginClass;
 	NSString* path;
 
+#ifndef GNUSTEP
+	pluginName = [pluginName stringByAppendingPathExtension: @"bundle"];
+#endif
+
 	NSDebugLLog(@"ULAnalysisManager", @"Plugin dir is %@. Plugin Name is %@", 
 		dir, 
 		pluginName);
@@ -59,72 +72,32 @@
 
 	return pluginClass;
 }
-
-- (void) _setCurrentPlugin: (NSString*) name
-{
-	Class pluginClass;
-	id holder;
-	
-	//check if this is the current plugin
-
-	if(![name isEqual: currentPluginName])
-	{
-		//get the principal class of the plugin	
-#ifndef GNUSTEP
-		name = [name stringByAppendingPathExtension: @"bundle"];
-#endif		
-		pluginClass = [self _loadBundle: name fromDir: pluginDir];	
-		if(pluginClass == nil)
-			[NSException raise: NSInvalidArgumentException
-				format: @"Cannot load plugin %@ - Doesnt exist",
-				name];
-
-		holder = [pluginClass new];
-		[currentPlugin release];
-		currentPlugin = holder;
-		[currentPluginName release];
-#ifndef GNUSTEP
-		//We need the currentPluginName to be the name without the .bundle
-		//extension. However on mac we have to add the .bundle extension
-		//above since, unlike gnustep, it must be present. Hence we
-		//have to delete it here before assigning name to currentPluginName
-		name = [name stringByDeletingPathExtension];
-#endif		
-		currentPluginName = [name retain];
-
-		if(![currentPlugin conformsToProtocol:@protocol(ULAnalysisPlugin)])
-		{
-			[currentPlugin release];
-			[currentPluginName release];
-			currentPluginName = nil;
-			currentPlugin = nil;
-			NSWarnLog(@"Plugin doesnt not conform to ULAnalysisPlugin protocol");
-			[NSException raise: NSInternalInconsistencyException 
-				format: @"Specified plugins (%@) principal class does not conform to ULAnalysisPlugin protocol", 
-				[pluginClass description]];
-		}
-	}
-}
-
-- (void) _findAvailablePlugins
+ 
+//Adds the plugins in dir to the available plugins list
+//Only adds them if they are not present in a previous dir
+- (NSArray*) _addPluginsInDirectory: (NSString*) dir
 {
 	BOOL isDir;
 	NSFileManager *fileManager = [NSFileManager defaultManager];
 	NSEnumerator *pluginDirEnum, *pluginEnum;
-	NSString *contentObject, *path, *pluginName, *pluginOutputDir;
+	NSString *contentObject, *path, *pluginName, *pluginOutputDir, *resourceFilename;
 	NSDictionary *infoDict;
-	NSMutableDictionary* defaults;
 	NSArray* inputInfo;
 	NSBundle* bundle;
 
 	//Scan the analysis plugins directory and get all the bundle names
-	pluginDirEnum = [[fileManager directoryContentsAtPath: pluginDir]
-				objectEnumerator];
-	availablePlugins = [NSMutableArray new];
-	pluginInfoDict = [NSMutableDictionary new];
+	pluginDirEnum = [[fileManager directoryContentsAtPath: dir]
+			 objectEnumerator];
+
 	while((contentObject = [pluginDirEnum nextObject]))
 	{
-		path = [pluginDir stringByAppendingPathComponent: contentObject];
+		//Only check the object if its not already present
+		if([availablePlugins containsObject: [contentObject stringByDeletingPathExtension]])
+		{
+			continue;
+		}
+		
+		path = [dir stringByAppendingPathComponent: contentObject];
 		[fileManager fileExistsAtPath: path isDirectory: &isDir];
 		if(isDir)
 		{
@@ -135,8 +108,12 @@
 #else
 			//With Cocoa we need to retrieve the gnustep info-dict to access the plugin.
 			//Later will have the option of putting the information in either one
+			//The GNUstep info dict has the same name as the Cocoa but without the -
+			//e.g. SystemAnalysis-Info.plist -> SystemAnalysisInfo.plist
+			resourceFilename = [NSString stringWithFormat: @"Contents/Resources/%@Info.plist",
+						NSStringFromClass([bundle principalClass])];
 			infoDict = [NSDictionary dictionaryWithContentsOfFile:
-					[path stringByAppendingPathComponent: @"Contents/Resources/Info-gnustep.plist"]];
+					[path stringByAppendingPathComponent: resourceFilename]];
 #endif
 			inputInfo = [infoDict objectForKey: @"ULAnalysisPluginInputInformation"];
 			if(infoDict == nil)
@@ -146,7 +123,7 @@
 			else if(inputInfo == nil)
 			{
 				NSWarnLog(@"%@ plugin Info.plist contains no input object information"
-					,contentObject);
+					  ,contentObject);
 			}
 			else
 			{
@@ -155,19 +132,36 @@
 				//extension which we have to remove. 
 				contentObject = [contentObject stringByDeletingPathExtension];
 #endif				
+				
 				[availablePlugins addObject: contentObject];
-				[pluginInfoDict setObject: inputInfo
-					forKey: contentObject];
+				[pluginInfoDict setObject: inputInfo forKey: contentObject];
 			}		
 		}	
 	}		
+	
+}
+
+- (void) _findAvailablePlugins
+{
+	NSEnumerator *pluginDirEnum, *pluginEnum;
+	NSString *pluginName, *pluginOutputDir, *directory;
+	NSMutableDictionary* defaults;
+
+	availablePlugins = [NSMutableArray new];
+	pluginInfoDict = [NSMutableDictionary new];
+	pluginDirEnum = [pluginDirs objectEnumerator];
+	while((directory = [pluginDirEnum nextObject]))
+	{
+		NSDebugLLog(@"ULAnalysisManager", @"Searching directory %@", directory);
+		[self _addPluginsInDirectory: directory];
+	}		
 
 	//For each plugin set the default plugin output directory as the
 	//location of their output directory default.
 	pluginOutputDir = [[ULIOManager appIOManager] defaultPluginOutputDir];
 	defaults = [NSMutableDictionary dictionary];
 	pluginEnum = [availablePlugins objectEnumerator];
-	while(pluginName = [pluginEnum nextObject])
+	while((pluginName = [pluginEnum nextObject]))
 		[defaults setObject: pluginOutputDir
 			forKey: [NSString stringWithFormat: @"%@OutputDir", pluginName]];
 			
@@ -177,28 +171,48 @@
 	NSDebugLLog(@"ULAnalysisManager", @"Plugin information %@", pluginInfoDict);
 }
 
-+ (id) managerWithDefaultLocation
++ (void) initialize
+{
+	[[NSUserDefaults standardUserDefaults] 
+		registerDefaults: [NSDictionary dictionaryWithObject: [NSArray array] 
+					forKey: @"PluginDirectories"]];
+}
+
++ (id) managerWithDefaultLocations
 {	
 	return [[ULAnalysisManager new] autorelease];
 }
 
 - (id) init
 {
-	NSString* location;
+	NSString* mainDir, *builtinDir;
+	NSMutableArray* directories;
 
-	location = [[[ULIOManager appIOManager] applicationDir]
+	mainDir = [[[ULIOManager appIOManager] applicationDir]
 				stringByAppendingPathComponent: @"Plugins/Analysis"];
-	return [self initWithLocation: location];
+	builtinDir =  [[[NSBundle mainBundle] builtInPlugInsPath] 
+			stringByAppendingPathComponent: @"Analysis"];			
+	directories = [[[NSUserDefaults standardUserDefaults] arrayForKey: @"PluginDirectories"] mutableCopy];
+	directories = [directories autorelease];
+	[directories insertObject: builtinDir atIndex: 0];
+	[directories insertObject: mainDir atIndex: 1];
+	
+	return [self initWithLocations: directories];
 }
 
 - (id) initWithLocation: (NSString*) aString
+{
+	return [self initWithLocations: [NSArray arrayWithObject: aString]];
+}
+
+- (id) initWithLocations: (NSArray*) anArray
 {
 	if((self = [super init]))
 	{
 		currentPluginName = nil;
 		results = nil;
-		pluginDir = aString;
-		[pluginDir retain];
+		pluginDirs = anArray;
+		[pluginDirs retain];
 		inputObjects = [NSMutableArray new];
 		objectsCountDict = [NSMutableDictionary new];
 		outputObjectsReferences = [NSMutableArray new];
@@ -217,7 +231,7 @@
 	[outputObjectsReferences release];
 	[availablePlugins release];
 	[pluginInfoDict release];
-	[pluginDir release];
+	[pluginDirs release];
 	[inputObjects release];
 	[objectsCountDict release];
 	[currentPluginName release];
@@ -236,7 +250,7 @@ unless clearOutputs are called
 	NSArray* dataSets;
 	NSError* internalError;
 	
-	[self _setCurrentPlugin: name];
+	[self setCurrentPlugin: name];
 	[results release];
 	[outputObjectsReferences removeAllObjects];
 	
@@ -346,7 +360,7 @@ unless clearOutputs are called
 
 - (id) optionsForPlugin: (NSString*) name
 {
-	[self _setCurrentPlugin: name];	
+	[self setCurrentPlugin: name];	
 	return [currentPlugin pluginOptions: inputObjects];
 }
 
@@ -550,14 +564,83 @@ unless clearOutputs are called
 	return [results objectForKey: @"ULAnalysisPluginFiles"];
 }
 
+- (NSArray*) availablePlugins
+{
+	return [[availablePlugins copy] autorelease];
+}
+
+- (NSString*) locationOfPlugin: (NSString*) pluginName
+{
+	NSEnumerator* pluginDirEnum;
+	NSString* directory, *result = nil;
+	
+#ifndef GNUSTEP
+	pluginName = [pluginName stringByAppendingPathExtension: @"bundle"];
+#endif	
+	pluginDirEnum = [pluginDirs objectEnumerator];
+	while(directory = [pluginDirEnum nextObject])
+	{
+		if([self _plugin: pluginName inDirectory: directory])
+		{
+			NSDebugLLog(@"ULAnalysisManager", @"Found %@ in %@", pluginName, directory);
+			result = directory;
+			break;
+		}
+	}
+	
+	return result;
+}
+
 - (id) currentPlugin
 {
 	return [[currentPlugin retain] autorelease];
 }
 
-- (void) loadPlugin: (NSString*) pluginName
+- (void) setCurrentPlugin: (NSString*) name
 {
-	[self _setCurrentPlugin: pluginName];
+	Class pluginClass;
+	NSString* directory;
+	id holder;
+	
+	//check if this is the current plugin
+	
+	if(![name isEqual: currentPluginName])
+	{
+		//get the principal class of the plugin		
+		directory = [self locationOfPlugin: name];
+		pluginClass = [self loadBundle: name fromDir: directory];	
+		if(pluginClass == nil)
+			[NSException raise: NSInvalidArgumentException
+				    format: @"Cannot load plugin %@ - Doesnt exist",
+			 name];
+		
+		holder = [pluginClass new];
+		[currentPlugin release];
+		currentPlugin = holder;
+		[currentPluginName release];
+	
+		currentPluginName = [name retain];
+		
+		if(![currentPlugin conformsToProtocol:@protocol(ULAnalysisPlugin)])
+		{
+			[currentPlugin release];
+			[currentPluginName release];
+			currentPluginName = nil;
+			currentPlugin = nil;
+			NSWarnLog(@"Plugin doesnt not conform to ULAnalysisPlugin protocol");
+			[NSException raise: NSInternalInconsistencyException 
+				    format: @"Specified plugins (%@) principal class does not conform to ULAnalysisPlugin protocol", 
+			 [pluginClass description]];
+		}
+	}
+}
+
+- (Class) loadPlugin: (NSString*) name
+{
+	NSString* directory;
+			
+	directory = [self locationOfPlugin: name];
+	return [self loadBundle: name fromDir: directory];		
 }
 
 @end
