@@ -374,7 +374,7 @@ static id databaseInterface;
 		
 		//find a client which allows writing to the objects schema
 		clientEnum = [clients objectEnumerator];
-		while(client = [clientEnum nextObject])
+		while((client = [clientEnum nextObject]))
 		{
 			backend = [self backendForClient: client];
 			mode = [backend modeForSchema: [object valueForMetadataKey: @"Schema"]];
@@ -439,14 +439,30 @@ static id databaseInterface;
 		error: error];
 }
 
-- (NSArray*) availableObjectsOfClass: (NSString*) className
+- (NSArray*) metadataForObjectsOfClass: (NSString*) className
 		inSchema: (NSString*) schema
 		ofClient: (NSString*) clientName
 {
 	id retval;
 
 	backend = [self backendForClient: clientName];
-	retval = [backend availableObjectsOfClass: className inSchema: schema];
+	retval = [backend metadataForObjectsOfClass: className inSchema: schema];
+	
+	return retval;
+}
+
+- (NSDictionary*) metadataForObjectAtIndex: (unsigned int) objectIndex
+		ofClass: (NSString*) className
+		inSchema: (NSString*) schema
+		ofClient: (NSString*) clientName
+{
+	id retval;
+	
+	backend = [self backendForClient: clientName];
+	retval = [backend metadataForObjectAtIndex: objectIndex
+			ofClass: className 
+			inSchema: schema];
+			
 	return retval;
 }
 
