@@ -267,6 +267,10 @@ static NSDictionary* classMap;
 		[unitsForInteraction setObject: array forKey: @"HarmonicImproperTorsion"];
 		array = [NSArray arrayWithObjects: @"KCalMol", @"Angstrom", nil];
 		[unitsForInteraction setObject: array forKey: @"TypeTwoVDWInteraction"];
+		array = [NSArray arrayWithObjects: @"KCalMol", @"Angstrom", nil];
+		[unitsForInteraction setObject: array forKey: @"1-4Interaction"];
+		array = [NSArray arrayWithObjects: @"KCalMol", @"Angstrom", nil];
+		[unitsForInteraction setObject: array forKey: @"UreyBradley"];
 		array = [NSArray arrayWithObjects: @"AMU", nil];
 		[unitsForInteraction setObject: array forKey: @"Mass"];
 
@@ -281,6 +285,10 @@ static NSDictionary* classMap;
 		[parametersForInteraction setObject: array forKey: @"HarmonicImproperTorsion"];
 		array = [NSArray arrayWithObjects: @"VDW WellDepth", @"VDW Separation", nil];
 		[parametersForInteraction setObject: array forKey: @"TypeTwoVDWInteraction"];
+		array = [NSArray arrayWithObjects: @"1-4 VDW WellDepth", @"1-4 VDW Separation", nil];
+		[parametersForInteraction setObject: array forKey: @"1-4Interaction"];
+		array = [NSArray arrayWithObjects: @"Constant", @"Separation", nil];
+		[parametersForInteraction setObject: array forKey: @"UreyBradley"];
 		array = [NSArray arrayWithObjects: @"Mass", nil];
 		[parametersForInteraction setObject: array forKey: @"Mass"];
 	}
@@ -377,6 +385,100 @@ Finding parameters
 
 ************************/
 
+//Necessary for improper torsion assignment
+//Check that the trigonal atom matches the begining or end of the tag
+//If trigonal atom matches, look for all possible combinations of atoms in the tag to match
+//return the idices of the matching atoms in the "correct" order (parameter order)
+//if no match retuns NULL
+- ( NSArray * ) _matchImproperTorsion: (NSArray*) idStrings 
+			 toParamLabel: (NSString*) idString fromRow: (id) row
+{
+	int i, number, index,trigonal;
+	NSNumber *trigonalIndex;
+	NSMutableArray *atomsTopo, *atomsParam;
+	NSMutableDictionary *atomsTopoDic, *atomsParamDic;
+	NSMutableArray *rowOut, *rowIn;
+	id iObject;
+	
+	atomsParam = [NSMutableArray arrayWithCapacity:1]; 
+	atomsTopo  = [NSMutableArray arrayWithCapacity:1];
+	
+	[atomsTopo addObjectsFromArray: 
+         [idString componentsSeparatedByString: @" "]];
+	
+	[atomsParam addObjectsFromArray: 
+         [[idStrings objectAtIndex: 0] componentsSeparatedByString: @" "]];
+	
+	//Dont do anything if parameters use only one atom
+	if ( [atomsParam count ] < 4 )
+		return NULL;
+	
+	//Check to see if the trigonal atom matches and remove it from the list
+	//Charmm parameters have the trigonal atom at the begining or end
+	
+	if (![[atomsParam objectAtIndex: 0] isEqual: [atomsTopo objectAtIndex: 2]])
+		if (![[atomsParam objectAtIndex: 3] isEqual: [atomsTopo objectAtIndex: 2]])
+			return NULL;
+	
+	if ([[atomsParam objectAtIndex: 0] isEqual: [atomsTopo objectAtIndex: 2]] )
+		trigonal = 0;
+	else
+		trigonal = 3;
+		
+	[atomsTopo removeObjectAtIndex: 2];
+	[atomsParam removeObjectAtIndex: trigonal];
+	
+	rowIn =  [NSMutableArray arrayWithCapacity:1];
+	[rowIn addObjectsFromArray: row];
+	trigonalIndex = [ rowIn objectAtIndex: 2 ];
+	[rowIn removeObjectAtIndex: 2 ];
+	
+	//check all permutations of the remaining 3 atom names
+	atomsTopoDic = [NSMutableDictionary dictionaryWithCapacity:1];
+	atomsParamDic = [NSMutableDictionary dictionaryWithCapacity:1];
+	for( i=0;i<3;i++)
+	{
+		[atomsTopoDic setObject: [NSNumber numberWithInt: 0] 
+				 forKey: [atomsTopo objectAtIndex: i]];
+		[atomsParamDic setObject:[NSNumber numberWithInt: 0 ] 
+				  forKey: [atomsParam objectAtIndex: i]];
+	}
+	
+	for( i=0;i<3;i++)
+	{  
+		number =  [[atomsTopoDic objectForKey: [atomsTopo objectAtIndex: i]]
+			   intValue ];
+		number++;
+		[atomsTopoDic setObject: [NSNumber numberWithInt: number] 
+				 forKey: [atomsTopo objectAtIndex: i]];
+		number =  [[atomsParamDic objectForKey: [atomsParam objectAtIndex: i]]
+			   intValue];
+		number++;
+		[atomsParamDic setObject:[NSNumber numberWithInt: number] 
+				  forKey: [atomsParam objectAtIndex: i]];
+	}
+	
+	// if its a match place the atoms in the correct order (parameter order)
+	if ( [ atomsTopoDic isEqualToDictionary: atomsParamDic] )
+	{
+		rowOut = [NSMutableArray arrayWithCapacity:4 ];
+		for(i=0;i<3;i++)
+		{
+			iObject = [atomsParam objectAtIndex: i ];
+			index = [atomsTopo indexOfObject: iObject ];
+			[atomsTopo removeObjectAtIndex: index ];
+			[rowOut addObject: [rowIn objectAtIndex: index ]];
+			[rowIn removeObjectAtIndex: index ];
+		}
+		[rowOut insertObject: trigonalIndex atIndex: trigonal];
+		return rowOut;
+	}
+	else
+	{
+		return NULL;
+	}
+}
+
 /**
 This is a workaround until FFML is fully developed.
 We know we are working with Enzymix so we can make
@@ -387,24 +489,24 @@ note: Improper Torsions that have no parameters are removed
 - (void) _findParametersForInteractions: (NSDictionary*) topology ofAtoms: (NSMutableArray*) atomList
 {
 	int rowCount;
-	id class, interaction, row, idString, idStrings, atom;
+	id class, interaction, row, trow, atom;
 	NSEnumerator *interactionEnum, *rowEnum, *atomEnum;	
 	NSDictionary* topologyClassNodes;
 	NSMutableArray* atomArray, *newRow;
-	NSString* interactionType;
+	NSString* interactionType, *idString;
 	NSMutableString* failString, *missingParams;
 	NSMutableIndexSet* unknownInteractions;
 	AdMutableDataMatrix* newMatrix;	//The matrix with parameters
-	NSArray* convertedParameters, *interactionUnits;
-
+	NSArray* convertedParameters, *interactionUnits, *idStrings;
+	
 	//get the correct topology information
-
+	
 	topologyClassNodes = [parameterLibrary topologiesForClass: @"generic"];
-
+	
 	NSDebugLLog(@"ULInteractionsBuilder", 
-			@"Finding parameters for %@", 
-			[topology valueForKey:@"InteractionType"]);
-
+		    @"Finding parameters for %@", 
+		    [topology valueForKey:@"InteractionType"]);
+	
 	rowEnum = [[topology valueForKey:@"Matrix"] rowEnumerator];
 	class = [topologyClassNodes valueForKey:[topology valueForKey:@"InteractionType"]];
 	atomArray  = [NSMutableArray arrayWithCapacity:1];
@@ -423,52 +525,74 @@ note: Improper Torsions that have no parameters are removed
 		//Hackity hack hack hack!
 		//We also have to set an array explaining the units of 
 		//the parameters to be returned.
-
-		if([interactionType isEqual:@"FourierTorsion"])
+		
+		if([interactionType isEqual:@"FourierTorsion"] && 
+		   [forceField isEqual:@"Enzymix"] )
 		{
 			[atomArray addObject: 
-				[atomList objectAtIndex: [[row objectAtIndex:1] intValue]]];
+			 [atomList objectAtIndex: [[row objectAtIndex:1] intValue]]];
 			[atomArray addObject: 
-				[atomList objectAtIndex: [[row objectAtIndex:2] intValue]]];
+			 [atomList objectAtIndex: [[row objectAtIndex:2] intValue]]];
 		}
-		else if([interactionType isEqual:@"HarmonicImproperTorsion"])
+		else if([interactionType isEqual:@"HarmonicImproperTorsion"] && 
+			[forceField isEqual:@"Enzymix"])
 		{
 			[atomArray addObject: 
-				[atomList objectAtIndex: [[row objectAtIndex:2] intValue]]];
+			 [atomList objectAtIndex: [[row objectAtIndex:2] intValue]]];
 		}
 		else
 		{
 			atomEnum = [row objectEnumerator];
 			while((atom = [atomEnum nextObject]))
 				[atomArray addObject:
-					 [atomList objectAtIndex: [atom intValue]]];
+				 [atomList objectAtIndex: [atom intValue]]];
 		}
-
-		idString = [atomArray componentsJoinedByString:@""];
+		
+		idString = [atomArray componentsJoinedByString:@" "];
 		interactionEnum = [[class children] objectEnumerator];
 		while((interaction = [interactionEnum nextObject]))
 		{
 			idStrings = [interaction idStringsForInteraction];
+			if ( [interactionType isEqual:@"HarmonicImproperTorsion"] )
+			{ 
+				// find if a improper torsion matches
+				// Enzymix will just return NULL
+				trow = [self _matchImproperTorsion: idStrings
+						      toParamLabel: idString
+							   fromRow: row ];
+				// if trow is not NULL it means we found parameters therefore
+				// place atoms in the right order and force a parameter match
+				if ( trow != NULL)
+				{
+					row = trow;
+					idString = [ idStrings objectAtIndex: 0 ];
+				}
+			}
+			
 			if([idStrings containsObject: idString])
 			{
+				[newRow removeAllObjects];
 				[newRow addObjectsFromArray: [interaction parameters]];
 				[newRow addObjectsFromArray: [interaction constraints]];
-
+				
 				//FFML 1.0 should return an array of paramters
 				//Plus their names and their units. For now we have
 				//to hack this part. We know were dealing with
 				//Enyzmix so we could set the conversions above.
-
+				
 				convertedParameters = [self convertParameters: newRow 
-							withUnits: interactionUnits];
+								    withUnits: interactionUnits];
 				[newRow removeAllObjects];			
 				[newRow addObjectsFromArray: row];
 				[newRow addObjectsFromArray: convertedParameters];
 				[newMatrix extendMatrixWithRow: newRow];
-				break;
+				//Search for multiple matching lines for torsions only
+				if (! ( [interactionType isEqual:@"FourierTorsion"] ||
+				       [interactionType isEqual:@"HarmonicImproperTorsion"] ) )
+					break;
 			}
 		}
-
+		
 		if([newRow count] == 0)
 		{
 			[missingParams appendString: @"("];
@@ -482,30 +606,40 @@ note: Improper Torsions that have no parameters are removed
 		[atomArray removeAllObjects];
 		rowCount++;
 	}
-
-	if([missingParams length] > 0)
+	
+	// hack: 1-4 interactions and UB terms cannot have missing parameters
+	if([missingParams length] > 0 && (![interactionType isEqual:  @"1-4Interaction"]  &&
+					  ![interactionType isEqual:  @"UreyBradley"] ))
 	{
-		[failString appendFormat: @"\nUnable to find parameters for the following %@ interactions.\n",
+		[failString appendFormat:
+			@"\nUnable to find parameters for the following %@ interactions.\n",
 			interactionType];
 		[failString appendString: missingParams];
 		
-		if([interactionType isEqual: @"Bond"] || [interactionType isEqual: @"Angle"])
+		if([interactionType isEqual: @"HarmonicBond"] || [interactionType isEqual: @"HarmonicAngle"])
 		{
+			GSPrintf(buildOutput, @"%@\n", failString);
+			GSPrintf(buildOutput, @"Aborting build\n");
 			[[NSException exceptionWithName: @"ULBuildException"
-				reason: [NSString stringWithFormat: @"Missing %@ parameters.", interactionType]
-				userInfo: [NSDictionary dictionaryWithObject: failString
-						forKey: @"ULBuildExceptionDetailedDescriptionKey"]] 
-				raise];
+						 reason: [NSString stringWithFormat: @"Missing %@ parameters.", interactionType]
+					       userInfo: [NSDictionary dictionaryWithObject: failString
+								forKey: @"ULBuildExceptionDetailedDescriptionKey"]] 
+			 raise];
+		}
+		else if ([interactionType isEqual: @"HarmonicImproperTorsion"]) 
+		{
+			GSPrintf(buildOutput, @"\nIgnore missing improper torsions for aromatic groups\n");
+			GSPrintf(buildOutput, @"%@",failString);
 		}
 		else	
 		{
 			[errorString appendString: failString];
 			[buildString appendFormat: @"\t\tRemoved %d %@ interactions.\nSee errors.\n", 
-				[unknownInteractions count],
-				interactionType];
+			 [unknownInteractions count],
+			 interactionType];
 		}
 	}
-
+	
 	//set the new matrix
 	[topology setValue: newMatrix forKey: @"Matrix"]; 
 }
@@ -651,7 +785,6 @@ Interaction Building
 				}
 		}
 	}
-	
 	interaction = [NSMutableDictionary dictionaryWithCapacity: 1];
 	[interaction setValue: @"FourierTorsion" forKey: @"InteractionType"];
 	[interaction setValue: [NSNumber numberWithInt: 4] forKey: @"ElementsPerInteraction"];
@@ -668,6 +801,7 @@ Improper Torsions Note:
 
 The itor interactions have the center as the third atom.
 The other atoms are in alphabetical order (AMBER convention applied to Enzymix).
+Charmm puts the trigonal atom first or last
 If we dont find parameters for the itor then it is removed
 */
 
@@ -705,7 +839,7 @@ If we dont find parameters for the itor then it is removed
 			[itor sortUsingFunction: sortIndexesByNames 
 				context: (void*)atomNames];
 			[itor insertObject: [NSNumber numberWithInt: i]
-				atIndex: 2];
+				   atIndex: 2];
 			[improperMatrix extendMatrixWithRow: itor]; 
 		}
 	}
@@ -757,6 +891,181 @@ If we dont find parameters for the itor then it is removed
 	return interaction;
 }
 
+- (id) _build14ForAtoms: (NSMutableArray*) atomNames 
+		withBondedInteractions: (NSMutableArray*) bondedInteractions 
+		withVDWAtoms: (NSMutableArray*) vdwAtomsList
+{
+	int i,index;
+	int noAtoms,noTorsions,noList14;
+	int firstatm,lastatm, noInteraction14;
+	double eps1,eps2,rmin1,rmin2;
+	NSMutableArray* list14; 
+	AdMutableDataMatrix* vdw14Matrix, *matrix14;
+	NSMutableArray* row;
+	NSArray* matrixRow;
+	NSMutableDictionary* interaction14, *params14, *params14Scaled;
+	NSMutableIndexSet* indexes;
+	NSNumber* atom1,* atom2;
+	NSMutableDictionary* interaction;
+	id torsions, itors,torMatrix;
+	
+	noAtoms = [atomNames count];
+	vdw14Matrix = [AdMutableDataMatrix new];
+	matrix14 = [AdMutableDataMatrix new];
+	[matrix14 autorelease];
+	
+	row = [NSMutableArray array];
+	for(i=0; i<noAtoms; i++)
+	{
+		[row addObject: [NSNumber numberWithInt: i]];
+		[vdw14Matrix extendMatrixWithRow: row];
+		[row removeAllObjects];
+	}
+	
+	interaction14 = [NSMutableDictionary dictionaryWithCapacity: 1];
+	[interaction14 setValue: @"1-4Interaction" forKey: @"InteractionType"];
+	[interaction14 setValue: vdw14Matrix forKey: @"Matrix"];
+	[interaction14 setValue: [NSNumber numberWithInt: 1] forKey: @"ElementsPerInteraction"];
+	[self _findParametersForInteractions: interaction14 ofAtoms: atomNames];   
+	noInteraction14 =[ [interaction14 valueForKey:@"Matrix"] numberOfRows ];
+	params14Scaled = [NSMutableDictionary dictionaryWithCapacity: noInteraction14  ];
+	
+	//create a dictionary with the scaled 1-4 VDW parameters
+	for(i=0; i<noInteraction14; i++)
+	{
+		matrixRow = [ [interaction14 valueForKey:@"Matrix"] row: i];
+		[ params14Scaled setObject: matrixRow forKey: [ matrixRow objectAtIndex: 0 ] ];
+	}
+	
+	//create a dictionary with the std. VDW parameters
+	noInteraction14 =[ [vdwAtomsList valueForKey:@"Matrix"] numberOfRows ];
+	params14 = [NSMutableDictionary dictionaryWithCapacity: noInteraction14  ];
+	for(i=0; i<noInteraction14; i++)
+	{
+		matrixRow = [ [vdwAtomsList valueForKey:@"Matrix"] row: i];
+		[ params14 setObject: matrixRow forKey: [ matrixRow objectAtIndex: 0 ] ];
+	}
+	
+	//create a list of 1-4 interactions using the torsion list
+	list14 = [NSMutableArray arrayWithCapacity: 1];
+	for(i=0; i<noAtoms-1; i++)
+	{
+		indexes = [NSMutableIndexSet indexSet ];
+		[list14 addObject: indexes];
+	}
+	torsions = [ bondedInteractions valueForKey: @"FourierTorsion" ];
+	torMatrix = [ torsions valueForKey:@"Matrix" ];
+	noTorsions = [ torMatrix numberOfRows ];
+	for(i=0; i<noTorsions; i++)
+	{
+		itors = [torMatrix row: i];
+		firstatm = [[ itors objectAtIndex: 0 ] intValue];
+		lastatm = [[ itors objectAtIndex: 3 ] intValue];
+		if ( firstatm < lastatm )
+			[ [ list14 objectAtIndex: firstatm ] addIndex: lastatm ];
+		else
+			[ [ list14 objectAtIndex: lastatm ] addIndex: firstatm ];
+	}
+	
+	noList14 = 0;
+	for(i=0; i<[list14 count]; i++)
+		noList14 += [[list14 objectAtIndex:i] count];
+	GSPrintf(buildOutput, @"There are %d 1-4 interactions\n", noList14);
+	[buildString appendFormat: @"\t\tThere are %d 1-4 interactions\n", noList14];
+	
+	//build a matrix for the interaction
+	row = [NSMutableArray arrayWithCapacity: 4];
+	for(i=0; i<noAtoms-1; i++)
+	{
+		indexes = [ list14 objectAtIndex: i ];
+		index = [indexes firstIndex];
+		atom1 = [NSNumber numberWithInt: i ];
+		while(index != NSNotFound)
+		{
+			atom2 = [NSNumber numberWithInt: index];
+			matrixRow = [ params14 objectForKey: atom1];
+			eps1  = [ [ matrixRow objectAtIndex: 1 ] doubleValue ];
+			rmin1 = [ [ matrixRow objectAtIndex: 2 ] doubleValue ];
+			matrixRow = [ params14 objectForKey: atom2];
+			eps2  = [ [ matrixRow objectAtIndex: 1 ] doubleValue ];
+			rmin2 = [ [ matrixRow objectAtIndex: 2 ] doubleValue ];
+			
+			if (([ params14Scaled objectForKey: atom1] != nil ) ) 
+			{
+				matrixRow =  [ params14Scaled objectForKey: atom1];
+				eps1  = [ [ matrixRow objectAtIndex: 1 ] doubleValue ];
+				rmin1 = [ [ matrixRow objectAtIndex: 2 ] doubleValue ];
+			}
+			if (([ params14Scaled objectForKey: atom2] != nil ) ) 
+			{
+				matrixRow =  [ params14Scaled objectForKey: atom2] ;
+				eps2  = [ [ matrixRow objectAtIndex: 1 ] doubleValue ];
+				rmin2 = [ [ matrixRow objectAtIndex: 2 ] doubleValue ];
+			}         
+			
+			[row addObject: atom1 ];
+			[row addObject: atom2 ];
+			[row addObject: [NSNumber numberWithDouble: (sqrt(eps1*eps2)) ]];
+			[row addObject: [NSNumber numberWithDouble: (rmin1+rmin2) ]];
+			[matrix14 extendMatrixWithRow: row];
+			[row removeAllObjects];
+			index = [indexes indexGreaterThanIndex: index];
+		}
+	}
+	interaction = [NSMutableDictionary dictionaryWithCapacity: 1];
+	[interaction setValue: @"1-4Interaction" forKey: @"InteractionType"];
+	[interaction setValue: [NSNumber numberWithInt: 2] forKey: @"ElementsPerInteraction"];
+	[interaction setValue: matrix14 forKey: @"Matrix"];
+	
+	[vdw14Matrix release];
+	return interaction;
+}
+
+
+- (id) _buildUreyBradleyForAtoms: (NSMutableArray*) atomNames 
+                 withBondedAtoms: (NSMutableArray*) bondedAtomsList
+{
+	int i, j, atom;
+	NSEnumerator *bondedAtomsListEnum;
+	NSMutableArray *angle;
+	NSMutableDictionary* interaction;
+	AdMutableDataMatrix *ubMatrix;
+	id bondedAtoms; 
+
+	ubMatrix = [[AdMutableDataMatrix alloc] 
+			initWithNumberOfColumns:0 
+			columnHeaders: nil
+			columnDataTypes: nil];
+	[ubMatrix autorelease];		
+	bondedAtomsListEnum = [bondedAtomsList objectEnumerator];
+
+	atom = 0;	
+	while((bondedAtoms = [bondedAtomsListEnum nextObject]))
+	{
+		if([bondedAtoms count] > 1)
+			for(i=0; i<[bondedAtoms count] - 1; i++)
+				for(j=i+1; j < [bondedAtoms count]; j++)	
+				{
+					angle = [NSMutableArray arrayWithCapacity: 3];
+					[angle addObject: [bondedAtoms objectAtIndex: i]];
+					[angle addObject: [NSNumber numberWithInt: atom]];
+					[angle addObject: [bondedAtoms objectAtIndex: j]];
+					[ubMatrix extendMatrixWithRow: angle];
+				}
+		atom++;
+	}
+	interaction = [NSMutableDictionary dictionaryWithCapacity: 1];
+	[interaction setValue: @"UreyBradley" forKey: @"InteractionType"];
+	[interaction setValue: [NSNumber numberWithInt: 3] forKey: @"ElementsPerInteraction"];
+	[interaction setValue: ubMatrix forKey: @"Matrix"];
+	[self _findParametersForInteractions: interaction ofAtoms: atomNames];
+   
+   ubMatrix = [ interaction objectForKey: @"Matrix"];
+	GSPrintf(buildOutput,@"There are %d UreyBradley interactions\n", [ubMatrix numberOfRows]);
+	[buildString appendFormat: @"\t\tThere are %d UreyBradley interactions\n", [ubMatrix numberOfRows]];
+	return interaction;
+}
+
 - (id) _findMassesForAtoms: (NSMutableArray*) atomNames
 {
 	int i;
@@ -1141,7 +1450,7 @@ Public Methods
 	[self _interactionsPerResidue: interaction residueIndexes: residueIndexes];
 	
 	[buildString appendString: @"\tFourierTorsions:\n"];
-	interaction = [self _buildTorsionsForAtoms: libraryNameList 	
+	   interaction = [self _buildTorsionsForAtoms: libraryNameList 	
 			withBondedAtoms: bondedAtoms 
 			bonds: [interactions valueForKeyPath: @"HarmonicBond.Matrix"]];
 	[interactions setObject: interaction forKey: [interaction valueForKey: @"InteractionType"]];
@@ -1173,6 +1482,26 @@ Public Methods
 	[nonbonded setValue: interaction forKey: @"VDWParameters"];
 
 	[topology setValue: nonbonded forKey:@"Nonbonded"];
+   //build the 1-4 VDW and electrostatic if the ff has them:
+   if([ [forceFieldInfo interactions] containsObject: @"1-4Interaction"])
+   {
+      interaction = [self _build14ForAtoms: libraryNameList 
+                     withBondedInteractions: [topology valueForKey:@"Bonded"]
+                     withVDWAtoms: [nonbonded objectForKey: @"VDWParameters"] ];
+      [self _interactionsPerResidue: interaction residueIndexes: residueIndexes];
+      [ [topology objectForKey: @"Bonded" ] setObject: interaction 
+         forKey: @"1-4Interaction" ];
+   }
+   //build the UreyBradley Interactions
+   if([ [forceFieldInfo interactions] containsObject: @"UreyBradley"])
+   {
+      interaction = [self _buildUreyBradleyForAtoms: libraryNameList
+                                    withBondedAtoms: bondedAtoms];
+      [self _interactionsPerResidue: interaction residueIndexes: residueIndexes];
+      [ [topology objectForKey: @"Bonded" ] setObject: interaction 
+         forKey: @"UreyBradley" ];
+   }
+
 
 	//we have to retrieve the masses as if they were an interaction
 	
