@@ -75,9 +75,11 @@ static id processManager;
 	/*
 	 * We receive this when the last simulation run by the server dies OR
 	 * when the server crashes. We must distinguish between these two possibilities.
-	 * 	In the first case simply remove the invalid connection from the connections dict.
+	 * 
+	 * In the first case simply remove the invalid connection from the connections dict.
 	 * When the next simulation is started a new one will be established
-	 *	In the second case we have to remove the connection, set all the processes running on 
+	 *
+	 * In the second case we have to remove the connection, set all the processes running on 
 	 * the host controlled by the server to "Disconnected", set an error and post a notification
 	 * of the servers death.
 	 */
@@ -249,7 +251,7 @@ when the application last closed.
 		array = [NSKeyedUnarchiver unarchiveObjectWithFile: fileName];
 		[fileManager removeFileAtPath: fileName handler: NULL];
 		processEnum = [array objectEnumerator];
-		while(process = [processEnum nextObject])
+		while((process = [processEnum nextObject]))
 		{
 			[notificationCenter
 				addObserver: self
@@ -312,24 +314,20 @@ when the application last closed.
 		connections = [NSMutableDictionary new];
 		automaticSpawn = NO;
 
-		hosts = [[ULIOManager appIOManager] valueForKey:@"AdunHosts"];
+		hosts = [[ULIOManager appIOManager] adunHosts];
 		[hosts retain];
 
 		NSDebugLLog(@"ULProcessManager", @"Available hosts %@", hosts);
 		processManager = self;
 
 		//The standard args for running Adun 
-		//Possibly make this a ULProcess ivar so we can have
-		//per process options
-		standardArgs = [NSMutableArray arrayWithObjects: @"-RunMode", @"Server",
+		//FIXME: Make this a ULProcess ivar so we can have per process options
+		standardArgs = [NSMutableArray arrayWithObjects: 
 					@"-SimulationOutputDir", 
 					[[[ULDatabaseInterface databaseInterface] 
 						primaryFileSystemBackend] simulationDir],
 					@"-ControllerOutputDir", 
 					[[ULIOManager appIOManager] controllerOutputDir],
-					@"-CreateLogFiles", @"YES",
-					@"-ConnectToAdServer", @"YES",
-					@"-RunInteractive", @"YES",
 					nil];
 		[standardArgs retain];
 		[self _restoreProcesses];
@@ -356,7 +354,7 @@ when the application last closed.
 	ULProcess* process;
 	
 	processEnum = [spawnedStack objectEnumerator];
-	while(process = [processEnum nextObject])
+	while((process = [processEnum nextObject]))
 		if(![process hasSentProcessData])
 		{
 			value = NO;
@@ -379,7 +377,7 @@ when the application last closed.
 	//This prevents the server sending any messages to these processes while
 	//the application is closed.
 	spawnedStackEnum = [spawnedStack objectEnumerator];
-	while(process = [spawnedStackEnum nextObject])
+	while((process = [spawnedStackEnum nextObject]))
 	{	
 		proxy = [self _proxyForHost: [process processHost]];
 		//Check the process hasnt finished since we started
@@ -404,6 +402,8 @@ when the application last closed.
 		host: (NSString*) host
 {
 	id process;
+	int index;
+	id ioManager = [ULIOManager appIOManager];
 
 	//add the process to the newStack
 	
@@ -411,17 +411,22 @@ when the application last closed.
 		@"Creating process with objects %@ and template %@", 
 		aDict, 
 		simulationTemplate);
-
+		
 	process = [ULProcess processWithInputData: aDict 
 			simulationTemplate: simulationTemplate
+			additionalArguments: nil
 			host: host];
-	[process setProcessStatus: @"Waiting"];
+
+	//Set controller output dir
+	index = [standardArgs indexOfObject: @"-ControllerOutputDir"];
+	[standardArgs replaceObjectAtIndex: index + 1 
+				withObject: [[ioManager controllerOutputDir]
+					     stringByAppendingPathComponent: [process name]]];	
+	[process addArguments: standardArgs];				     
+	
 	[newStack addObject: process]; 
 	//FIXME: Change when we change simulationTemplate to a real template
-	[process setValue: [simulationTemplate 
-				valueForKeyPath: @"metadata.simulationName"]
-		forMetadataKey: @"Name"];		
-				 
+		
 	[[NSNotificationCenter defaultCenter] 
 		postNotificationName: @"ULDidCreateNewProcessNotification"
 		object: self];
@@ -436,10 +441,8 @@ when the application last closed.
 
 - (void) spawnNewProcess
 {
-	int index;
 	id proxy, process, host, error;
-	id ioManager = [ULIOManager appIOManager];
-
+	
 	NSDebugLLog(@"ULProcessManager", @"Preparing process for launch");
 
 	if([newStack count] == 0)
@@ -454,19 +457,13 @@ when the application last closed.
 	proxy = [self _proxyForHost: host];
 
 	GSPrintf(stdout, @"Starting Sim using proxy %@\n", proxy);
-	NSDebugLLog(@"ULProcessManager", @"Process is %@ , %@", 
+	NSDebugLLog(@"ULProcessManager", @"Process is %@ , %@, %@", 
 		[process inputData], 
-		[process simulationTemplate]);
-	//Set controller output dir
-	//FIXME: Possibly change how args are passed
-	index = [standardArgs indexOfObject: @"-ControllerOutputDir"];
-	[standardArgs replaceObjectAtIndex: index + 1 
-		withObject: [[ioManager controllerOutputDir]
-				stringByAppendingPathComponent: [process name]]];
-
-	error = [proxy startSimulation: process arguments: standardArgs];
-	[process setValue: [NSDate date] forKey: @"started"];
+		[process simulationTemplate],
+		[process arguments]);
 	
+	error = [proxy startSimulation: process];
+		
 	if(error == nil)
 	{
 		[[NSNotificationCenter defaultCenter] 
@@ -476,7 +473,7 @@ when the application last closed.
 			object: process];
 		[newStack removeObject: process];
 		[spawnedStack addObject: process];
-		[process setProcessStatus: @"Running"];
+	
 		[[NSNotificationCenter defaultCenter] 
 			postNotificationName: @"ULDidLaunchProcessNotification"
 			object: self];
@@ -607,7 +604,7 @@ when the application last closed.
 	ULProcess* process;
 	
 	processEnum = [newStack objectEnumerator];
-	while(process = [processEnum nextObject])
+	while((process = [processEnum nextObject]))
 	{
 		if([[process processHost] isEqual: hostname])
 			[array addObject: process];
@@ -639,133 +636,6 @@ when the application last closed.
 	return [[hosts copy] autorelease];
 }
 
-- (void) setArgument: (NSString*) arg withValue: (NSString*) val
-{
-	if(arg != nil)
-		[standardArgs addObject: arg];
-
-	if(val != nil)
-		[standardArgs addObject: val];
-}
-
-- (NSArray*) arguments
-{
-	return [[standardArgs copy] autorelease];
-}
-
-//Basic Commands
-//Refactor these messages to use execute:error:process
-//FIXME: Deprecate
-
-- (BOOL) exportProcess: (ULProcess*) process 
-		toFile: (NSString*) filename 
-		overwrite: (BOOL) flag
-		error: (NSError**) error 
-{
-	NSMutableData* data;
-	NSKeyedArchiver* archiver;
-	BOOL isDir;
-	NSString* dir, *reason;
-	NSMutableDictionary* userInfo;
-	id temp, name, fileManager;
-
-	fileManager = [NSFileManager defaultManager];
-	userInfo = [NSMutableDictionary dictionary];
-	
-	//check file
-	name = 	[[filename lastPathComponent] stringByTrimmingCharactersInSet: 
-						[NSCharacterSet whitespaceCharacterSet]];
-	NSDebugLLog(@"Export", @"Name of file is %@", name);
-	if([name isEqual: @""])
-	{
-		reason = @"Filename invalid - Must contain characters other than whitespace";
-		[userInfo setObject: reason forKey: NSLocalizedDescriptionKey];
-		*error = [NSError errorWithDomain: @"ULErrorDomain"
-				code: 1
-				userInfo: userInfo];
-		return NO;
-	}
-
-	temp = [[filename pathComponents] mutableCopy];
-	[temp removeLastObject];
-	dir = [NSString pathWithComponents: temp];
-
-	NSDebugLLog(@"Export", @"Exporting to dir %@", dir);
-	if(![fileManager fileExistsAtPath: dir isDirectory: &isDir])
-	{
-		reason = [NSString stringWithFormat: @"The specfied directory (%@) does not exist", dir];
-		[userInfo setObject: reason forKey: NSLocalizedDescriptionKey];
-		*error = [NSError errorWithDomain: @"ULErrorDomain"
-				code: 1
-				userInfo: userInfo];
-		return NO;
-	}
-
-	if(![fileManager isWritableFileAtPath: dir])
-	{
-		reason = [NSString stringWithFormat: @"Specified directory (%@) is not writable", dir];
-		[userInfo setObject: reason forKey: NSLocalizedDescriptionKey];
-		*error = [NSError errorWithDomain: @"ULErrorDomain"
-				code: 1
-				userInfo: userInfo];
-		return NO;
-	}
-
-	if(!isDir)
-	{
-		reason = [NSString stringWithFormat: @"%@ is not a directory", dir];
-		[userInfo setObject: reason forKey: NSLocalizedDescriptionKey];
-		*error = [NSError errorWithDomain: @"ULErrorDomain"
-				code: 1
-				userInfo: userInfo];
-		return NO;
-	}
-
-
-	if([fileManager fileExistsAtPath: filename] && flag == NO)
-	{
-		reason = @"File already exists - Really overwrite?";
-		[userInfo setObject: reason forKey: NSLocalizedDescriptionKey];
-		*error = [NSError errorWithDomain: @"ULErrorDomain"
-				code: 10
-				userInfo: userInfo];
-		return NO;
-	}
-	else if([fileManager fileExistsAtPath: filename] && flag == YES)
-	{
-		if(![fileManager isWritableFileAtPath:filename]) 
-		{
-			reason = @"Cannot overwrite file - write protected";
-			[userInfo setObject: reason forKey: NSLocalizedDescriptionKey];
-			*error = [NSError errorWithDomain: @"ULErrorDomain"
-					code: 1
-					userInfo: userInfo];
-			return NO;
-		}
-	}
-
-	data = [NSMutableData new];
-	archiver = [[NSKeyedArchiver alloc] initForWritingWithMutableData: data];
-	[archiver setOutputFormat: NSPropertyListXMLFormat_v1_0];
-	[archiver encodeObject: process forKey: @"process"];
-	[archiver finishEncoding];
-
-	if(![data writeToFile: filename atomically: NO])
-	{
-		reason = @"Unable to export file - Reason unknown";
-		[userInfo setObject: reason forKey: NSLocalizedDescriptionKey];
-		*error = [NSError errorWithDomain: @"ULErrorDomain"
-				code: 1
-				userInfo: userInfo];
-		return NO;
-	}
-
-	[archiver release];
-	[data release];
-
-	return YES;
-}
-
 - (void) haltProcess: (ULProcess*) process
 {
 	id proxy;
