@@ -33,7 +33,7 @@ Controls the status table in the main Adun window.
 \ingroup interface
 */
 
-@interface ULStatusTable : NSObject
+@interface ULStatusTable : NSObject <ULPasteboardDataSource>
 {
 	BOOL isActive;
 	id processManager;
@@ -47,11 +47,6 @@ Controls the status table in the main Adun window.
 - (void) setProcessManager: (id) object;
 - (void) setActive: (BOOL) value;
 - (BOOL) isActive;
-//Pasteboard delegate
-- (NSArray*) availableTypes;
-- (id) objectForType: (NSString*) type;
-- (NSArray*) objectsForType: (NSString*) type;
-- (int) countOfObjectsForType: (NSString*) type;
 @end
 
 #endif
