@@ -23,6 +23,10 @@
 #include <AppKit/AppKit.h>
 #include "ULSystemViewController.h"
 
+@interface ULSystemViewController (PDBDownloadAdditions)
+- (void) downloadPDB: (NSString*) pdbID;
+@end
+
 @implementation ULSystemViewController
 
 /************************
@@ -260,9 +264,7 @@ Initialisation Methods
 	[[logView textStorage] deleteCharactersInRange: 
 		NSMakeRange(0, [[logView textStorage] length])];
 	
-	//we have to make sure nothing is left from a previous
-	//hidden solvent build
-
+	//Clear everything from previous build
 	currentOptions = nil;
 	[self _setDelegateForOptions: currentOptions];
 	[systemController removeMolecule];
@@ -334,6 +336,10 @@ Creating a System
 
 ************************/
 
+/*
+Called when a user loads a pdb.
+Writes out data on the pdb to the text-view of the create sytem window
+*/
 - (void) _outputInfoForPDBFile: (NSString*) filename
 {
 	MTStructure* structure;
@@ -345,7 +351,7 @@ Creating a System
 	//causes MolTalk to create a phantom extra chain (only for the first model however).
 	structure = [MTStructureFactory newStructureFromPDBFile: filename 
 			options: 256 + 16 + 32];
-	analysisManager = [ULAnalysisManager managerWithDefaultLocation];
+	analysisManager = [ULAnalysisManager managerWithDefaultLocations];
 	[analysisManager addInputObject: structure];
 	if([[analysisManager pluginsForCurrentInputs] containsObject: @"PDBInfo"])
 	{
@@ -361,12 +367,80 @@ Creating a System
 	}
 }
 
+- (void) getStructureFile: (id) sender
+{
+	NSString* fieldValue;
+
+	fieldValue = [moleculePathField stringValue];
+	if([fieldValue length] == 4)
+	{
+		[self downloadPDB: fieldValue];
+	}
+	else
+	{
+		[moleculePathField setStringValue: @""];
+		[self showFileBrowser: self];
+	}
+}
+
+- (void) loadStructureFile: (NSString*) fileName
+{
+	NSRange endRange;
+	NSMutableString* failString;
+
+	[[logView textStorage] deleteCharactersInRange: 
+		NSMakeRange(0, [[logView textStorage] length])];
+	
+	endRange = NSMakeRange([[logView string] length], 0);
+	failString = [NSMutableString string];
+	NS_DURING
+	{
+		[systemController setBuildMolecule: fileName];
+		[self _updatePluginList];
+		
+		//Output some information on the pdb
+		[self _outputInfoForPDBFile: fileName];
+		
+		//Show the correct options depending on the selected tab.
+		[currentOptions release];
+		if([[[tabView selectedTabViewItem] label] isEqual: @"Build"])
+			currentOptions = [systemController buildOptions];
+		else
+			currentOptions = [systemController preprocessOptions];
+		
+		[currentOptions retain];
+		[self _setDelegateForOptions: currentOptions];
+	}
+	NS_HANDLER
+	{
+		failString = [NSMutableString stringWithCapacity: 1];
+		[failString appendString: @"Failed to load molecule ...\n"];
+		[failString appendString: [localException reason]];
+		[failString appendString: @"\n"];	
+		
+		if([[localException name] isEqual: @"ULBuildException"])
+		{
+			[failString appendString: [[localException userInfo] 
+					objectForKey:
+				@"ULBuildRecoverySuggestionKey"]];
+			[failString appendString: @"\n"];	
+		}
+		
+		[logView replaceCharactersInRange: endRange withString: failString];
+		NSRunAlertPanel(@"Load Molecule Failed", 
+				[localException reason], 
+				@"Dismiss", 
+				nil, 
+				nil);
+	}
+	NS_ENDHANDLER		
+	
+}
+
 - (void) showFileBrowser: (id) sender
 {
 	id fileBrowser;
 	int result;
-	NSRange endRange;
-	NSMutableString* failString;
 
 	//if there is a current build prevent loading
 	//another until it ends
@@ -388,63 +462,17 @@ Creating a System
 	[fileBrowser setAllowsMultipleSelection: NO];
 	result = [fileBrowser runModalForTypes: allowedFileTypes];
 	if(result == NSOKButton)
-	{
+	{	
 		if([allowedFileTypes containsObject: [[fileBrowser filename] pathExtension]])
 			[moleculePathField setStringValue: [fileBrowser filename]];
 		else
 			NSRunAlertPanel(@"Error", 
-				[NSString stringWithFormat: @"Unknown file type - %@\nSupported types %@", 
-					[[fileBrowser filename] pathExtension], allowedFileTypes],
-				@"Dismiss",
-				nil,
-				nil);
-		
-		[[logView textStorage] deleteCharactersInRange: 
-			NSMakeRange(0, [[logView textStorage] length])];
-
-		endRange = NSMakeRange([[logView string] length], 0);
-		failString = [NSMutableString string];
-		NS_DURING
-		{
-			[systemController setBuildMolecule: [moleculePathField stringValue]];
-			[self _updatePluginList];
-			
-			//Output some information on the pdb
-			[self _outputInfoForPDBFile: [moleculePathField stringValue]];
-						
-			//Show the correct options depending on the selected tab.
-			[currentOptions release];
-			if([[[tabView selectedTabViewItem] label] isEqual: @"Build"])
-				currentOptions = [systemController buildOptions];
-			else
-				currentOptions = [systemController preprocessOptions];
-
-			[currentOptions retain];
-			[self _setDelegateForOptions: currentOptions];
-		}
-		NS_HANDLER
-		{
-			failString = [NSMutableString stringWithCapacity: 1];
-			[failString appendString: @"Failed to load molecule ...\n"];
-			[failString appendString: [localException reason]];
-			[failString appendString: @"\n"];	
-
-			if([[localException name] isEqual: @"ULBuildException"])
-			{
-				[failString appendString: [[localException userInfo] 
-					objectForKey:
-					@"ULBuildRecoverySuggestionKey"]];
-				[failString appendString: @"\n"];	
-			}
-
-			[logView replaceCharactersInRange: endRange withString: failString];
-			NSRunAlertPanel(@"Load Molecule Failed", 
-				[localException reason], 
-				@"Dismiss", 
-				nil, 
-				nil);
-		}
-		NS_ENDHANDLER		
+					[NSString stringWithFormat: @"Unknown file type - %@\nSupported types %@", 
+						[[fileBrowser filename] pathExtension], allowedFileTypes],
+					@"Dismiss",
+					nil,
+					nil);
+		[self loadStructureFile: [fileBrowser filename]];
 	}
 }
 
@@ -488,7 +516,7 @@ Creating a System
 	if([[localException userInfo] objectForKey: @"ULBuildExceptionDetailedDescriptionKey"] != nil)	
 	{
 		[failString appendString: [[localException userInfo] 
-				objectForKey: @"ULBuildRecoverySuggestionKey"]];
+				objectForKey: @"ULBuildExceptionDetailedDescriptionKey"]];
 		[failString appendString: @"\n"];	
 	}
 
@@ -803,3 +831,117 @@ Window Delegate Methods
 }
 
 @end
+
+@implementation ULSystemViewController (PDBDownloadAdditions)
+
+- (void) downloadPDB: (NSString*) pdbID
+{
+	NSError* error;
+
+	//Create a download connection
+	urlDownload = [ULURLDownload downloadForPDB: pdbID];
+	[urlDownload retain];
+	
+	//Register for download end notification
+	[[NSNotificationCenter defaultCenter]
+		addObserver: self 
+		selector: @selector(downloadFinished) 
+		name: ULURLDownloadDidEndNotification
+		object: urlDownload];
+	
+	//Create and run a progress panel
+	progressPanel = [[ULProgressPanel alloc] initWithTitle: @"Downloading" 
+				message:  [NSString stringWithFormat: @"Downloading %@.pdb", pdbID]
+				progressInfo: @"Waiting to connect ..."];
+	
+	[progressPanel updateStatusOnNotification: ULURLDownloadStatusNotification 
+		fromObject: urlDownload];
+	[progressPanel runProgressPanel: NO];
+	
+	//Start the download
+	if(![urlDownload beginDownloadToFile: nil overwrite: YES])
+	{
+		//Handle error
+		[progressPanel endPanel];
+		[progressPanel release];
+		error = [urlDownload downloadError];
+		ULRunErrorPanel(error);
+		[urlDownload release];
+		urlDownload = nil;
+		[moleculePathField setStringValue: @""];
+	}
+	else
+	{
+		//Lock the text field and load button
+		[loadButton setEnabled: NO];
+		[moleculePathField setSelectable: NO];
+	}
+}
+
+- (void) downloadFinished
+{
+	BOOL deleteFile;
+	NSData* data;
+	NSString* filename;
+	NSError* error;
+
+	//Let the panel linger briefly
+	[progressPanel orderFront];
+	sleep(1.0);
+	[progressPanel endPanel];
+	[progressPanel release];
+
+	//Check the result
+	if((error = [urlDownload downloadError]) == nil)
+	{
+		filename = [urlDownload filename];
+		//Check it ends in pdb - otherwise no pdb was downloaded
+		if(![[filename pathExtension] isEqual: @"pdb"])
+		{
+			NSLog(@"Error downloading pdb");
+			NSLog(@"URL %@", [urlDownload URL]);
+			NSLog(@"See the file %@ for what was retrieved", filename);
+			error = AdCreateError(ULFrameworkErrorDomain,
+					10,
+					@"Download Error",
+					@"Retrieved data does not correspond to a pdb structure",
+					@"Check UL.log for more information");
+			ULRunErrorPanel(error);	
+			[moleculePathField setStringValue: @""];	
+		}
+		else
+		{
+			//Load the file then delete it (if asked)
+			[self loadStructureFile: filename];
+			deleteFile = [[NSUserDefaults standardUserDefaults]
+					boolForKey: @"DeleteDownloadedPDBs"];
+			if(deleteFile)		
+				[[NSFileManager defaultManager]
+					removeFileAtPath: filename handler: NULL];
+					
+			//Trick so hitting load will open the file browser 
+			//instead of restarting the same download.
+			[moleculePathField setStringValue: [filename lastPathComponent]];	
+		}
+	}
+	else
+	{
+		ULRunErrorPanel(error);
+		[moleculePathField setStringValue: @""];
+	}
+		
+	//Clean up
+	[[NSNotificationCenter defaultCenter] 
+		removeObserver: self 
+			  name: ULURLDownloadDidEndNotification
+			object: urlDownload];
+	[urlDownload release];
+	urlDownload = nil;
+	[loadButton setEnabled: YES];
+	[moleculePathField setSelectable: YES];	
+	[moleculePathField setEditable: YES];	
+}
+
+@end
+
+
