@@ -80,33 +80,30 @@
 	[defaults setObject: [NSNumber numberWithBool: NO]
 		forKey: @"AutomaticallyStartWaitingProcesses"];
 #ifdef GNUSTEP	
-	[defaults setObject: [NSHomeDirectory() stringByAppendingPathComponent: @"adun/buildOutput"]
-		forKey: @"BuildOutput"];
 	[defaults setObject: [NSHomeDirectory() stringByAppendingPathComponent: @"adun/UL.log"]
 		forKey: @"LogFile"];
 	[defaults setObject: @"gedit" forKey: @"Editor"];
 #else	
-	[defaults setObject: [NSHomeDirectory() stringByAppendingPathComponent: @".adun/buildOutput"]
-		     forKey: @"BuildOutput"];
 	[defaults setObject: [NSHomeDirectory() stringByAppendingPathComponent: @".adun/UL.log"]
 		     forKey: @"LogFile"];
-	//On Mac we install the server package into /usr/local/bin.
-	//However when testing it is installed into $HOME/bin.
-	//This is because when installing from xcode its easier to install locally.
-	//While with PackageMaker the opposite is true.
-	//We set a default to $HOME/bin which we override during the package install process
-	[defaults setObject: [NSHomeDirectory() stringByAppendingPathComponent: @"bin"]
-		forKey: @"AdunServerPath"];
+	//On Mac we install the server package into /usr/local/bin during a real install
+	[defaults setObject: @"/usr/local/bin" forKey: @"AdunServerPath"];
+	//FIXME: Temporary
+	[defaults setObject: [NSNumber numberWithBool: NO]
+		     forKey: @"Setup"];	
 #endif		     	
 	[defaults setObject: [NSNumber numberWithDouble: 300] forKey: @"AutoUpdateInterval"];
 	[defaults setObject: [NSNumber numberWithBool: YES] forKey: @"AutoUpdate"];
+	[defaults setObject: [NSNumber numberWithBool: YES] forKey: @"CreateUnmodifiedPDBStructures"];
 	[defaults setObject: [NSArray array] forKey: @"DebugLevels"];
-	[defaults setObject: @"Enzymix" forKey: @"DefaultForceField"];
 	[defaults setObject: @"/usr/bin/gnuplot" forKey: @"GnuplotPath"];
+	[defaults setObject: [NSNumber numberWithBool: YES] forKey: @"DeleteDownloadedPDBs"];
 	debugLevels = [[NSProcessInfo processInfo] debugSet];
 	[debugLevels addObjectsFromArray: [[NSUserDefaults standardUserDefaults] 
 		objectForKey: @"DebugLevels"]];
 
+	NSDebugLLog(@"ViewController", @"Defaults %@", defaults);
+
 	[[NSUserDefaults standardUserDefaults] registerDefaults:defaults];
 	[[NSUserDefaults standardUserDefaults] synchronize];
 }
@@ -167,6 +164,17 @@
 
 	if((self = [super init]))
 	{
+	
+#ifndef GNUSTEP
+		//Temporary code to ease install on OSX
+		if([[NSUserDefaults standardUserDefaults] boolForKey: @"Setup"])
+		{
+			id ioManager;
+			ioManager = [ULIOManager appIOManager];
+			exit(0);
+		}
+#endif		
+	
 		//redirect output
 
 		logFile = [[NSUserDefaults standardUserDefaults] stringForKey: @"LogFile"];
@@ -191,6 +199,7 @@
 					@"paste:",
 					@"remove:",
 					@"export:",
+					@"exportAs:",
 					@"import:",
 					@"deselectAllRows:",
 					nil];
@@ -335,14 +344,15 @@
 	NSConnection* connection;
 	NSMutableDictionary* userInfo;
 
+	[NSHost setHostCacheEnabled: YES];
 	[simulationCreator closeWindow: self];
 
 	//check now if there is a local instance of AdunServer running
 	connection = [NSConnection connectionWithRegisteredName: @"AdunServer" host: nil];
-	if(connection == nil)
+	/*if(connection == nil)
 		connection = [NSConnection connectionWithRegisteredName: @"AdunServer"
 				host: nil
-				usingNameServer: [NSSocketPortNameServer sharedInstance]];
+				usingNameServer: [NSSocketPortNameServer sharedInstance]];*/
 				
 	//if there is no AdunServer listening locally alert the user and ask if they want one to be started
 	if(connection == nil)
@@ -680,6 +690,10 @@ Menu validation
 	if([action isEqual: @"convert:"])
 		return [converter validateMenuItem: menuItem];
 		
+	if([action isEqual: @"createAttributeFile:"])
+		return [[ULExportController sharedExportController]
+			validateCreateAttribute: self];
+		
 	if([action isEqual: @"viewErrorLog:"])	
 		return [self validateLogDisplay];
 		
@@ -720,6 +734,7 @@ Menu validation
  * Displaying simulation logs.
  */
 
+//Testing Services
 - (void) _displayStringOSX: (NSString*) string
 {
 	NSPasteboard* pasteboard;
@@ -759,6 +774,28 @@ Menu validation
 	NS_ENDHANDLER
 }
 
+- (void) _displayFileOSX: (NSString*) filename
+{
+	NSTask* task;
+	
+	//Use Console via Open on OSX
+	//FIXME: Change to use NSWorkspace
+	NS_DURING
+	{			
+		//Open should be always in the same place.
+		task = [NSTask launchedTaskWithLaunchPath: @"/usr/bin/open" 
+			arguments: [NSArray arrayWithObjects: @"-a", @"Console", filename, nil]];
+	}
+	NS_HANDLER
+	{		
+		NSRunAlertPanel(@"Unable to launch Console", 
+				[NSString stringWithFormat: @"Could not open log %@", filename],
+				@"Dismiss",
+				nil, nil);
+	}
+	NS_ENDHANDLER
+}
+
 - (void) _viewLog: (NSString*) name
 {
 	BOOL isWaiting = NO;
@@ -800,9 +837,11 @@ Menu validation
 	{
 		filename = [[object dataStorage] storagePath];
 		filename = [filename stringByAppendingPathComponent: name];
-		string = [NSString stringWithContentsOfFile: filename];
+		//string = [NSString stringWithContentsOfFile: filename];
+		//FIMXE: Should be able to change this to use NSWorkspace on 
+		//both platforms at some stage.
 #if NeXT_RUNTIME == 1
-		[self _displayStringOSX: string];
+		[self _displayFileOSX: filename];
 #else
 		[self _displayFileLinux: filename];
 #endif	
@@ -850,6 +889,11 @@ Menu validation
 	[activeDelegate export: self];
 }
 
+- (void) exportAs: (id) sender
+{
+	[activeDelegate exportAs: sender];
+}
+
 - (void) import: (id) sender
 {
 	[activeDelegate import: self];
@@ -871,10 +915,7 @@ Delegate to other tools
 	
 	objectType = [[pasteboard availableTypes] objectAtIndex: 0];
 
-	//FIXME: Change
-/*	if([objectType isEqual: @"ULProcess"])
-		[propertiesPanel display: self]; 
-	else*/ if([objectType isEqual: @"ULTemplate"])
+	if([objectType isEqual: @"ULTemplate"])
 		[templateController display: self]; 
 	else
 		[analyser display: self];
@@ -895,6 +936,12 @@ Delegate to other tools
 	[converter convert: self];
 }
 
+- (void) createAttributeFile: (id) sender
+{
+	[[ULExportController sharedExportController] 
+		openAttributeWindow: self];
+}
+
 - (void) addDatabase: (id) sender
 {
 	[databaseManager showAddDatabasePanel: self];
